package org.apache.turbine;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.io.*;
import java.util.*;
import java.sql.*;

// Java Servlet Classes
import javax.servlet.*;
import javax.servlet.http.*;

// Turbine Modules
import org.apache.turbine.modules.*;

//Turbine Scheduler
import org.apache.turbine.services.*;
import org.apache.turbine.services.schedule.*;

// Turbine Utility Classes
import org.apache.turbine.util.*;
import org.apache.turbine.util.db.*;
import org.apache.turbine.om.security.*;
import org.apache.turbine.services.resources.*;
import org.apache.turbine.modules.actions.sessionvalidator.SessionValidator;

// ECS Clases
import org.apache.ecs.*;
import org.apache.ecs.html.*;

/**
 *
 * Turbine is the main servlet for the entire System
 *
 * @author <a href="mailto:jon@latchkey.com">Jon S. Stevens</a>
 * @author <a href="mailto:bmclaugh@algx.net">Brett McLaughlin</a>
 * @author <a href="mailto:greg@shwoop.com">Greg Ritter</a>
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @author <a href="mailto:frank.kim@clearink.com">Frank Y. Kim</a>
 * @version $Id: Turbine.java,v 1.11 2000/09/06 21:36:45 dlr Exp $
 */
public class Turbine extends HttpServlet
{
    /**
     * Whether init succeeded or not.
     */
    private static String initFailureMessage = null;

    /**
     * This is the string that is looked for when getInitParameter is
     * called.
     */
    private static final String INIT_PROPS_KEY = "properties";

    /**
     * Should initialization activities be performed during doGet()
     * execution?
     */
    private boolean firstDoGet;

    /**
     * This init method will load the default resources from a
     * properties file.
     *
     * @param config typical Servlet initialization parameter.
     * @exception ServletException a servlet exception.
     */
    public void init(ServletConfig config)
        throws ServletException
    {
        super.init(config);

        String props = getInitParameter(INIT_PROPS_KEY);

        /*
         * This will attempt to find the location of the properties
         * file from the relative path to the WAR archive (ie:
         * docroot). Since JServ returns null for getRealPath()
         * because it was never implemented correctly, then we know we
         * will not have an issue with using it this way. I don't know
         * if this will break other servlet engines, but it probably
         * shouldn't since WAR files are the future anyways.
         */
        if ( props != null )
        {
            String realPath = getServletContext().getRealPath(props);
            if ( realPath != null )
                props = realPath;
        }

        // Check for errors in locating the props file. if there is
        // one this method will throw an exception and you should look
        // in your log file for more information.
        propertyFileErrorCheck(props);

        // Setup some defaults from the properties file.
        try
        {
            TurbineResourceService.setPropertiesFileName(props);
        }
        catch ( IOException e )
        {
            log( e.toString() );
            e.printStackTrace();
        }

        // Initialize services that need ServletConfig.
        TurbineServices.getInstance().initServices(config);

        // Set a flag that forces sending RunData to the services upon
        // first doGet() invocation.
        firstDoGet = true;

        // START SCHEDULER HERE
        if ( TurbineResources
            .getBoolean("scheduler.enabled", false) )
        {
            TurbineServices.getInstance()
                .getService(ScheduleService.SERVICE_NAME);
        }

        log ("Turbine initialized");
    }

    /**
     * The <code>Servlet</code> destroy method.  Invokes 
     * <code>ServiceBroker</code> tear down method.
     */
    public void destroy()
    {
        // Shut down all Turbine Services.
        TurbineServices.getInstance().shutdownServices();

        log("Turbine done shutting down");
    }

    /**
     * The primary method invoked when the Turbine servlet is executed.
     *
     * @param req Servlet request.
     * @param res Servlet response.
     * @exception IOException a servlet exception.
     * @exception ServletException a servlet exception.
     */
    public void doGet (HttpServletRequest req,
                       HttpServletResponse res)
        throws IOException,
               ServletException
    {
        // Placeholder for the RunData object.
        RunData data = null;
        try
        {
            // Check to make sure that we started up properly.
            if (initFailureMessage != null)
            {
                throw new Exception ( initFailureMessage );
            }

            // Get general RunData here...
            // Perform turbine specific initialization below.
            data = RunDataFactory.getRunData( req, res,
                                              getServletConfig() );

            // If this is the first invocation, perform some
            // initialization.
            if(firstDoGet)
            {
                synchronized(Turbine.class)
                {
                    if(firstDoGet)
                    {
                        // Certain services need RunData to initialize
                        // themselves.
                        TurbineServices.getInstance().initServices(data);
                        // Mark that we're done.
                        firstDoGet = false;
                    }
                }

            }

            // Get the instance of the Session Validator.
            SessionValidator sessionValidator =
                (SessionValidator)ActionLoader
                .getInstance()
                .getInstance(TurbineResources
                             .getString("action.sessionvalidator") );

            // Insist that the client starts a session before access
            // to data is allowed. this is done by redirecting them to
            // the "screen.homepage" page but you could have them go
            // to any page as a starter (ie: the homepage)
            // "data.getResponse()" represents the HTTP servlet
            // response.
            if ( sessionValidator.requiresNewSession(data) &&
                 data.getSession().isNew() )
            {
                DynamicURI duri = new DynamicURI (data, true);
                String homepage =
                    TurbineResources.getString("screen.homepage", null);
                if (homepage != null)
                {
                    duri.setScreen(homepage);
                }
                // Pass on the sent data in pathinfo.
                for (Enumeration e = data.getParameters().keys() ;
                     e.hasMoreElements() ;)
                {
                    String key = (String) e.nextElement();
                    String value =
                        (String) data.getParameters().getString ( key );
                    duri.addPathInfo((String)key, (String)value );
                }

                /*
                  Currently commented out because it caused a bunch of
                  problems for some people. Left in here for
                  reference.

                // Make sure that this URL you are about to redirect
                // to is not the current URL.  If it is and you
                // redirect to it you will get caught in an infinite
                // redirect loop.
                String currentURI = DynamicURI.toString( data );

                if ( duri.toString().equals( currentURI ) )
                {
                    String message = "Infinite redirect detected...";
                    log(message);
                    Log.error(message);
                    throw new Exception(message);
                }

                */

                data.getResponse().sendRedirect( duri.toString() );
                return;
            }

            // Fill in the screen and action variables.
            data.setScreen ( data.getParameters().getString("screen") );
            data.setAction ( data.getParameters().getString("action") );

            // Special case for login, this must happen before the
            // SessionManager is executed in order to allow a user to
            // even login.
            if ( data.hasAction() &&
                data.getAction()
                 .equalsIgnoreCase(TurbineResources
                                   .getString("action.login")) )
            {
                // If a User is logging in, we should refresh the
                // session here.  Invalidating session and starting a
                // new session would seem to be a good method, but I
                // (JDM) could not get this to work well (it always
                // required the user to login twice).  Maybe related
                // to JServ?  If we do not clear out the session, it
                // is possible a new User may accidently (if they
                // login incorrectly) continue on with information
                // associated with the previous User.  Currently the
                // only keys stored in the session are "turbine.user"
                // and "turbine.acl".
                String[] names = data.getSession().getValueNames();
                if (names != null)
                {
                    for (int i=0; i< names.length; i++)
                    {
                        data.getSession().removeValue(names[i]);
                    }
                }
                ActionLoader.getInstance().exec ( data, data.getAction() );
            }

            // This is where the validation of the Session information
            // is performed if the user has not logged in yet, then
            // the screen is set to be Login. This also handles the
            // case of not having a screen defined by also setting the
            // screen to Login. If you want people to go to another
            // screen other than Login, you need to change that within
            // TurbineResources.properties...screen.homepage; or, you
            // can specify your own SessionValidator action.
            ActionLoader.getInstance()
                .exec( data,
                       TurbineResources
                       .getString("action.sessionvalidator") );

            // Put the Access Control List into the RunData object, so
            // it is easily available to modules.  It is also placed
            // into the session for serialization.  Modules can null
            // out the ACL to force it to be rebuilt based on more
            // information.
            ActionLoader.getInstance()
                .exec(data,
                      TurbineResources
                      .getString("action.accesscontroller"));

            // Start the execution phase. DefaultPage will execute the
            // appropriate action as well as get the Layout from the
            // Screen and then execute that. The Layout is then
            // responsible for executing the Navigation and Screen
            // modules.
            //
            // Note that by default, this cannot be overridden from
            // parameters passed in via post/query data. This is for
            // security purposes.  You should really never need more
            // than just the default page.  If you do, add logic to
            // DefaultPage to do what you want.
            PageLoader.getInstance()
                .exec(data,
                      TurbineResources
                      .getString("page.default"));
    
            // If a module has set data.acl = null, remove acl from
            // the session.
            if ( data.acl == null )
            {
                data.getSession()
                    .removeValue(AccessControlList.SESSION_KEY);
            }

            try
            {
                if ( data.isPageSet() == false &&
                     data.isOutSet() == false )
                    throw new Exception ( "Nothing to output" );

                // We are all done! if isPageSet() output that way
                // otherwise, data.getOut() has already been written
                // to the data.getOut().close() happens below in the
                // finally.
                if ( data.isPageSet() && data.isOutSet() == false )
                {
                    // Modules can override this.
                    data.getResponse()
                        .setContentType( data.getContentType() );

                    // Handle the case where a module may want to send
                    // a redirect.
                    if ( ( data.getStatusCode() == 301 ||
                           data.getStatusCode() ==  302 ) &&
                         data.getRedirectURI() != null )
                    {
                        data.getResponse()
                            .sendRedirect ( data.getRedirectURI() );
                    }

                    // Set the status code.
                    data.getResponse().setStatus ( data.getStatusCode() );
                    // Output the Page.
                    data.getPage().output (data.getOut());
                }
            }
            catch ( Exception e )
            {
                // The output stream was probably closed by the client
                // end of things ie: the client clicked the Stop
                // button on the browser, so ignore any errors that
                // result.
            }
        }
        catch ( Exception e )
        {
            handleException(data, res, e);
        }
        catch (Throwable t)
        {
            handleException(data, res, t);
        }
        finally
        {
            // Make sure to close the outputstream when we are done.
            try
            {
                data.getOut().close();
            }
            catch (Exception e)
            {
                // Ignore.
            }
        }
    }

    /**
     * In this application doGet and doPost are the same thing.
     *
     * @param req Servlet request.
     * @param res Servlet response.
     * @exception IOException a servlet exception.
     * @exception ServletException a servlet exception.
     */
    public void doPost (HttpServletRequest req,
                        HttpServletResponse res)
        throws IOException,
               ServletException
    {
        doGet ( req, res );
    }

    /**
     * Return the servlet info.
     *
     * @return a string with the servlet information.
     */
    public String getServletInfo()
    {
        return "Turbine Servlet";
    }

    /**
     * This code is used to make sure that the property file is
     * properly found and initialized. If it isn't, then set
     * initFailureMessage string to the error and throw a
     * ServletException. This code is called from this servlet's
     * init() method.
     *
     * @param props Properties file name.
     * @exception ServletException a servlet exception.
     */
    public void propertyFileErrorCheck(String props)
        throws ServletException
    {
        try
        {
            ErrorResources.init();
            if ( props == null)
            {
                Object params[] = {(Object) INIT_PROPS_KEY};
                initFailureMessage =
                    ErrorResources
                    .getMessage(ErrorResources
                                .PROPERTIES_INIT_PARAMETER_NOT_FOUND,
                                params);
                log(initFailureMessage);
                Log.error (initFailureMessage);
                throw new Exception(initFailureMessage);
            }
            else if(props.length() == 0)
            {
                Object params[] = {(Object) INIT_PROPS_KEY};
                initFailureMessage =
                    ErrorResources
                    .getMessage(ErrorResources
                                .PROPERTIES_INIT_PARAMETER_ZERO_LENGTH_STRING,
                                params);
                log(initFailureMessage);
                Log.error (initFailureMessage);
                throw new Exception(initFailureMessage);
            }
            else if(!(new File(props).exists()))
            {
                Object params[] = {(Object) props, (Object) INIT_PROPS_KEY};
                initFailureMessage =
                    ErrorResources
                    .getMessage(ErrorResources
                                .TURBINE_RESOURCE_FILE_DOES_NOT_EXIST,
                                params);
                log(initFailureMessage);
                Log.error (initFailureMessage);
                throw new Exception(initFailureMessage);
            }
            else if(!(new File(props).canRead()))
            {
                Object params[] = {(Object) props, (Object)
                       System.getProperty("user.name")};
                initFailureMessage =
                    ErrorResources
                    .getMessage(ErrorResources
                                .TURBINE_RESOURCE_FILE_NOT_READABLE,
                                params);
                log(initFailureMessage);
                Log.error (initFailureMessage);
                throw new Exception(initFailureMessage);
            }
        }
        catch ( Exception e )
        {
            throw new ServletException ( e.toString() );
        }
    }

    /**
     * This method is about making sure that we catch and display
     * errors to the screen in one fashion or another. What happens is
     * that it will attempt to show the error using your user defined
     * Error Screen. If that fails, then it will resort to just
     * displaying the error and logging it all over the place
     * including the servlet engine log file, the Turbine log file and
     * on the screen.
     *
     * @param data A Turbine RunData object.
     * @param res Servlet response.
     * @param e The exception to report.
     */
    private void handleException(RunData data,
                                 HttpServletResponse res,
                                 Throwable t)
    {
        String mimeType = "text/plain";
        try
        {
            // This is where we capture all exceptions and show the
            // Error Screen.
            data.stackTrace = StringUtils.stackTrace(t);
            data.stackTraceException = t;

            // setup the screen
            data.setScreen(TurbineResources
                           .getString("screen.error"));

            // do more screen setup for template execution if needed
            if (data.getTemplateInfo() != null)
                data.getTemplateInfo()
                    .setScreenTemplate(TurbineResources
                                       .getString("screen.error"));

            // Make sure to not execute an action.
            data.setAction ("");

            PageLoader.getInstance()
                .exec(data,
                      TurbineResources.getString("page.default"));

            data.getResponse().setContentType( data.getContentType() );
            data.getResponse().setStatus ( data.getStatusCode() );
            data.getOut().print ( data.getPage().toString() );
        }

        // Catch this one because it occurs if some code hasn't been
        // completely re-compiled after a change..
        catch ( java.lang.NoSuchFieldError e )
        {
            try
            {
                data.getResponse().setContentType( mimeType );
                data.getResponse().setStatus ( 200 );
            }
            catch (Exception ignored) {}

            try
            {
                data.getOut().print ("java.lang.NoSuchFieldError: " +
                                     "Please recompile all of your " +
                                     "source code.");
            }
            catch (IOException ignored) {}

            log ( data.stackTrace );
            org.apache.turbine.util.Log.error ( e );
        }
        // Attempt to do *something* at this point...
        catch ( Exception reallyScrewedNow )
        {
            String msg = "Exception: " ;
            if (data != null)
            {
                msg += data.stackTrace;
            }
            else
            {
                msg += t;
            }
            try
            {
                res.setContentType( mimeType );
                res.setStatus ( 200 );
                res.getWriter().print (msg);
            }
            catch (Exception ignored) {}
            org.apache.turbine.util.Log.error ( reallyScrewedNow );
        }
    }
}
