package org.apache.turbine.om;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.io.*;
import java.math.BigDecimal;

/**
 * This class contains attributes and methods that are used by all
 * business objects within the system.
 *
 * @author <a href="mailto:frank.kim@clearink.com">Frank Y. Kim</a>
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @version $Id: BaseObject.java,v 1.3 2000/08/29 15:39:49 gonzalo Exp $
 */
public class BaseObject implements Serializable
{
    public static final int NEW_ID = -1;

    /**
     * The unique id for the object which can be used for persistence.
     */
    private Object id = null;

    /**
     * A flag which can be set to indicate that an object has been
     * modified, since it was last retrieved from the persistence
     * mechanism.
     */
    private boolean modified = false;

    /**
     * This method is here for historical reasons.  It will probably
     * be deprecated sometime soon.
     *
     * @return the object id as an int.
     */
    public int getId()
    {
        int idInt = NEW_ID;
        try
        {
            idInt = Integer.parseInt(id.toString());
        }
        catch (Exception e) {}
        return idInt;
    }

    /**
     * Attempts to return the object id as an int.
     *
     * @return the object id as an int; returns -1 if id was not set
     * or could not be represented as an int.
     */
    public int getIdAsInt()
    {
        int idInt = NEW_ID;
        try
        {
            idInt = Integer.parseInt(id.toString());
        }
        catch (Exception e) {}
        return idInt;
    }

    /**
     * Attempts to return the object id as a long.
     *
     * @return the object id as a long; returns -1 if id was not set
     * or could not be represented as a long.
     */
    public long getIdAsLong()
    {
        long idLong = (long)NEW_ID;
        try
        {
            idLong = Long.parseLong(id.toString());
        }
        catch (Exception e) {}
        return idLong;
    }

    /**
     * Attempts to return the object id as a BigDecimal.
     *
     * @return the object id as a BigDecimal; returns -1 if id was not
     * set or could not be represented as a BigDecimal.
     */
    public BigDecimal getIdAsBigDecimal()
    {
        BigDecimal bd = null;
        try
        {
            bd = new BigDecimal(id.toString());
        }
        catch (Exception e)
        {
            bd = new BigDecimal(NEW_ID);
        }
        return bd;
    }

    /**
     * Gets the object id as a String.
     *
     * @return the object id as a String; returns -1 if id was not
     * set.
     */
    public String getIdAsString()
    {
        return id.toString();
    }

    /**
     * Gets the object id as an Object.
     *
     * @return the object id as an Object; returns null if id was not
     * set.
     */
    public Object getIdAsObject()
    {
        return id;
    }

    /**
     * Returns whether the object has been modified.
     *
     * @return True if the object has been modified.
     */
    public boolean isModified()
    {
        return modified;
    }

    /**
     * Sets the id for the object as an int.
     *
     * @param id The new id for the object.
     */
    public void setId(int id)
    {
        this.id = new Integer(id);
    }

    /**
     * Sets the id for the object as an Object.
     *
     * @param id The new id for the object.
     */
    public void setId(Object id)
    {
        this.id = id;
    }

    /**
     * Sets the modified state for the object.
     *
     * @param m The new modified state for the object.
     */
    public void setModified(boolean m)
    {
        modified = m;
    }

    /**
     * Sets the modified state for the object to be false.
     */
    public void resetModified()
    {
        modified = false;
    }

    /*
      //
      // Deletes the object.
      //
      public void delete()
      {
      }

      //
      // Retrieves an object.
      //
      public BaseObject retrieve()
      {
      }

      //
      // Retrieves a set of objects based on a criteria.
      //
      // @param criteria The given criteria.
      //
      public static Vector retrieve( Criteria criteria )
      {
      }

      //
      // Saves the object.
      //
      public void save()
      {
      }
    */

    /**
     * Compares two Strings, returns true if their values are the
     * same.
     *
     * @param s1 The first string.
     * @param s2 The second string.
     * @return True if the values of both strings are the same.
     */
    public static boolean equals( String s1,
                                  String s2 )
    {
        if (s1 == null)
        {
            return (s2 == null);
        }
        else if (s2 == null)
        {
            // s1 is not null
            return false;
        }
        else
        {
            return s1.equals(s2);
        }
    }
}
