package org.apache.turbine.om.security;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Turbine utility classes
import org.apache.turbine.util.*;
import org.apache.turbine.om.user.*;
import org.apache.turbine.om.user.peer.*;
import org.apache.turbine.services.resources.TurbineResources;

// Java stuff.
import java.io.*;

/**
 * This is a control class that makes it easy to find out if a
 * particular User has a given Permission.  It also determines if a
 * User has a a particular Role.
 *
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @author <a href="mailto:bmclaugh@algx.net">Brett McLaughlin</a>
 * @author <a href="mailto:greg@shwoop.com">Greg Ritter</a>
 * @version $Id: AccessControlList.java,v 1.2 2000/08/29 15:39:51 gonzalo Exp $
 */
public class AccessControlList implements Serializable
{
    public static final String SESSION_KEY = "turbine.acl";

    /** The permissions for this session. */
    private PermissionSet permissionSet = null;
    /** The roles for this User. */
    private RoleSet roleSet = null;

    /**
     * Constructor.
     *
     * @param username The name of the User.
     * @param obj An Object.
     * @exception Exception, a generic exception.
     */
    public AccessControlList( String username,
                              Object obj )
        throws Exception
    {
        AccessControlBuilder acb = getACB();
        roleSet = acb.getRoles(username, obj);
        permissionSet = acb.getPermissions();
    }

    /**
     * Constructor.
     *
     * @param user A User.
     * @param obj An Object.
     * @exception Exception, a generic exception.
     */
    public AccessControlList( User user, Object obj ) throws Exception
    {
        AccessControlBuilder acb = getACB();
        roleSet = acb.getRoles(user, obj);
        permissionSet = acb.getPermissions();
    }

    /**
     * Get an AccessControlBuilder object.
     *
     * @return An AccessControlBuilder.
     * @exception Exception, a generic exception.
     */
    private AccessControlBuilder getACB()
        throws Exception
    {
        String acClassName = TurbineResources.getString("access.control");
        if ( acClassName == null  || acClassName.equalsIgnoreCase("") )
        {
            acClassName = "org.apache.turbine.om.access.DefaultAccessControl";
        }
        return (AccessControlBuilder) Class.forName(acClassName).newInstance();
    }

    /**
     * Return the Permissions associated with this User.
     *
     * @return A PermissionSet.
     */
    public PermissionSet getPermissions()
    {
        return permissionSet;
    }

    /**
     * Return the Roles associated with this User.
     *
     * @return A RoleSet.
     */
    public RoleSet getRoles()
    {
        return roleSet;
    }

    /**
     * Checks to see if the User's PermissionSet contains the given
     * Permission.
     *
     * @param permission A Permission.
     * @return True if the User has the Permission.
     * @exception Exception, a generic exception.
     */
    public boolean hasPermission(Permission permission)
        throws Exception
    {
        if ( permissionSet != null && permission != null)
        {
            if ( permissionSet.contains(permission) )
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Checks to see if the User's PermissionSet contains the given
     * Permission Name.
     *
     * @param permissionName A String with the permission name.
     * @return True if the User has the Permission.
     * @exception Exception, a generic exception.
     */
    public boolean hasPermission(String permissionName)
        throws Exception
    {
        if ( permissionSet != null && permissionName != null)
        {
            if ( permissionSet.contains(permissionName) )
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Checks to see if the User's RoleSet contains the given Role.
     *
     * @param role A Role.
     * @return True if the User has the Role.
     * @exception Exception, a generic exception.
     */
    public boolean hasRole(Role role)
        throws Exception
    {
        if ( roleSet != null  &&  role != null )
        {
            return roleSet.contains(role);
        }
        return false;
    }
    /**
     * Checks to see if the User's RoleSet contains the given Role
     * name.
     *
     * @param roleName A String with the role name.
     * @return True if the User has the Role.
     * @exception Exception, a generic exception.
     */
    public boolean hasRole(String roleName)
        throws Exception
    {
        if ( roleSet != null  &&  roleName != null )
        {
            return roleSet.contains(roleName);
        }
        return false;
    }

    /**
     * Checks to see if the User's RoleSet contains the given Role
     * name.
     *
     * @param roleId An int with the role id.
     * @return True if the User has the Role.
     * @exception Exception, a generic exception.
     */
    public boolean hasRole(int roleId)
        throws Exception
    {
        if ( roleSet != null )
            {
            return roleSet.contains(roleId);
        }
        return false;
    }
}
