package org.apache.turbine.om.security;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.*;

// Turbine utility classes
import org.apache.turbine.om.user.*;
import org.apache.turbine.om.user.peer.*;
import org.apache.turbine.om.security.peer.*;
import org.apache.turbine.util.*;
import org.apache.turbine.util.db.*;
import org.apache.turbine.services.resources.TurbineResources;

/**
 * This class is responsible for assigning Roles to Users.  The
 * methods in this class are called from the AccessControlList
 * constructor in order to build the ACL.  The reason for implementing
 * this as a separate class is that different applications may have
 * different criteria for building the list.  An application should
 * set the "access.control" property in TurbineResources.properties
 * with the fully qualified class name which implements the
 * AccessControlBuilder interface.
 *
 * This default implementation should be adequate for many needs.
 *
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @author <a href="mailto:bmclaugh@algx.net">Brett McLaughlin</a>
 * @author <a href="mailto:greg@shwoop.com">Greg Ritter</a>
 * @version $Id: DefaultAccessControl.java,v 1.2 2000/08/29 15:39:51 gonzalo Exp $
 */
public class DefaultAccessControl implements AccessControlBuilder
{
    private RoleSet roleSet = null;
    private PermissionSet permissionSet = null;
    // private User user = null;

    /**
     * Builds the RoleSet and PermissionSet on which the
     * AccessControlList is based.  Returns the RoleSet.
     * getPermissionSet() should then be called to return the
     * PermissionSet.  This default implementation does not use the
     * Object obj parameter.
     *
     * @param userName Name of user.
     * @param obj An Object.
     * @return A RoleSet.
     * @exception Exception, a generic exception.
     */
    public RoleSet getRoles(String username,
                            Object obj)
        throws Exception
    {
        String userClassName = TurbineResources.getString("user.class",
            "org.apache.turbine.om.user.TurbineUser");
        User user = ((User)Class.forName(userClassName).newInstance()).retrieveFromStorage( username );
        build(user, obj);
        return roleSet;
    }

    /**
     * Builds the RoleSet and PermissionSet on which the
     * AccessControlList is based.  Returns the RoleSet.
     * getPermissionSet() should then be called to return the
     * PermissionSet.  This default implementation does not use the
     * Object obj parameter.
     *
     * Using this method over getRoles(String username, Object obj)
     * saves a DB hit.
     *
     * @param user A User.
     * @param obj An Object.
     * @return A RoleSet.
     * @exception Exception, a generic exception.
     */
    public RoleSet getRoles(User user,
                            Object obj)
        throws Exception
    {
        build(user, obj);
        return roleSet;
    }

    /**
     * Returns the PermissionSet used in the AccessContolList.
     * getRoles() should be called before this method.
     *
     * @return A PermissionSet.
     */
    public PermissionSet getPermissions()
    {
        return permissionSet;
    }

    /**
     * Build two things:
     *
     * A RoleSet with all roles for a user.
     * A PermissionSet with all permissions for those Roles.
     *
     * @param user A User.
     * @param obj An Object.
     * @exception Exception, a generic exception.
     */
    private void build(User user,
                       Object obj)
        throws Exception
    {
        Criteria criteria = new Criteria();
        criteria.add( VisitorRolePeer.VISITOR_ID, user.getId() );
        roleSet = RolePeer.retrieveSet(criteria);
        if (roleSet != null)
        {
            // Combines Permissions from all Roles into a single
            // PermissionSet.
            permissionSet = new PermissionSet();
            while ( roleSet.hasMoreElements() )
            {
                Role role = roleSet.next();
                criteria = new Criteria();
                criteria.add(RolePermissionPeer.ROLE_ID, role.getId() );
                permissionSet.add(PermissionPeer.retrieveSet(criteria));
            }
        }
    }
}
