package org.apache.turbine.om.security;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.*;
import java.io.*;

/**
 * This class represents a set of Permissions.  It makes it easy to
 * build a UI that would allow someone to add a group of Permissions
 * to a Role.  It wraps a TreeSet object to enforce that only
 * Permission objects are allowed in the set and only relevant methods
 * are available.  TreeSet's contain only unique Objects (no
 * duplicates).
 *
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @author <a href="mailto:bmclaugh@algx.net">Brett McLaughlin</a>
 * @version $Id: PermissionSet.java,v 1.2 2000/08/29 15:39:52 gonzalo Exp $
 */
public class PermissionSet implements Serializable
{
    /* For jdk 1.2:
       // Set to hold the Permission Set
       private TreeSet set = new TreeSet();

       // Iterator for Set
       private Iterator iterator = null;
    */

    /** Vector to hold the Permission set */
    private Vector v = new Vector();
    /** Enumeration for the Permission set */
    private Enumeration enum;

    /**
     * Adds a Permission to this PermissionSet.
     *
     * @param permission A Permission.
     * @return True if Permission was added; false if PermissionSet
     * already contained the Permission.
     */
    public boolean add(Permission permission)
    {
        int index;
        for ( index=0; index<v.size(); index++ )
        {
            int compareResult = permission.getName().compareTo(
                ((Permission) v.elementAt(index)).getName() );
            if ( compareResult < 0 ) break;
            if ( compareResult == 0 ) return false;
        }
        v.insertElementAt( permission, index );
        return true;
    }

    /**
     * Adds the Permissions in another PermissionSet to this
     * PermissionSet.
     *
     * @param permissionSet A PermissionSet.
     * @return True if this PermissionSet changed as a result; false
     * if no change to this PermissionSet occured (this PermissionSet
     * already contained all members of the added PermissionSet).
     */
    public boolean add(PermissionSet permissionSet)
    {
        boolean changed = false;
        while ( permissionSet.hasMoreElements() )
        {
            if ( add( (Permission) permissionSet.next() ) )
                changed = true;
        }
        return changed;
    }

    /**
     * Removes a Permission from this PermissionSet.
     *
     * @param permission A Permission.
     * @return True if this PermissionSet contained the Permission
     * before it was removed, false otherwise.
     */
    public boolean remove(Permission permission)
    {
        for (int index=0; index<v.size(); index++ )
        {
            boolean compareResult = permission.getName()
                .equals( ((Permission)v.elementAt(index)).getName() );
            if (compareResult)
            {
                v.removeElementAt( index );
                return true;
            }
        }
        return false;
    }

    /**
     * Removes all Permissions from this PermissionSet.
     */
    public void clear()
    {
        v.removeAllElements();
    }

    /**
     * Checks whether this PermissionSet contains a Permission.
     *
     * @param permission A Permission.
     * @return True if this PermissionSet contains the Permission,
     * false otherwise.
     */
    public boolean contains(Permission permission)
    {
        return v.contains( (Object) permission );
    }

    /**
     * Compares by name a Permission with the Permissions contained in
     * this PermissionSet.
     *
     * @param permissionName Name of Permission.
     * @return True if argument matched a Permission in this
     * PermissionSet; false if no match.
     */
    public boolean contains(String permissionName)
    {
        Enumeration e = v.elements();
        while ( e.hasMoreElements() )
        {
            Permission permission = (Permission) e.nextElement();
            if ( permissionName != null  &&
                 permissionName.equals( permission.getName() ) )
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Returns a Permission with the given name, if it is contained in
     * this PermissionSet.
     *
     * @param permissionName Name of Permission.
     * @return Permission if argument matched a Permission in this
     * PermissionSet; null if no match.
     */
    public Permission getPermission(String permissionName)
    {
        Enumeration e = v.elements();
        while ( e.hasMoreElements() )
        {
            Permission permission = (Permission)e.nextElement();
            if ( permissionName != null  &&  permissionName.equals( permission.getName() ) )
            {
                return permission;
            }
        }
        return null;
    }

    /**
     * Returns the next Permission in this PermissionSet.
     *
     * @return A Permission.
     */
    public Permission next()
    {
        if (enum == null )
        {
            enum = v.elements();
        }
        return (Permission) enum.nextElement();
    }

    /**
     * Checks to see if any more Permissions exist after the current
     * position of the iterator/enumeration.
     *
     * @return True if next() would return another Permission.
     */
    public boolean hasMoreElements()
    {
        if (enum == null )
        {
            enum = v.elements();
        }
        return enum.hasMoreElements();
    }

    /**
     * Resets the iterator/enumeration to beginning.
     */
    public void resetIterator()
    {
        enum = v.elements();
    }

    /**
     * Returns size (cardinality) of this set.
     *
     * @return The cardinality of this PermissionSet.
     */
    public int size()
    {
        return v.size();
    }

    /**
     * Override writeObject() to only write out the Vector.
     *
     * @param stream The ObjectOutputStream.
     * @exception IOException, an I/O problem.
     */
    private void writeObject(ObjectOutputStream stream) throws IOException
    {
        stream.writeObject(v);
    }

    /**
     * Override the readObject to only read the Vector.
     *
     * @param stream The ObjectInputStream.
     * @exception ClassNotFoundException, an unknown object type was
     * read.
     * @exception IOException, an I/O problem.
     */
    private void readObject(ObjectInputStream stream)
        throws ClassNotFoundException,
               IOException
    {
        v = (Vector) stream.readObject();
    }



    /* For jdk 1.2:


    //
    // Adds a Permission to this PermissionSet.
    //
    // @param permission A Permission.
    // @return True if Permission was added; false if PermissionSet
    // already contained the Permission.
    //
    public boolean add(Permission permission)
    {
        return set.add( (Object)permission );
    }

    //
    // Adds the Permissions in another PermissionSet to this
    // PermissionSet.
    //
    // @param permissionSet A PermissionSet.
    // @return True if this PermissionSet changed as a result; false
    // if no change to this PermissionSet occured (this PermissionSet
    // already contained all members of the added PermissionSet).
    //
    public boolean add(PermissionSet permissionSet)
    {
        return set.addAll(  (Collection) permissionSet.set );
    }

    //
    // Removes a Permission from this PermissionSet.
    //
    // @param permission A Permission.
    // @return True if this PermissionSet contained the Permission
    // before it was removed.
    //
    public boolean remove(Permission permission)
    {
        return set.remove( (Object)permission );
    }

    //
    // Removes all Permissions from this PermissionSet.
    //
    public void clear()
    {
        set.clear();
    }

    //
    // Checks whether this PermissionSet contains a Permission.
    //
    // @param permission A Permission.
    // @return True if this PermissionSet contains the Permission,
    // false otherwise.
    //
    public boolean contains(Permission permission)
    {
        return set.contains( (Object)permission );
    }

    //
    // Compares by name a Permission with the Permissions contained in
    // this PermissionSet.
    //
    // @param permissionName Name of Permission.
    // @return True if argument matched a Permission in this
    // PermissionSet; false if no match.
    //
    public boolean contains(String permissionName)
    {
        Iterator iter = set.iterator();
        while ( iter.hasNext() )
        {
            Permission permission = (Permission)iter.next();
            if ( permissionName != null  &&
                 permissionName.equals( permission.getName() ) )
            {
                return true;
            }
        }
        return false;
    }

    //
    // Returns a Permission with the given name, if it is contained in
    // this PermissionSet.
    //
    // @param permissionName Name of Permission.
    // @return Permission if argument matched a Permission in this
    // PermissionSet; null if no match.
    //
    public Permission getPermission(String permissionName)
    {
        Iterator iter = set.iterator();
        while ( iter.hasNext() )
        {
            Permission permission = (Permission)iter.next();
            if ( permissionName != null  &&
                 permissionName.equals( permission.getName() ) )
            {
                return permission;
            }
        }
        return null;
    }

    //
    // Returns the next Permission in this PermissionSet.
    //
    // @return A Permission.
    //
    public Permission next()
    {
        if (iterator == null )
        {
            iterator = set.iterator();
        }
        return (Permission)(iterator.next());
    }

    //
    // Checks to see if any more Permissions exist after the current
    // position of the iterator/enumeration.
    //
    // @return True if next() would return another Permission.
    //
    public boolean hasMoreElements()
    {
        if (iterator == null )
        {
            iterator = set.iterator();
        }
        return iterator.hasNext();
    }

    //
    // Resets the iterator to beginning.
    //
    public void resetIterator()
    {
        iterator = set.iterator();
    }

    //
    // Returns size (cardinality) of this set.
    //
    // @return The cardinality of this PermissionSet.
    //
    public int size()
    {
        return set.size();
    }

    For jdk 1.2 */
}
