package org.apache.turbine.om.security;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Turbine Utility Classes
import org.apache.turbine.util.*;
import org.apache.turbine.util.db.*;
import org.apache.turbine.util.db.map.*;
import org.apache.turbine.om.*;
import org.apache.turbine.om.peer.*;
import org.apache.turbine.om.user.*;
import org.apache.turbine.om.user.peer.*;
import org.apache.turbine.om.security.peer.*;

/**
 * This class represents a role played by the User associated with the
 * current Session.
 *
 * @author <a href="mailto:frank.kim@clearink.com">Frank Y. Kim</a>
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @author <a href="mailto:bmclaugh@algx.net">Brett McLaughlin</a>
 * @version $Id: Role.java,v 1.2 2000/08/29 15:39:52 gonzalo Exp $
 */
public class Role extends BaseObject
{
    // jdk 1.2:
    // public class Role extends BaseObject implements Comparable

    public static final String ROOT = "Super User";
    public static final String ADMIN = "Administrator";
    public static final String ASST_ADMIN = "Assistant Administrator";
    public static final String MGR = "Manager";
    public static final String ASST_MGR = "Assistant Manager";
    public static final String USER_HIGH = "User Level 1";
    public static final String USER = "User Level 2";
    public static final String USER_LOW = "User Level 3";
    public static final String GUEST = "Guest";

    /** The name of this role. */
    private String name;

    /** The permsissions for this role. */
    private PermissionSet permissionSet = null;

    /**
     * Deletes this role from the database.
     *
     * @exception Exception, a generic exception.
     */
    public void delete()
        throws Exception
    {
        Criteria criteria = new Criteria(2)
            .add( VisitorRolePeer.ROLE_ID, getId() );
        VisitorRolePeer.doDelete( criteria );
        criteria = new Criteria(2)
            .add( RolePeer.ROLE_ID, getId() );
        RolePeer.doDelete( criteria );
    }

    /**
     * Returns the name of this Role.
     *
     * @return A String with the name of the Role.
     */
    public String getName()
    {
        return name;
    }

    /**
     * Returns the set of Permissions associated with this Role.
     *
     * @return A PermissionSet.
     * @exception Exception, a generic exception.
     */
    public PermissionSet getPermissions()
        throws Exception
    {
        return permissionSet;
    }

    /**
     * Renames the role
     *
     * @param name The new Role name.
     * @exception Exception, a generic exception.
     */
    public void rename(String name)
        throws Exception
    {
        setName(name);
        Criteria criteria = new Criteria(2)
            .add( RolePeer.ROLE_ID, getId() )
            .add( RolePeer.NAME, getName() );
        RolePeer.doUpdate( criteria );
    }

    /**
     * Sets the name of this Role.
     *
     * @param name The new Role name.
     */
    public void setName(String name)
    {
        this.name = name;
    }
    /**
     * Sets the Permissions associated with this Role.
     *
     * @param permissionSet A PermissionSet.
     */
    public void setPermissions(PermissionSet permissionSet)
    {
        this.permissionSet = permissionSet;
    }

    /**
     * Adds a Permission to this Role.
     *
     * @param permission A Permission.
     * @return true if Permission was added; false if Role already had
     * the Permission.
     */
    public boolean addPermission(Permission permission)
    {
        if (permissionSet == null)
        {
            permissionSet = new PermissionSet();
        }
        return permissionSet.add(permission);
    }

    /**
     * Adds Permissions from a PermissionSet to this Role.
     *
     * @param permissionSet A PermissionSet.
     * @return true if any new Permissions were added; false if Role
     * already had these Permissions.
     */
    public boolean addPermissions(PermissionSet permissionSet)
    {
        if (this.permissionSet == null)
        {
            this.permissionSet = new PermissionSet();
        }
        return this.permissionSet.add(permissionSet);
    }

    /**
     * Used to order Roles in a RoleSet
     *
     * @param obj An Object to compare to.
     * @return An int indicating the ordering of the two Objects.
     */
    public int compareTo(Object obj)
    {
        String name1 = ((Role)obj).getName();
        String name2 = this.getName();

        return name2.compareTo(name1);
    }

    /**
     * Same as getName().
     *
     * @return A String.
     */
    public String toString()
    {
        return this.getName();
    }
}
