package org.apache.turbine.om.user.peer;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java stuff.
import javax.mail.internet.MimeUtility;
import java.security.MessageDigest;
import java.io.*;

// Turbine stuff.
import org.apache.turbine.om.security.LoginFailedException;
import org.apache.turbine.om.user.*;
import org.apache.turbine.util.*;
import org.apache.turbine.services.resources.TurbineResources;

/**
 * Abstracts user authentication from the user.  All User creation and
 * authentication should be done here.
 *
 * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
 * @version $Id: UserFactory.java,v 1.2 2000/08/29 20:00:22 gonzalo Exp $
 */
public class UserFactory
{
    /**
     * Provides a mechanism to authenticate users.
     *
     * @param username The user name.
     * @param password The user password.
     * @return An authenticated Turbine User.
     * @exception LoginFailedException, thrown if anything goes wrong
     * during authentication.
     */
    public static User getUser( String username,
                                String password )
        throws LoginFailedException
    {
        User user = getUser( username );

        if ( validateUser(user, password) > 0 ) {
            throw new LoginFailedException( TurbineResources.getString("login.error") );
        } else {
            return user;
        }
    }

    /**
     * Get a user from Turbine by name.
     *
     * @param username The user name.
     * @return A non-authenticated Turbine User.
     * @exception LoginFailedException, thrown if anything goes wrong.
     */
    public static User getUser( String username )
        throws LoginFailedException
    {
        String userClassName = TurbineResources
            .getString("user.class", "org.apache.turbine.om.user.TurbineUser");

        User user = null;
        try {
            return user = ((User)Class.forName(userClassName).newInstance()).retrieveFromStorage( username );
        } catch (Exception e) {
            throw new LoginFailedException( e.getMessage() );
        }
    }

    /**
     * Get a user from Turbine by id.
     *
     * @param id The user id.
     * @return A non-authenticated Turbine User.
     * @exception LoginFailedException, thrown if anything goes wrong.
     */
    public static User getUser( int id )
        throws LoginFailedException
    {
        String userClassName = TurbineResources
            .getString("user.class", "org.apache.turbine.om.user.TurbineUser");

        User user = null;
        try {
            return user = ((User)Class.forName(userClassName).newInstance()).retrieveFromStorage( id );
        } catch (Exception e) {
            throw new LoginFailedException( e.getMessage() );
        }
    }

    /**
     * Get a user from Turbine. This user is anonymous.
     *
     * @return A non-authenticated anonymous Turbine User.
     * @exception LoginFailedException, thrown if anything goes wrong.
     */
    public static User getUser()
        throws LoginFailedException
    {
        String userClassName = TurbineResources
            .getString("user.class", "org.apache.turbine.om.user.TurbineUser");

        User user = null;
        try {
            user = (User)Class.forName(userClassName).newInstance();
            user.setUserName("");
            return user;
        } catch (Exception e) {
            throw new LoginFailedException( e.getMessage() );
        }
    }

    /**
     * Validate a user, return a status for the validation.
     *
     * @param user The User object.
     * @param password The user password.
     * @return 0 if username and password are valid, 1 if username is
     * not valid, 2 if password is not valid.
     */
    static int validateUser(User user,
                            String password)
    {
        if (TurbineResources.getBoolean("secure.passwords", false))
        {
            try
            {
                MessageDigest md = MessageDigest.getInstance("SHA");
                byte[] digest = md.digest(password.getBytes());
                byte[] digestPlus = new byte[digest.length +
                                            digest.length % 3];
                for (int i=0; i<digest.length; i++)
                {
                    digestPlus[i] = digest[i];
                }
                ByteArrayOutputStream baos = new ByteArrayOutputStream(digestPlus.length);
                OutputStream encodedStream = MimeUtility.encode(baos, "base64");
                encodedStream.write(digestPlus);
                password = baos.toString();
            }
            catch (Exception e)
            {
                Log.error("Unable to validate encrypted password.", e);
                return 2;
            }
        }
        if ( user != null )
        {
            if ( (user.getPassword() != null) && user.getPassword().equals(password) )
            {
                return 0;
            }
            else
            {
                return 2;
            }
        }
        return 1;
    }
}
