package org.apache.turbine.samples.actions;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.io.*;
import java.util.*;
import java.lang.reflect.*;

// Java Servlet Classes
import javax.servlet.*;
import javax.servlet.http.*;

// Turbine Modules
import org.apache.turbine.modules.*;
import org.apache.turbine.util.*;
import org.apache.turbine.util.upload.*;
import org.apache.turbine.om.security.*;
import org.apache.turbine.om.security.peer.*;
import org.apache.turbine.om.user.*;
import org.apache.turbine.om.user.peer.*;
import org.apache.turbine.services.upload.*;

// ECS Classes
import org.apache.ecs.html.*;

/**
 * This class is an example Action that deals with file uploads.  You
 * may want to only use this class as an example on how to implement
 * file uploads or you can use this class directly.
 *
 * @author <a href="mailto:mbryson@mont.mindspring.com">Dave Bryson</a>
 * @version $Id: UploadFile.java,v 1.3 2000/08/29 20:45:44 gonzalo Exp $
 */
public class UploadFile
    extends Action
{
    /**
     * Perform the action.
     *
     * @param data Turbine information.
     * @exception Exception, a generic exception.
     */
    public void doPerform( RunData data )
        throws Exception
    {
        // Only those with proper permissions should be able to upload
        // files.
        if ( data.getACL().hasPermission(Permission.UPLOAD_FILE) )
        {
            readInputStream(data);
            data.addMessage("Did not perform Upload due to lack of permission.");
            return;
        }

        if ( !data.getRequest().getContentType().startsWith(Form.ENC_UPLOAD) )
        {
            readInputStream(data);
            data.addMessage("This action requires input of type: multipart/from-data.  The data type is " +
                data.getRequest().getContentType() + ".  No further action was performed.");
            return;
        }

        // Default max file size.
        int maxSize = TurbineUpload.getSizeMax();

        // Permissions can override default.
        PermissionSet perms = data.getACL().getPermissions();
        perms.resetIterator();
        while ( perms.hasMoreElements() )
        {
            String permName = perms.next().getName();
            if (permName.startsWith(Permission.FILE_SIZE_LIMIT) )
            {
                int unit = 0;
                int multiplier = 1;
                if (permName.toUpperCase().endsWith("MB"))
                {
                     multiplier = 1024*1024;
                     unit = 2;
                }
                else if (permName.toUpperCase().endsWith("KB"))
                {
                     multiplier = 1024;
                     unit = 2;
                }
                try
                {
                    maxSize = (multiplier *
                               Integer.parseInt(permName.substring(permName.indexOf("="), permName.length() - unit )));
                }
                catch(Exception e)
                {
                    // Leave as default.
                }
            }
        }
        perms.resetIterator();

        if (data.getRequest().getContentLength() > maxSize)
        {
            readInputStream(data);
            data.addMessage( "File is too large.  Upload cancelled." );
            return;
        }

        String path = TurbineUpload.getRepository();
        // Override default directory.
        path = data.getParameters().getString(UploadService.REPOSITORY_PARAMETER, path);
        if ( path == null  ||  !(new File(path).isDirectory()) )
        {
            readInputStream(data);
            data.addMessage( "Specified path: " +
                             path +
                             " is not a directory.  Upload cancelled." );
            return;
        }

        TurbineUpload.parseRequest(data.getRequest(), data.getParameters(), path);
    }

    /**
     * This is a helper method that reads the servlet request input
     * stream.  Apache JServ barfs if you don't read the request on
     * multipart/form-data requests.
     *
     * @param data Turbine information.
     * @exception IOException, an I/O exception.
     */
    private static void readInputStream(RunData data)
        throws IOException
    {
        ServletInputStream sis = data.getRequest().getInputStream();
        byte[] b = new byte[8192];
        while ( sis.read( b, 0, 8192) != -1 );
    }
}
