package org.apache.turbine.services;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java stuff.
import java.io.*;
import java.util.*;
import javax.servlet.*;

// Turbine stuff.
import org.apache.turbine.util.*;

/**
 * A generic implementation of <code>InitableBroker</code>.
 * Functionality provided by the broker includes:
 *
 * <ul>
 *
 * <li>Maintaining single instance of each <code>Initable</code> in
 * the system.</li>
 *
 * <li>Early initialization of <code>Initables</code> during system
 * startup.</li>
 *
 * <li>Late initialization of <code>Initables</code> before they are
 * used.</li>
 *
 * <li>Providing instances of <code>Initables</code> to requesting
 * parties.</li>
 *
 * <li>Maintaining dependencies between <code>Initables</code> during
 * early initalization phases, including circular dependencies
 * detection.</li>
 *
 * </ul>
 *
 * @author <a href="mailto:burton@apache.org">Kevin Burton</a>
 * @author <a href="mailto:krzewski@e-point.pl">Rafal Krzewski</a>
 * @version $Id: BaseInitableBroker.java,v 1.7 2000/09/06 21:50:44 dlr Exp $
 */
public class BaseInitableBroker
    implements InitableBroker
{
    /** A repository of Initable instances. */
    protected Hashtable initables;

    /**
     * Names of classes being early-initialized are pushed onto this
     * stack.  A name appearing twice indicates a circular dependency
     * chain.
     */
    protected Stack stack;


    /**
     * Default constructor of InitableBorker.
     *
     * This constructor does nothing. Your brokers should be
     * singletons, therefore their constructors should be
     * private. They should also have public YourBroker getInstance()
     * methods.
     */
    protected BaseInitableBroker( )
    {
        initables = new Hashtable();
        stack = new Stack();
    }

    /**
     * Performs early initialization of an Initable class.
     *
     * @param className The name of the class to be initailized.
     * @param data An Object to be used for initialization activities.
     * @exception InitializationException, if initilaization of this
     * class was not successful.
     */
    public void initClass( String className,
                           Object data )
        throws InitializationException
    {
        int pos = stack.search(className);
        if(pos != -1)
        {
            StringBuffer msg = new StringBuffer().append(className+" couldn't be initialized because of cricular depency chain:\n");
            for(int i=pos; i>0; i--)
            {
                msg.append((String)stack.elementAt(stack.size()-i-1)+"->");
            }
            msg.append(className+"\n");

            throw new InitializationException(msg.toString());
        }
        stack.push(className);
        try
        {
            Initable instance = getInitableInstance(className);
            if(!instance.getInit())
            {
                instance.init(data);
            }
        }
        finally
        {
            // Succeeded or not, make sure the name gets off the
            // stack.
            stack.pop();
        }
    }

    /**
     * Shuts down an <code>Initable</code>.
     *
     * This method is used to release resources allocated by an 
     * <code>Initable</code>, and return it to its initial (uninitailized)
     * state.
     *
     * @param className The name of the class to be uninitialized.
     */
    public void shutdownClass( String className )
    {
        try
        {
            Initable initable = getInitableInstance(className);
            if(initable.getInit())
            {
                initable.shutdown();
            }
        }
        catch( InstantiationException ignored )
        {
            // Shutdown of a nonexistent class was requested.
            // This does not hurt anything, so we ignore this error.
        }
    }

    /**
     * Provides an instance of Initable class ready to work.
     *
     * If the requested class couldn't be instatiated or initialized,
     * an InstantiationException will be thrown. You needn't handle
     * this exception in your code, since it indicates fatal
     * misconfigurtion of the system.
     *
     * @param className The name of the Initable requested.
     * @return An instance of the requested Initable.
     * @exception InstantiationException, if there was a problem
     * during instantiation or initialization of the Initable.
     */
    public Initable getInitable( String className )
        throws InstantiationException
    {
        Initable initable;
        try
        {
            initable = getInitableInstance(className);
            if(!initable.getInit())
            {
                initable.init();
            }
            if(!initable.getInit())
            {
                throw new InitializationException("init() failed to initialize class "+className);
            }
            return initable;
        }
        catch( InitializationException e )
        {
            throw new InstantiationException("Class "+className+" failed to initialize:\n"+e.getMessage());
        }
    }

    /**
     * Retrieves an instance of an Initable from the repository.
     *
     * If the requested class is not present in the repository, it is
     * instantiated and passed a reference to the broker, saved and
     * then returned.
     *
     * @param className The name of the class to be instantiated.
     * @exception InstantiationException, if the requested class can't
     * be instantiated.
     */
    protected Initable getInitableInstance( String className )
        throws InstantiationException
    {
        Initable initable = (Initable)initables.get(className);

        if(initable == null)
        {
            // Used to indicate error condition.
            String msg = null;
            try
            {
                initable = (Initable)Class.forName(className).newInstance();
            }
            catch( NoClassDefFoundError e)
            {
                msg = "Class "+className+" is unavailable. Check your jars and classes.";
            }
            catch( ClassNotFoundException e)
            {
                msg = "Class "+className+" is unavailable. Check your jars and classes.";
            }
            catch( ClassCastException e )
            {
                msg = "Class "+className+" doesn't implement Initable.";
            }
            catch( Exception e )
            {
                msg = "Failed to instantiate "+className+":\n"+e.getMessage();
            }

            if(msg != null)
            {
                throw new InstantiationException(msg);
            }

            initable.setInitableBroker(this);
            initables.put(className, initable);
        }

        return initable;
    }
}
