package org.apache.turbine.services;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java stuff.
import java.util.*;

// Turbine
import org.apache.turbine.util.Log;

/**
 * A generic implementation of a <code>ServiceBroker</code>.
 *
 * Functionality that <code>ServiceBroker</code> provides in addition
 * to <code>InitableBroker</code> functionality includes:
 *
 * <ul>
 *
 * <li>Maintaining service name to class name mapping, allowing
 * plugable service implementations.</li>
 *
 * <li>Providing <code>Services</code> with <code>Properties</code>
 * based on system wide configuration mechanism.</li>
 *
 * </ul>
 *
 * @author <a href="mailto:burton@apache.org">Kevin Burton</a>
 * @author <a href="mailto:krzewski@e-point.pl">Rafal Krzewski</a>
 * @version $Id: BaseServiceBroker.java,v 1.5 2000/09/06 21:50:44 dlr Exp $
 */
public class BaseServiceBroker
    extends BaseInitableBroker
    implements ServiceBroker
{
    /** Mapping of Service names to class names. */
    protected Hashtable mapping;

    /** A repository of Service instances. */
    protected Hashtable services;

    /**
     * A constructor that initializes member objects, and calls
     * InitMapping.
     */
    protected BaseServiceBroker( )
    {
        super();
        services = new Hashtable();
        mapping = new Hashtable();
        initMapping();
    }

    /**
     * Creates mapping of Service names to class names.
     * BaseServiceBroker knows no mappings.
     */
    protected void initMapping()
    {
    }

    /**
     * Performs early initialization of specified service.
     *
     * @param name The name of the service.
     * @param data An Object to use for initialization activities.
     * @exception InitializationException, if initilaization of this
     * service was not successful.
     */
    public void initService( String name,
                             Object data )
        throws InitializationException
    {
        String className = (String)mapping.get(name);
        if(className == null)
        {
            throw new RuntimeException("ServiceBroker: initialization of unknown service "+name+" requested.");
        }
        initClass(className, data);
    }

    /**
     * Performs early initialization of all services.  Failed early
     * initialization of a Service may be non-fatal to the system.
     * This method ignores such events.  Your ServiceBroker should use
     * some logging mechanism to notify the user about failed
     * initializations.
     *
     * @param data An Object to use for initialization activities.
     */
    public void initServices( Object data )
    {
        Enumeration names = mapping.keys();
        while(names.hasMoreElements())
        {
            String name = (String)names.nextElement();
            try
            {
                // Make sure the service has it's name and broker
                // reference set before initialization.
                getServiceInstance(name);

                // Perform early initialization.
                initClass((String)mapping.get(name), data);
            }
            catch( Exception ignored ) {}
        }
    }

    /**
     * Shuts down a <code>Service</code>.
     *
     * This method is used to release resources allocated by a
     * Service, and return it to its initial (uninitailized) state.
     *
     * @param name The fully qualified class name of the <code>Service</code> 
     *             to be uninitialized.
     */
    public void shutdownService( String name )
    {
        String className = (String)mapping.get(name);
        if (className != null)
        {
            shutdownClass(className);
        }
    }

    /**
     * Shuts down all Turbine services, releasing allocated resources and 
     * returning them to their initial (uninitailized) state.
     */
    public void shutdownServices( )
    {
        Enumeration serviceNames = mapping.keys();
        String serviceName = null;
        while (serviceNames.hasMoreElements())
        {
            serviceName = (String)serviceNames.nextElement();
            shutdownClass((String)mapping.get(serviceName));
            Log.note(serviceName + " service shut down");
        }
    }

    /**
     * Returns an instance of requested Service.
     *
     * @param name The name of the Service requested.
     * @return An instance of requested Service.
     * @exception InstantiationException, if the service is unknown or
     * can't be initialized.
     */
    public Service getService( String name )
        throws InstantiationException
    {
        Service service;
        try
        {
            service = getServiceInstance(name);
            if(!service.getInit())
            {
                service.init();
            }
            if(!service.getInit())
            {
                throw new InitializationException("init() failed to initialize service "+name);
            }
            return service;
        }
        catch( InitializationException e )
        {
            throw new InstantiationException("Service "+name+" failed to initialize:\n"+e.getMessage());
        }
    }

    /**
     * Retrieves an instance of a Service without tirggering late
     * initialization.
     *
     * Early initialization of a Service can require access to Service
     * properties.  The Service must have its name and serviceBroker
     * set by then.  Therefore, before calling
     * Initable.initClass(Object), the class must be instantiated with
     * InitableBroker.getInitableInstance(), and
     * Service.setServiceBroker() and Service.setName() must be
     * called.  This calls for two - level accesing the Services
     * instances.
     *
     * @param name The name of the service requested.
     * @exception InstantiationException, if the service is unknown or
     * can't be initialized.
     */
    protected Service getServiceInstance( String name )
        throws InstantiationException
    {
        Service service = (Service)services.get(name);
        if(service != null)
        {
            return service;
        }
        else
        {
            String className = (String)mapping.get(name);
            if(className == null)
            {
                throw new InstantiationException("ServiceBroker: unknown service "+name+" requested.");
            }
            try
            {
                service = (Service)getInitableInstance(className);
            }
            catch(ClassCastException e)
            {
                throw new InstantiationException("ServiceBroker: class "+className+" does not implement Service.");
            }
            catch(InstantiationException e)
            {
                throw new InstantiationException("ServiceBroker: failed to instantiate service "+name+":\n"+e.getMessage());
            }
            service.setServiceBroker(this);
            service.setName(name);
            services.put(name, service);
            return service;
        }
    }

    /**
     * Returns the properites of a specific service.
     *
     * Generic ServiceBroker returns empty set of Properties.
     *
     * @param name The name of the service.
     * @return Properties of requested Service.
     */
     public Properties getProperties( String name )
     {
        return new Properties();
     }
}
