package org.apache.turbine.services;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.io.*;
import java.util.*;
import javax.servlet.ServletConfig;

// Turbine Utility Classes
import org.apache.turbine.util.*;
import org.apache.turbine.services.resources.TurbineResources;

/**
 * This is a singleton utility class that acts as a Services broker.
 *
 * @author <a href="mailto:greg@shwoop.com">Greg Ritter</a>
 * @author <a href="mailto:bmclaugh@algx.net">Brett McLaughlin</a>
 * @author <a href="mailto:burton@apache.org">Kevin Burton</a>
 * @author <a href="mailto:krzewski@e-point.pl">Rafal Krzewski</a>
 * @version $Id: TurbineServices.java,v 1.4 2000/08/29 22:07:12 gonzalo Exp $
 */
public class TurbineServices
    extends BaseServiceBroker
{
    /**
     * A prefix for <code>Service</code> properties in
     * TurbineResource.properties.
     */
    public static final String SERVICE_PREFIX = "services.";

    /**
     * A <code>Service</code> property determining its implementing
     * class name .
     */
    public static final String CLASSNAME_SUFFIX = ".classname";

    /** The single instance of this class. */
    private static TurbineServices instance = null;

    /**
     * This constructor is protected to force clients to use
     * getInstance() to access this class.
     */
    protected TurbineServices()
    {
        super();
    }

    /**
     * The method through which this class is accessed.
     *
     * @return The single instance of this class.
     */
    public static TurbineServices getInstance()
    {
        if (instance == null)
        {
            synchronized (TurbineServices.class)
            {
                if (instance == null)
                {
                    instance = new TurbineServices();
                }
            }
        }
        return instance;
    }

    /**
     * Creates a mapping between Service names and class names.
     *
     * The mapping is built according to settings present in
     * TurbineResources.properties.  The entries should have the
     * following form:
     *
     * <br>
     *
     * services.MyService.classname=com.mycompany.MyServiceImpl
     *
     * <br>
     *
     * services.MyOtherService.classname=com.mycompany.MyOtherServiceImpl
     *
     * <br>
     *
     * Generic ServiceBroker provides no Services.
     */
    protected void initMapping()
    {
        super.initMapping();

        int pref = SERVICE_PREFIX.length();
        int suff = CLASSNAME_SUFFIX.length();

        Enumeration keys = TurbineResources.getKeys();
        while(keys.hasMoreElements())
        {
            String key = (String)keys.nextElement();
            if(key.startsWith(SERVICE_PREFIX) && key.endsWith(CLASSNAME_SUFFIX))
            {
                mapping.put(key.substring(pref, key.length() - suff), TurbineResources.getString(key));
            }
        }
    }

    /**
     * Returns the properites of a specific service.  Properties are
     * retrieved from TurbineResources.properties provided that you
     * have the following entires:
     *
     * <br>
     *
     * services.MyService.greeting=Hello, I'm Jan B.
     *
     * <br>
     *
     * services.MyService.defaultAction=beep
     *
     * <br>
     *
     * Service "MyService" will get a set of two properites:
     * "greeting" = "Hello, I'm Jan B." and "defaultAction" = "beep".
     *
     * @param name The name of the service.
     * @return Properties of requested Service.
     */
    public Properties getProperties( String name )
    {
        Properties properties = new Properties();

        Enumeration keys = TurbineResources.getKeys(SERVICE_PREFIX+name+".");
        int pref = SERVICE_PREFIX.length()+name.length()+1;
        while(keys.hasMoreElements())
        {
            String key = (String)keys.nextElement();
            properties.put(key.substring(pref), TurbineResources.getString(key));
        }

        return properties;
    }

    /**
     * Performs early initialization of all services.  Failed early
     * initialization of a Service may be non-fatal to the system.
     * This method logs failed initializations using the standard
     * Turbine logging facility.
     *
     * @param data An Object to use for initialization activities.
     */
    public void initServices( Object data )
    {
        Enumeration names = mapping.keys();
        while(names.hasMoreElements())
        {
            String name = (String)names.nextElement();
            try
            {
                // Make sure the service has it's name and broker
                // reference set before initialization.
                getServiceInstance(name);

                // Perform early initialization.
                initClass((String)mapping.get(name), data);
            }

            // In case of an exception, file an error message; the
            // system may be still functional, though.
            catch( InstantiationException e )
            {
                org.apache.turbine.util.Log.error(e);
            }
            catch( InitializationException e )
            {
                org.apache.turbine.util.Log.error(e);
            }
        }
    }
}
