package org.apache.turbine.services.freemarker;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Stuff
import java.util.*;
import java.io.*;

// Java Servlet Classes
import javax.servlet.*;

// Turbine Stuff
import org.apache.turbine.services.*;
import org.apache.turbine.services.resources.*;
import org.apache.turbine.util.*;
import org.apache.turbine.om.security.*;

import org.apache.ecs.StringElement;

// FreeMarker Stuff
import freemarker.template.*;

/*
 * This is a Service that can process FreeMarker templates from within
 * a Turbine Screen.  Here's an example of how you might use it from a
 * screen:
 *
 * <br>
 *
 * FreeMarkerService fm = (FreeMarkerService)TurbineServices.getInstance()
 * .getService(FreeMarkerService.SERVICE_NAME);
 * SimpleHash context = fm.getContext(data);
 * context.put("message", "Hello from Turbine!");
 * String results = fm.handleRequest(context,"helloWorld.wm");
 * data.getPage().getBody().addElement(results);
 *
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @version $Id: TurbineFreeMarkerService.java,v 1.4 2000/08/29 23:19:40 gonzalo Exp $
 */
public class TurbineFreeMarkerService
    extends TurbineBaseService
    implements FreeMarkerService, CacheListener
{
    /** A cache to store parsed templates. */
    private FileTemplateCache templateCache;

    /** The base path prepended to filenames given in arguments. */
    private String path;

    /**
     * Constructor.
     *
     * @exception Exception, a generic exception.
     */
    public TurbineFreeMarkerService()
        throws Exception
    {
    }

    /**
     * Called during Turbine.init()
     *
     * @param config A ServletConfig.
     */
    public void init( ServletConfig config )
    {
        try
        {
            org.apache.turbine.util.Log.note ( "FreeMarkerService early init()....starting!");
            initFreeMarker(config);
            setInit(true);
            org.apache.turbine.util.Log.note ( "FreeMarkerService early init()....finished!");
        }
        catch (Exception e)
        {
            org.apache.turbine.util.Log.error ( "Cannot initialize TurbineFreeMarkerService!" );
            org.apache.turbine.util.Log.error (e);
        }
   }


    /*

    //
    // Called the first time the Service is used.
    //
    public void init()
    {
        try
        {
            org.apache.turbine.util.Log.note ( "FreeMarkerService init()....starting!");
            initFreeMarker();
            setInit(true);
            org.apache.turbine.util.Log.note ( "FreeMarkerService init()....finished!");
        }
        catch (Exception e)
        {
            org.apache.turbine.util.Log.error ( "Cannot initialize TurbineFreeMarkerService!" );
            org.apache.turbine.util.Log.error (e);
        }
    }

    */

    /**
     * Create a context needed by the FreeMarker template.  This
     * method just returns an SimpleHash with the request parameters
     * copied into a model called request.
     *
     * @return SimpleHash which can be used as the model for a
     * template.
     */
    public SimpleHash getContext()
    {
        return new SimpleHash();
    }

    /**
     * Create a context needed by the FreeMarker template.  This
     * method just returns an SimpleHash with the request parameters
     * copied into a model called request.
     *
     * @param req A ServletRequest.
     * @return SimpleHash which can be used as the model for a
     * template.
     */
    public SimpleHash getContext(ServletRequest req)
    {
        return (SimpleHash)TemplateServletUtils.copyRequest(req);
    }

    /**
     * Create a context from the RunData object.  Values found in
     * RunData are copied into the modelRoot under similar names as
     * they can be found in RunData. e.g. data.serverName,
     * data.parameters.form_field_name
     * data.acl.permissions.can_write_file.  Some default links are
     * also made available under links.
     *
     * @param data The Turbine RunData object.
     * @return a SimpleHash populated with RunData data.
     */
    public SimpleHash getContext(RunData data)
    {
        SimpleHash modelRoot = new SimpleHash();
        SimpleHash modelLinks = new SimpleHash();
        modelRoot.put("link", new DynamicURIModel(data));
        modelRoot.put("links", modelLinks);
        // modelLinks.put("default", new DynamicURI(data, "FreeMarkerScreen").toString());
        // modelLinks.put("no_cache", new DynamicURI(data, "FreeMarkerNoCacheScreen").toString());
        // modelLinks.put("simple", new DynamicURI(data).toString());
        if (data.getUser() != null && data.getUser().hasLoggedIn())
        {
            modelRoot.put("loginout_link", new DynamicURI(data, "Login", "LogoutUser").toString());
            modelRoot.put("loginout_text", "Logout");
        }
        else
        {
            modelRoot.put("loginout_link", new DynamicURI(data, "Login").toString());
            modelRoot.put("loginout_text", "Login");
        }

        // Add default.
        SimpleHash modelData = new SimpleHash();
        SimpleHash modelParameters = new SimpleHash();
        SimpleHash modelACL = new SimpleHash();
        SimpleHash modelRoles = new SimpleHash();
        SimpleHash modelPermissions = new SimpleHash();
        modelRoot.put("data", modelData);
        modelData.put("parameters", modelParameters);
        modelData.put("acl", modelACL);
        modelACL.put("roles", modelRoles);
        modelACL.put("permissions", modelPermissions);

        modelData.put("serverName", data.getServerName() );
        modelData.put("serverPort", String.valueOf(data.getServerPort()) );
        modelData.put("serverScheme", data.getServerScheme() );
        modelData.put("scriptName", data.getScriptName() );
        modelData.put("screen", data.getScreen() );
        modelData.put("action", data.getAction() );
        modelData.put("title", data.getTitle() );
        modelData.put("message", data.getMessage() );

        ParameterParser params = data.getParameters();
        Enumeration e = params.keys();
        while (e.hasMoreElements() )
        {
            String key = (String)e.nextElement();
            String[] values = params.getStrings(key);
            if (values.length==1)
            {
                modelParameters.put(key, values[0]);
            }
            else
            {
                SimpleList listModel = new SimpleList();
                modelParameters.put(key, listModel);
                for (int i=0; i<values.length; i++)
                {
                    listModel.add(values[i]);
                }
            }
        }

        if (data.getACL() != null)
        {
            RoleSet roles = data.getACL().getRoles();
            roles.resetIterator();
            while (roles.hasMoreElements() )
            {
                String key = roles.next().getName();
                modelRoles.put(key, true);
            }
            roles.resetIterator();

            PermissionSet permissions = data.getACL().getPermissions();
            permissions.resetIterator();
            while (permissions.hasMoreElements() )
            {
                String key = permissions.next().getName();
                modelPermissions.put(key, true);
            }
            permissions.resetIterator();
        }

        modelRoot.put("setTitle", new SetTitleModel(data));
        modelRoot.put("addToHead", new AddToHeadModel(data));
        modelRoot.put("addTemplatesToHead", new AddTemplatesToHeadModel(data));
        modelRoot.put("setBodyAttributes", new SetBodyAttributesModel(data));

        String templatePath = params.getString("template", null);
        if (templatePath != null)
        {
            StringTokenizer st = new StringTokenizer(templatePath, "/");
            int max = st.countTokens() - 1;
            for (int i=0; i<max; i++)
            {
                modelRoot.put("template_path_" + (i+1), st.nextToken());
            }
        }

        return modelRoot;
    }

    /**
     * Process the request and fill in the template with the values
     * you set in the WebContext.
     *
     * @param context A SimpleHash with the context.
     * @param templateName A String with the filename of the template.
     * @param cache True if the parsed template should be cached.
     * @return The processed template as a String.
     */
    public String handleRequest(SimpleHash context,
                                String filename,
                                boolean cache)
                                // throws Exception
    {
        String results = null;
        StringWriter sw = null;
        PrintWriter out = null;
        try
        {
            sw = new StringWriter();
            out = new PrintWriter(sw);
            Template template = null;
            if (cache)
            {
                template = getCachedTemplate(filename);
            }
            else
            {
                template = getNonCachedTemplate(filename);
            }

            template.process(context, out);
            results = sw.toString();
        }
        catch(Exception e)
        {
            org.apache.turbine.util.Log.error ("Error encountered processing a template:" + filename);
            org.apache.turbine.util.Log.error (e);
            results = "Error encountered processing a template: " + e.toString();
        }
        finally
        {
            try
            {
                if (out != null) out.close();
                if (sw != null) sw.close();
            }
            catch (Exception e) {}
        }
        return results;
    }

    /**
     * Gets the base path for the FreeMarker templates.
     *
     * @return The base path for the FreeMarker templates.
     */
    public String getBasePath()
    {
        return path;
    }

    /**
     * Return a FreeMarker template from the cache.  If the template
     * has not been cached yet, it will be added to the cache.
     *
     * @param templateName A String with the name of the template.
     * @return A Template.
     */
    public Template getCachedTemplate(String templateName)
    {
        Template template = templateCache.getTemplate(templateName);
        // if (template == null)
        // {
        //     Exception e = new Exception("Template " + templateName +
        //                                 " is not available.");
        //     Log.error ( e );
        //     throw e;
        // }
        return template;
    }

    /**
     * Return a FreeMarker template. It will not be added to the
     * cache.
     *
     * @param templateName A String with the name of the template.
     * @return A Template.
     * @exception IOException, if there was an I/O problem.
     */
    public Template getNonCachedTemplate(String templateName)
        throws IOException
    {
        templateName = templateName.replace('/', File.separatorChar);
        File file = new File(path, templateName);
        if (file.canRead())
        {
            return new Template(file);
        }
        // else
        // {
        //     Exception e = new Exception("Template " + templateName +
        //                                 " could not be read.");
        //     Log.error ( e );
        //     throw e;
        // }
        return null;
    }

    /**
     * This method sets up the FreeMarker template cache.
     *
     * @param config A ServletConfig.
     * @exception Exception, a generic exception.
     */
    private void initFreeMarker(ServletConfig config)
        throws Exception
    {
        ServletContext context = config.getServletContext();
        Properties props = getProperties();

        path = props.getProperty("templates", "/WEB_INF/templates");

        // If getRealPath is working, transform paths to be webapp
        // root relative.
        if (context.getRealPath("/") != null)
        {
            String pathSep = System.getProperty("path.separator");
            StringTokenizer st = new StringTokenizer(path,pathSep);
            StringBuffer buf = new StringBuffer();
            while(st.hasMoreTokens())
            {
                buf.append(context.getRealPath(st.nextToken()));
                if(st.hasMoreTokens())
                {
                    buf.append(pathSep);
                }
            }
            path = buf.toString();
        }

        // Store the converted path in service properties for Turbine
        // based providers.
        props.put("templates", path);

        templateCache = new FileTemplateCache(path);
        templateCache.addCacheListener(this);
        templateCache.startAutoUpdate();
    }

    /**
     * Method called by templateCache.
     *
     * @param e A CacheEvent.
     */
    public void cacheUnavailable(CacheEvent e)
    {
        Log.error("Cache unavailable: " + e.getException().toString());
    }

    /**
     * Method called by templateCache.
     *
     * @param e A CacheEvent.
     */
    public void elementUpdated(CacheEvent e)
    {
        Log.note("Template updated: " + e.getElementName());
    }

    /**
     * Method called by templateCache.
     *
     * @param e A CacheEvent.
     */
    public void elementUpdateFailed(CacheEvent e)
    {
        Log.error("Update of template " + e.getElementName() +
                  " failed: " + e.getException().toString());
    }

    /**
     * Method called by templateCache.
     *
     * @param e A CacheEvent.
     */
    public void elementRemoved(CacheEvent e)
    {
        Log.warn("Template removed: " + e.getElementName());
    }
}
