package org.apache.turbine.services.resources;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.io.*;
import java.util.*;

// Turbine stuff.
import org.apache.java.util.*;
import org.apache.turbine.services.resources.TurbineResources;

/**
 * This class is an error resource bundle.  It is a collection of
 * error messages that can be used for log files and exception
 * handling to define messages.  The main addition to this bundle from
 * a generic resource bundle is the getMessage(String, String[])
 * method.  This method returns a string to be printed in a log file.
 * The first string is a resource name or key, the array of strings is
 * a parameter array.  The getMessage method looks up the message
 * string that corresponds to the key.  It then parses the message
 * string, inserting the parameters where they belong.  The parameters
 * are inserted in the string where there is a $[number] combination.
 * This was modelled after shell scripts which use $1, $2, etc., to
 * represent command line arguments.  $1 corresponds to params[0], $2
 * corresponds to params[1], etc.
 *
 * @author <a href="mailto:jeffprickett@mindspring.com">Jeff Prickett</a>
 * @version $Id: ErrorResources.java,v 1.2 2000/08/30 20:57:07 gonzalo Exp $
 */
public class ErrorResources
{
    /**
     * The key that corresponds to the error condition that exists
     * when a properties parameter is not found.  The message for this
     * error has one parameter argument.  It is as follows: $1 =
     * properties init argument name.
     */
    public static final String PROPERTIES_INIT_PARAMETER_NOT_FOUND =
           "error.org.apache.Turbine.init.1";

    /**
     * The key that corresponds to the error condition that exists
     * when a propeties parameter is a zero length string.  The
     * message for this error has one parameter argument.  It is as
     * follows: $1 = properties init argument name.
     */
    public static final String PROPERTIES_INIT_PARAMETER_ZERO_LENGTH_STRING =
           "error.org.apache.Turbine.init.2";

    /**
     * The key that corresponds to the error condition that exists
     * when the turbine resources file does not exist.  The message
     * for this error has two parameters:
     *
     *        $1 = Turbine Resources property file name
     *        $2 = properties init argument name
     */
    public static final String TURBINE_RESOURCE_FILE_DOES_NOT_EXIST =
           "error.org.apache.Turbine.init.3";

    /**
     * The key that corresponds to the error condition that exists
     * when the turbine resources file is not readable.  The message
     * for this error has two parameters:
     *
     *       $1 = Turbine Resources property file name
     *       $2 = user who is running JVM process
     */
    public static final String TURBINE_RESOURCE_FILE_NOT_READABLE =
           "error.org.apache.Turbine.init.4";

    private static ExtendedProperties defaults = null;
    private static MessageResources generic = null;
    private static String fileName = null;

    /**
     * Constructor.
     *
     * @exception IOException, if there was an IO problem.
     */
    public ErrorResources()
        throws IOException
    {
        init();
    }

    /**
     * Basic constructor that allows you to specify the locations of
     * the Configurations object.  If you use this constructor, you
     * don't need to call setPropertiesFileName(), since the path to
     * the file is already set in the constructor for Configurations.
     *
     * @param conf A Configurations.
     * @exception IOException, if there was an IO problem.
     */
    public ErrorResources(Configurations confs)
        throws IOException
    {
        init(confs);
    }

    /**
     * Basic constructor that allows you to specify the locations of
     * the Configurations object.  If you use this constructor, you
     * don't need to call setPropertiesFileName() since the path to
     * the file is already set in the constructor for Configurations.
     *
     * @param fileName A String the the configuration file name.
     * @exception IOException, if there was an IO problem.
     */
    public ErrorResources(String fileName)
        throws IOException
    {
        init(fileName);
    }

    /**
     * Initializer method that sets up the generic default resources.
     *
     * @exception IOException, if there was an IO problem.
     */
    public static void init()
        throws IOException
    {
        if ( fileName == null || fileName.length() == 0 )
            init(new Configurations((ConfigurationsRepository) defaults));
        else
            init(new Configurations(new ExtendedProperties(fileName)));
    }

    /**
     * Initializer method that sets up the generic resources.
     *
     * @param conf A Configurations.
     */
    private static void init(Configurations confs)
    {
        generic = new MessageResources (confs);
    }

    /**
     * Initializer method that sets up the generic resources.
     *
     * @param fileName A String the the configuration file name.
     * @exception IOException, if there was an IO problem.
     */
    private static void init(String fileName)
        throws IOException
    {
        setPropertiesFileName(fileName);
    }

    /**
     * Set the file name of the properties file.  This
     * <strong>MUST</strong> be called before this class is used with
     * the default no argument constructor.
     *
     * @param propertiesFileName The file name.
     * @exception IOException, if there was an IO problem.
     */
    public static void setPropertiesFileName(String propertiesFileName)
        throws IOException
    {
        fileName = propertiesFileName;
        init();
    }

    /**
     * Static initializer to make sure that the defaults are
     * initialized.
     */
    static
    {
        initDefaults();
    }

    /**
     * Non static initializer to make sure that the defaults are
     * initialized.
     */
    {
        initDefaults();
    }

    /**
     * Initialize the defaults.
     */
    private static void initDefaults()
    {
        try
        {
            /*
             * Populate the default error resources with messages.
             */
            defaults = new ExtendedProperties();
            defaults.put((Object) PROPERTIES_INIT_PARAMETER_NOT_FOUND,
                   (Object) "Turbine::init(ServletConfig) - '$1' Init Parameter"
                   + " not found. Please specify a value for the '$1' Init " +
                   "Parameter by editing the servlet zone properties " +
                   "file. This is done by adding the following line to the zone"
                   + " properties file: " +
                   "servlet.Turbine.initArgs=$1=" +
                   "/path/to/TurbineResources.properties");

            defaults.put((Object) PROPERTIES_INIT_PARAMETER_ZERO_LENGTH_STRING,
                  (Object) "Turbine::init(ServletConfig) - '$1' Init Parameter"
                  + " value is empty. The '$1' Init Parameter exists but it is "
                  + "a zero length string. Please specify a value for the '$1'"
                  + " Init Parameter. This is done by adding the following line "
                  + "to the zone properties file: " +
                  "servlet.Turbine.initArgs=properties=" +
                  "/path/to/TurbineResources.properties");

            defaults.put((Object) TURBINE_RESOURCE_FILE_DOES_NOT_EXIST,
                   (Object) "Turbine::init(ServletConfig) - The Turbine " +
                   "Resources "
                   + "File - '$1' does not exist. Please create it or change "
                   + "the"
                   + " '$2' Init Parameter Value to a file name that exists and"
                   + " contains the TurbineResources.");

            defaults.put((Object) TURBINE_RESOURCE_FILE_NOT_READABLE,
                   (Object) "Turbine::init(ServletConfig) - The Turbine " +
                   "Resources"
                   + " Property file - '$1' is not readable by user '$2'. "
                   + "Please"
                   + " change permissions on the file. In UNIX this is done by "
                   + "executing the chmod command.");
        }
        catch(Exception e)
        {
            System.err.println(e);
            e.printStackTrace();
        }
    }

    /**
     * The purpose of this method is to get the configuration message,
     * with the given name and parameters, as a string.
     *
     * @param messageName The name of the message.
     * @param messageParams The parameters for the message.
     * @return The value of the message as a String.
     * @exception Exception, a generic exception.
     */
    public static String getMessage(String messageName,
                                    Object[] messageParams)
        throws Exception
    {
        return generic.getMessage(messageName, messageParams);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a string.
     *
     * @param name The resource name.
     * @return The value of the resource as a String.
     */
    public static String getString(String name)
    {
        return generic.getString(name);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a string, or a default value.
     *
     * @param name The resource name.
     * @param def The default value of the resource.
     * @return The value of the resource as a String.
     */
    public static String getString(String name,
                                   String def)
    {
        return generic.getString(name, def);
    }
}
