package org.apache.turbine.services.resources;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.io.*;
import java.util.*;

// Turbine stuff.
import org.apache.java.util.Configurations;
import org.apache.java.util.ExtendedProperties;

/**
 * This is a mostly static class for accessing the
 * TurbineResources.properties file.  It is built in such a way that
 * it is also possible to access this class through the
 * TurbineResourceService as well, but since this class is mostly
 * static, there isn't much of a reason to do so.
 *
 * <p>
 *
 * An example bit of code:
 *
 * <p>
 *
 * TurbineResourceService.setPropertiesFileName("d:/foo/TurbineResources.properties");
 * String foo = TurbineResources.getString("foo");
 * int bar = TurbineResources.getInt("bar",1);
 *
 * <p>
 *
 * Since this class is mostly static, you only need to call
 * setPropertiesFileName() once per JVM invocation.  Once you have
 * called that method, you can call the other methods which are
 * essentially wrappers around the GenericResources class.
 *
 * @author <a href="mailto:jon@clearink.com">Jon S. Stevens</a>
 * @author <a href="mailto:greg@shwoop.com">Greg Ritter</a>
 * @version $Id: TurbineResources.java,v 1.6 2000/08/30 20:57:08 gonzalo Exp $
 */
public class TurbineResources
{
    public static final String MAIL_SERVER_KEY = "mail.server";
    public static final String MODULE_CACHE_KEY = "module.cache";
    public static final String ACCESS_CONTROL_KEY = "access.control";
    public static final String MODULE_PACKAGES_KEY = "module.packages";
    public static final String ACTION_CACHE_SIZE_KEY = "action.cache.size";
    public static final String LAYOUT_CACHE_SIZE_KEY = "layout.cache.size";
    public static final String NAVIGATION_CACHE_SIZE_KEY = "navigation.cache.size";
    public static final String PAGE_CACHE_SIZE_KEY = "page.cache.size";
    public static final String SCREEN_CACHE_SIZE_KEY = "screen.cache.size";
    public static final String USER_CLASS_KEY = "user.class";
    public static final String MAX_FILE_SIZE_KEY = "max.file.size.bytes";
    public static final String DB_DRIVER_KEY = "database.default.driver";
    public static final String DB_URL_KEY = "database.default.url";
    public static final String DB_USERNAME_KEY = "database.default.username";
    public static final String DB_PASSWORD_KEY = "database.default.password";
    public static final String DB_MAXCONNECTIONS_KEY = "database.maxConnections";
    public static final String DB_EXPIRYTIME_KEY = "database.expiryTime";
    public static final String FILE_SERVER = "file.server";
    public static final String LOGIN_MESSAGE = "login.message";
    public static final String LOGIN_ERROR = "login.error";
    public static final String LOGIN_MESSAGE_NOSCREEN = "login.message.noscreen";
    public static final String LOGOUT_MESSAGE = "logout.message";
    public static final String DB_IDBROKER_CLEVERQUANTITY = "database.idbroker.cleverquantity";

    /** The name of the file to load properties from. */
    private static String fileName = null;

    /** The container for the generic resources. */
    private static GenericResources generic = null;

    /**
     * Package private constructor.  This class should be only used
     * through the context of the TurbineResourceService class.
     * Please see the javadoc for this class for more information and
     * example usage.
     */
    TurbineResources()
    {
    }

    /**
     * Set the file name of the properties file.  This method is
     * pacakge dependent because you should call
     * TurbineResourceService.setPropertiesFileName() before any
     * static methods in this class are used.
     *
     * @param propertiesFileName The file name.
     * @exception IOException, if there was an I/O problem.
     */
    static void setPropertiesFileName(String propertiesFileName)
        throws IOException
    {
        fileName = propertiesFileName;
        init();
    }

    /**
     * Set the properties.  This method is pacakge dependent because
     * you should call TurbineResources.setProperties() before any
     * static methods in this class are used.
     *
     * @param properties A Properties object.
     */
    static void setProperties(Properties properties)
    {
        init(properties);
    }

    /**
     * Wrapper around the generic resources.
     *
     * @return A GenericResources.
     */
    private static GenericResources getGeneric()
    {
        return generic;
    }

    /**
     * Private initializer method that sets up the generic
     * resources.
     *
     * @exception IOException, if there was an I/O problem.
     */
    private static void init()
        throws IOException
    {
        if ( fileName == null )
            throw new IOException ( "TurbineResources: fileName must not be null!" );

        init(new Configurations(new ExtendedProperties(fileName)));
    }

    /**
     * Initializer method that sets up the generic resources.
     *
     * @param confs A Configurations object.
     */
    private static void init(Configurations confs)
    {
        generic = new GenericResources (confs);
    }

    /**
     * The purpose of this method is to init the configuration
     * resource with a Properties object sent from a different system.
     * For example, a Properties sent over from Cocoon.  The reason
     * for this code is to provide a bridge between an
     * org.apache.java.util.Configurations class and an
     * org.apache.cocoon.framework.Configurations class.
     *
     * @param props A Properties object.
     */
    private static void init(Properties props)
    {
        ExtendedProperties extProperties = new ExtendedProperties();
        Enumeration e = props.keys();

        while (e.hasMoreElements())
        {
            String key = (String) e.nextElement();
            String value = (String) props.get(key);
            int commaSign = value.indexOf(',');
            if (commaSign > 0)
            {
                StringTokenizer tokenizer = new StringTokenizer(value, ",");
                Vector v = new Vector(2);
                while (tokenizer.hasMoreTokens())
                {
                    String token = tokenizer.nextToken();
                    v.addElement(token);
                }
                extProperties.put ( key, v );
            }
            else
            {
                extProperties.put ( key, value );
            }
        }
        init(new Configurations(extProperties));
    }

    /**
     * Get the Configurations that was used to define this object.
     *
     * @return A Configurations object.
     */
    public static Configurations getConfig()
    {
        return getGeneric().getConfig();
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a boolean value.
     *
     * @param name The resource name.
     * @return The value of the named resource as a boolean.
     */
    public static boolean getBoolean(String name)
    {
        return getGeneric().getBoolean (name);
    }

    /**
     * The purppose of this method is to get the configuration
     * resource with the given name as a boolean value, or a default
     * value.
     *
     * @param name The resource name.
     * @param def The default value of the resource.
     * @return The value of the named resource as a boolean.
     */
    public static boolean getBoolean(String name,
                                     boolean def)
    {
        return getGeneric().getBoolean(name, def);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a double.
     *
     * @param name The resoource name.
     * @return The value of the named resource as double.
     */
    public static double getDouble(String name)
    {
        return getGeneric().getDouble(name);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a double, or a default value.
     *
     * @param name The resource name.
     * @param def The default value of the resource.
     * @return The value of the named resource as a double.
     */
    public static double getDouble(String name,
                                   double def)
    {
        return getGeneric().getDouble(name, def);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a float.
     *
     * @param name The resource name.
     * @return The value of the resource as a float.
     */
    public static float getFloat(String name)
    {
        return getGeneric().getFloat(name);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a float, or a default value.
     *
     * @param name The resource name.
     * @param def The default value of the resource.
     * @return The value of the resource as a float.
     */
    public static float getFloat(String name,
                                 float def)
    {
        return getGeneric().getFloat(name, def);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as an Integer.
     *
     * @param name The resource name.
     * @return The value of the resource as an Integer.
     */
    public static Integer getInteger(String name)
    {
        return getGeneric().getInteger(name);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as an Integer, or a default value.
     *
     * @param name The resource name.
     * @param def The default value of the resource.
     * @return The value of the resource as an Integer.
     */
    public static Integer getInteger(String name,
                                     int def)
    {
        return getGeneric().getInteger(name, def);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as an integer.
     *
     * @param name The resource name.
     * @return The value of the resource as an integer.
     */
    public static int getInt(String name)
    {
        return getGeneric().getInt(name);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as an integer, or a default value.
     *
     * @param name The resource name.
     * @param def The default value of the resource.
     * @return The value of the resource as an integer.
     */
    public static int getInt(String name,
                             int def)
    {
        return getGeneric().getInt(name, def);
    }

    /**
     * Get the list of the keys contained in the configuration
     * repository.
     *
     * @return An Enumeration with all the keys.
     */
    public static Enumeration getKeys()
    {
        return getGeneric().getKeys();
    }

    /**
     * Get the list of the keys contained in the configuration
     * repository that match the specified prefix.
     *
     * @param prefix A String prefix to test against.
     * @return An Enumeration of keys that match the prefix.
     */
    public static Enumeration getKeys(String prefix)
    {
        Enumeration keys = getGeneric().getKeys();
        Vector matchingKeys = new Vector();
        while( keys.hasMoreElements() )
        {
            Object key = keys.nextElement();
            if( key instanceof String && ((String) key).startsWith(prefix) )
            {
                matchingKeys.addElement(key);
            }
        }
        return matchingKeys.elements();
    }

    /*

    //
    // The purpose of this method is to get an Enumeration of all the
    // keys in the resource bundle.
    //
    // @return An enumeration full of all the keys.
    //
    public static Enumeration getKeys()
    {
        return getGeneric().getKeys();
    }

    //
    // The purpose of this method is to get an Enumeration full of all
    // the keys that begin with the given prefix.
    //
    // @param prefix The prefix of the keys to retrieve.
    // @return An enumeration full of keys that begin with the given
    // prefix.
    //
    public static Enumeration getKeys(String prefix)
    {
        return getGeneric().getKeys(prefix);
    }

    */

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a long.
     *
     * @param name The resource name.
     * @return The value of the resource as a long.
     */
    public static long getLong(String name)
    {
        return getGeneric().getLong(name);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a long, or a default value.
     *
     * @param name The resource name.
     * @param def The default value of the resource.
     * @return The value of the resource as a long.
     */
    public static long getLong(String name,
                               long def)
    {
        return getGeneric().getLong(name, def);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a string.
     *
     * @param name The resource name.
     * @return The value of the resource as a string.
     */
    public static String getString(String name)
    {
        return getGeneric().getString(name);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a string, or a default value.
     *
     * @param name The resource name.
     * @param def The default value of the resource.
     * @return The value of the resource as a string.
     */
    public static String getString(String name,
                                   String def)
    {
        return getGeneric().getString(name, def);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a string array.
     *
     * @param name The resource name.
     * @return The value of the resource as a string array.
     */
    public static String[] getStringArray(String name)
    {
        return getGeneric().getStringArray(name);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a vector.
     *
     * @param name The resource name.
     * @return The value of the resource as a vector.
     */
    public static Vector getVector(String name)
    {
        return getGeneric().getVector(name);
    }

    /**
     * The purpose of this method is to get the configuration resource
     * with the given name as a vector, or a default value.
     *
     * @param name The resource name.
     * @param def The default value of the resource.
     * @return The value of the resource as a vector.
     */
    public static Vector getVector(String name,
                                   Vector def)
    {
        Vector vec = getGeneric().getVector(name);
        if ( vec == null)
            return def;
        return vec;
    }
}
