package org.apache.turbine.services.upload;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.io.*;
import java.util.*;
import javax.servlet.http.*;

// Turbine stuff.
import org.apache.turbine.services.*;
import org.apache.turbine.util.*;

/**
 * <p> This is a facade class for {@link UploadService}.
 *
 * <p> This class provides static methods that retrieve the configured
 * (in TurbineResource.properties) implementation of {@link
 * UploadService} and perform certain operations on it.  It uses
 * constants defined in {@link UploadService} interface for accessing
 * the service's properties and default values for them.
 *
 * @author <a href="mailto:Rafal.Krzewski@e-point.pl">Rafal Krzewski</a>
 * @version $Id: TurbineUpload.java,v 1.2 2000/08/31 14:12:08 gonzalo Exp $
 */
public class TurbineUpload
{
    /**
     * <p> Retrieves the value of 'automatic' property of {@link
     * UploadService}.
     *
     * @return The value of 'automatic' property of {@link
     * UploadService}.
     */
    public static boolean getAutomatic()
    {
        UploadService upload = (UploadService)TurbineServices
            .getInstance().getService(UploadService.SERVICE_NAME);
        String auto = upload.getProperties()
            .getProperty(UploadService.AUTOMATIC_KEY,
                         UploadService.AUTOMATIC_DEFAULT.toString())
            .toLowerCase();
        if(auto.equals("true") || auto.equals("yes") || auto.equals("1"))
        {
            return true;
        }
        if(auto.equals("false") || auto.equals("no") || auto.equals("0"))
        {
            return false;
        }
        return UploadService.AUTOMATIC_DEFAULT.booleanValue();
    }

    /**
     * <p> Retrieves the value of 'size.max' property of {@link
     * UploadService}.
     *
     * @return The value of 'size.max' property of {@link
     * UploadService}.
     */
    public static int getSizeMax()
    {
        UploadService upload = (UploadService)TurbineServices
            .getInstance().getService(UploadService.SERVICE_NAME);
        String sizeMax = upload.getProperties()
            .getProperty(UploadService.SIZE_MAX_KEY,
                         UploadService.SIZE_MAX_DEFAULT.toString());
        try
        {
            return Integer.parseInt(sizeMax);
        }
        catch(NumberFormatException e)
        {
            return UploadService.SIZE_MAX_DEFAULT.intValue();
        }
    }

    /**
     * <p> Retrieves the value of 'size.threshold' property of {@link
     * UploadService}.
     *
     * @return The value of 'size.threshold' property of {@link
     * UploadService}.
     */
    public static int getSizeThreshold()
    {
        UploadService upload = (UploadService)TurbineServices
            .getInstance().getService(UploadService.SERVICE_NAME);
        String sizeThreshold = upload.getProperties()
            .getProperty(UploadService.SIZE_THRESHOLD_KEY,
                         UploadService.SIZE_THRESHOLD_DEFAULT.toString());
        try
        {
            return Integer.parseInt(sizeThreshold);
        }
        catch(NumberFormatException e)
        {
            return UploadService.SIZE_THRESHOLD_DEFAULT.intValue();
        }
    }

    /**
     * <p> Retrieves the value of 'repository' property of {@link
     * UploadService}.
     *
     * @return The value of 'repository' property of {@link
     * UploadService}.
     */
    public static String getRepository()
    {
        UploadService upload = (UploadService)TurbineServices
            .getInstance().getService(UploadService.SERVICE_NAME);
        return upload.getProperties()
            .getProperty(UploadService.REPOSITORY_KEY,
                         UploadService.REPOSITORY_DEFAULT.toString());
    }

    /**
     * <p> Performs parsing the request and storing files and form
     * fields.  Default file repository is used.  This method is
     * called by the {@link ParameterParser} if automatic upload is
     * enabled.
     *
     * @param req The servlet request to be parsed.
     * @param params The ParameterParser instance to insert form
     * fields into.
     * @exception IOException If there are problems reading/parsing
     * the request or storing files.
     */
    public static void parseRequest( HttpServletRequest req,
                                     ParameterParser params )
        throws IOException
    {
        UploadService upload = (UploadService)TurbineServices
            .getInstance().getService(UploadService.SERVICE_NAME);
        String path = upload.getProperties()
            .getProperty(UploadService.REPOSITORY_KEY,
                         UploadService.REPOSITORY_DEFAULT.toString());
        upload.parseRequest(req, params, path);
    }

    /**
     * <p> Performs parsing the request and storing files and form
     * fields.  Custom file repository may be specified.  You can call
     * this method in your file upload {@link
     * org.apache.turbine.modules.Action} to if you need to specify a
     * custom directory for storing files.
     *
     * @param req The servlet request to be parsed.
     * @param params The ParameterParser instance to insert form
     * fields into.
     * @param path The location where the files should be stored.
     * @exception IOException If there are problems reading/parsing
     * the request or storing files.
     */
    public static void parseRequest( HttpServletRequest req,
                                     ParameterParser params,
                                     String path )
        throws IOException
    {
        UploadService upload = (UploadService)TurbineServices
            .getInstance().getService(UploadService.SERVICE_NAME);
        upload.parseRequest(req, params, path);
    }
}
