package org.apache.turbine.services.webmacro;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.*;
import java.io.*;

// WebMacro stuff.
import org.webmacro.broker.*;
import org.webmacro.util.*;
import org.webmacro.*;
import org.webmacro.engine.FileTemplate;

// Turbine stuff.
import org.apache.turbine.services.*;

/**
 * <p>This class provides WebMacro engine with <code>Temlates</code>.
 *
 * <p> This class was created as a replacement of
 * <code>org.webmacro.resource.TemplateProvider</code> that relies on
 * the properties of <code>WebMacroService</code> for locating the
 * templates.
 *
 * @author <a href="mailto:Rafal.Krzewski@e-point.pl">Rafal Krzewski</a>
 * @version $Id: TurbineTemplateProvider.java,v 1.3 2000/09/02 00:15:01 jonbolt Exp $
 */
public class TurbineTemplateProvider
    implements ResourceProvider
{
    /** The single resource type we provide. */
    public static String TYPE = "template";

    /** An array of supported resource types. */
    private static String[] types =  { TYPE };

    /** Expiration time of the provided resources. */
    private int cacheTime;

    /** Stored paths that are searched by this provider. */
    private String[] paths;

    /** Resource broker that manages this provider. */
    private Broker broker;

    /**
     * Returns an array of supported resource types.
     *
     * @return A String[] with the supported resource types.
     */
    public String[] getTypes()
    {
        return types;
    }

    /**
     * Returns expiration time of the provided resources.
     *
     * @return An int with the expiration time of the provided
     * resources.
     */
    public int resourceExpireTime()
    {
        return cacheTime;
    }

    /**
     * Returns the number of additional worker threads to be used by
     * the provider
     *
     * @return The number of additional worker threads to be used by
     * the provider.
     */
    public int resourceThreads()
    {
        // Provider is I/O bound, therefore spawning an extra thread
        // may be beneficial.
        return 1;
    }

    /**
     * Initializes the provider.
     *
     * @param broker The <code>ResourceBroker</code> that manages this
     * provider.
     * @exception ResourceInitException.
     */
    public void init(ResourceBroker broker)
        throws ResourceInitException
    {
        this.broker = broker;

        org.apache.turbine.util.Log.note("TurbineTemplateProvider starting!");

        String separator = System.getProperty("path.separator", ";");
        try
        {
            try
            {
                String tmp = (String)broker.getValue(Config.TYPE,
                                                     Config.TEMPLATE_CACHE);
                cacheTime = Integer.valueOf(tmp).intValue();
            }
            catch (Exception e)
            {
                // 10 minutes.
                cacheTime = 10 * 60 * 1000;
            }

            String templatePaths;

            Properties props = TurbineServices.getInstance()
                .getService(WebMacroService.SERVICE_NAME).getProperties();
            templatePaths = props.getProperty("templates");

            if(templatePaths == null)
            {
                templatePaths = (String)broker.getValue(Config.TYPE,
                                                        Config.TEMPLATE_DIR);
            }

            StringTokenizer st = new StringTokenizer(templatePaths,
                                                     separator);
            paths = new String[st.countTokens()];
            int pos=0;
            while(st.hasMoreTokens())
            {
                paths[pos++] = st.nextToken();
                org.apache.turbine.util.Log.note("TurbineTemplateProvider uses "+paths[pos-1]);
            }
        }
        catch(Exception e)
        {
            org.apache.turbine.util.Log.error("TurbineTemplateProvider failed to initialize");
            org.apache.turbine.util.Log.error(e);
            throw new ResourceInitException("failed to initialize");
        }
        org.apache.turbine.util.Log.note("TurbineTemplateProvider started!");
    }

    /**
     * Shuts down the provider.
     */
    public void destroy()
    {
        // Do nothing.
    }

    /**
     * Retrieves a resource.
     *
     * @param request A <code>RequestResourceEvent</code>.
     * @exception NotFoundException.
     * @exception InterruptedException.
     */
    public void resourceRequest(RequestResourceEvent request)
        throws NotFoundException,
               InterruptedException
    {

        Template t = findTemplate(request.getName());
        if (t == null)
        {
            // Let other providers handle it.
            return;
        }
        try
        {
            // Announce that we have it.
            request.set(t);
        }
        catch (Exception e)
        {
            // Ignore.
        }
    }

    /**
     * Creates a new resource (not implemented).
     *
     * @param create A <code>CreateResourceEvent</code>.
     * @exception NotFoundException.
     * @exception InterruptedException.
     */
    public void resourceCreate(CreateResourceEvent create)
        throws NotFoundException,
               InterruptedException
    {
        // Unsupported.
    }

    /**
     * Deletes a resource (not implemented).
     *
     * @param delete A <code>ResourceEvent</code>.
     * @return Always false.
     */
    public boolean resourceDelete(ResourceEvent delete)
    {
        // Unsupported.
        return false;
    }

    /**
     * Saves a resource to permanet storage (not implemented).
     *
     * @param save A <code>ResourceEvent</code>.
     * @return Always false.
     */
    public boolean resourceSave(ResourceEvent save)
    {
        // Unsupported.
        return false;
    }

    /**
     * Locates a template file, and instantiates <code>Template</code>
     * object.
     *
     * @param name the name of the template to be located
     * @return Instantiated <code>Template</code> object.
     */
    private Template findTemplate( String name )
    {
        for(int i=0; i<paths.length; i++)
        {
            Template template;
            File file = new File(paths[i], name);
            if(file.canRead())
            {
                try
                {
                    template = new FileTemplate(broker, file);
                    template.parse();
                    return template;
                }
                catch(Exception e)
                {
                    org.apache.turbine.util.Log.error("TurbineTemplateProvider failed to load template "+file.getPath());
                    org.apache.turbine.util.Log.error(e);
                }
            }
        }
        return null;
    }
}
