package org.apache.turbine.services.webmacro;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Stuff
import java.util.*;
import java.io.*;

// ServletConfig
import javax.servlet.*;

// WebMacro Stuff
import org.webmacro.*;
import org.webmacro.util.*;
import org.webmacro.servlet.*;
import org.webmacro.broker.*;

// Turbine Stuff
import org.apache.turbine.services.*;
import org.apache.turbine.services.resources.*;
import org.apache.turbine.util.*;
import org.apache.turbine.util.webmacro.*;

/**
 * This is a Service that can process WebMacro templates from within a
 * Turbine Screen.  Here's an example of how you might use it from a
 * screen:<br>
 *
 * <code>
 * WebContext context = WebMacro.getContext(data);<br>
 * context.put("message", "Hello from Turbine!");<br>
 * String results = WebMacro.handleRequest(context,"helloWorld.wm");<br>
 * data.getPage().getBody().addElement(results);<br>
 * </code>
 *
 * @author <a href="mailto:mbryson@mont.mindspring.com">Dave Bryson</a>
 * @author <a href="mailto:krzewski@e-point.pl">Rafal Krzewski</a>
 * @version $Id: TurbineWebMacroService.java,v 1.7 2000/09/04 00:56:18 jonbolt Exp $
 */
public class TurbineWebMacroService
    extends TurbineBaseService
    implements WebMacroService
{
    /** WebMacro Engine.*/
    private org.webmacro.WebMacro wm = null;

    /** WebContext - used by WebMacro. */
    private org.webmacro.servlet.WebContext wc = null;

    /** WebMacro Broker. */
    private org.webmacro.Broker broker = null;

    /**
     * Constructor.
     *
     * @exception Exception, a generic exception.
     */
    public TurbineWebMacroService()
        throws Exception
    {
    }

    /**
     * Called during Turbine.init()
     *
     * @param config A ServletConfig.
     */
    public void init( ServletConfig config )
    {
        try
        {
            org.apache.turbine.util.Log.note ( "WebMacroService early init()....starting!");
            initWebMacro(config);
            org.apache.turbine.util.Log.note ( "WebMacroService early init()....finished!");
        }
        catch (Exception e)
        {
            org.apache.turbine.util.Log.error ( "Cannot initialize TurbineWebMacroService!" );
            org.apache.turbine.util.Log.error (e);
        }
        // we don't call setInit(true) yet, because we want init() to be called also
    }

    /**
     * Called the first time the Service is used.
     */
    public void init()
    {
        try
        {
            org.apache.turbine.util.Log.note ( "WebMacroService init()....starting!");
            initWebContext();
            setInit(true);
            org.apache.turbine.util.Log.note ( "WebMacroService init()....finished!");
        }
        catch (Exception e)
        {
            org.apache.turbine.util.Log.error ( "Cannot initialize TurbineWebMacroService!" );
            org.apache.turbine.util.Log.error (e);
        }
    }

    /**
     * Create an empty WebContext object.
     *
     * @return An empty WebContext object.
     */
    public WebContext getContext()
    {
        return new WebContext(broker);
    }

    /**
     * Create a WebContext from the RunData object.  Adds a pointer to
     * the RunData object to the WC so that RunData is available in
     * the templates.
     *
     * @param data The Turbine RunData object.
     * @return A clone of the WebContext needed by WebMacro.
     */
    public WebContext getContext(RunData data)
    {
        WebContext newWC = wc.newInstance(data.getRequest(),
                                          data.getResponse());
        newWC.put ( "data", data );
        newWC.put ( "link", new WebMacroLink(data) );
        newWC.put ( "page", new WebMacroPageAttributes(data) );
        newWC.put ( "formatter", new WebMacroFormatter(newWC) );
        return newWC;
    }

    /**
     * Process the request and fill in the template with the values
     * you set in the WebContext.
     *
     * @param wc A WebContext.
     * @param filename A String with the filename of the template.
     * @return The process template as a String.
     * @exception Exception, a generic exception.
     */
    public String handleRequest(WebContext wc,
                                String filename)
        throws Exception
    {
        String results = null;
        StringWriter sw = null;
        try
        {
            sw = new StringWriter();
            handleRequest (wc,filename,sw);
            results = sw.toString();
        }
        finally
        {
            if (sw != null)
                sw.close();
        }
        return results;
    }

    /**
     * Process the request and fill in the template with the values
     * you set in the WebContext.
     *
     * @param wc A WebContext.
     * @param filename A String with the filename of the template.
     * @param out A Writer where we will write the process template as
     * a String.
     * @exception Exception, a generic exception.
     */
    public void handleRequest(WebContext wc,
                              String filename,
                              Writer out)
        throws Exception
    {
        Template template = getTemplate(filename);
        template.write(out, wc);
    }

    /**
     * Return a template from WebMacro.
     *
     * @param filename A String with the name of the template.
     * @return A Template.
     * @exception NotFoundException.
     */
    public Template getTemplate(String filename)
        throws NotFoundException
    {
        return wm.getTemplate(filename);
    }

    /**
     * This method returns the WebMacro object which will be used to
     * load, access, and manage the Broker.  The WM first tries to
     * locate the WebMacro properties file in the location specified
     * by the 'services.TurbineWebmacroService.properties' key in the
     * TurbineResource file.  If the key is not set, it will search
     * the classpath looking for the file.
     *
     * @param config A ServletConfig.
     * @exception InitException.
     */
    private void initWebMacro( ServletConfig config )
        throws InitException
    {
        ServletContext context = config.getServletContext();

        Properties props = getProperties();

        String WMProperties = props.getProperty("properties",null);
        String WMTemplates = props.getProperty("templates",null);
        String WMTemplateProvider =
            props.getProperty("templates.provider",null);

        //debug
        org.apache.turbine.util.Log.note("WebMacroService properties:");
        Enumeration keys = props.propertyNames();
        while(keys.hasMoreElements())  {
            String key = (String)keys.nextElement();
            org.apache.turbine.util.Log.note(key+":"+props.getProperty(key));
        }
        //debug


        if(WMProperties != null)
        {
            // Check if getRealPath is working.
            if(context.getRealPath("/") != null)
            {
                // Make properties path relative to our web
                // application root.
                WMProperties = context.getRealPath(WMProperties);
            }

            //debug
            org.apache.turbine.util.Log.note("Creating WM object using "+WMProperties);
            //debug

            wm = new WM(WMProperties);
        }
        else
        {
            // Not specified: fallback to WM style properties search
            // through the classpath.

            //debug
            org.apache.turbine.util.Log.note("Creating WM object using traditional method");
            //debug

            wm = new WM();
        }
        broker = wm.getBroker();


        if(WMTemplates == null)
        {
            // Templates path is not specified in
            // TrubineResources.properties fallback to WM default
            // behaviour.
            return;
        }

        try
        {
            // If getRealPath is working, transform paths to be webapp
            // root relative.
            if(context.getRealPath("/") != null)
            {
                String pathSep = System.getProperty("path.separator");
                StringTokenizer st = new StringTokenizer(WMTemplates,pathSep);
                StringBuffer buf = new StringBuffer();
                while(st.hasMoreTokens())
                {
                    buf.append(context.getRealPath(st.nextToken()));
                    if(st.hasMoreTokens())
                    {
                        buf.append(pathSep);
                    }
                }
                WMTemplates = buf.toString();
            }

            // Store the converted path in service properties for
            // Turbine based providers.
            props.put("templates", WMTemplates);

            // Create a resource map to access WebMacro configuration
            // entries.
            ResourceMap map = ((ResourceBroker)broker).get(Config.TYPE);

            // Update templates path in WebMacro config broker, for
            // WebMacro based providers.
            map.put("TemplatePath",WMTemplates);

            // NOTE! This requires modified WebMacro, to allow mutable
            // configuration entries.  A patch is available at
            // http://suokong.e-point.pl/turbine.
            //
            // Rafal

            // Instantiate template provider.
            ResourceProvider provider =
                (ResourceProvider)Class.forName(WMTemplateProvider).newInstance();

            // Add the provider to the broker.
            ((ResourceBroker)broker).join(provider);

            // Note that WebMacro.properties shouldn't contain
            // org.webmacro.resource.TemplateProvider in the Providers
            // entry, because then we would have two TemplateProviders
            // in the system, one of wich would be non functional and
            // would waste cycles during every request.
            //
            // It seems reasonable to me, that there should be a
            // possibility to ask the broker if it knows any providers
            // of a specified type of resource.  This would allow us
            // to notify the user that there is a non-functional
            // provider in the system, that should be removed from the
            // config file.  Better yet, if the browser supported
            // enumerating providers of a specified type of resource,
            // we could locate such provider, and ask it to terminate.
            //
            // The broker class in the WM snapshot I have doesn't
            // support either of these options, and I don't feel
            // competent enough to implement it.
            //
            // Rafal
        }
        catch(Exception e)
        {
            org.apache.turbine.util.Log.error("Failed to set up WebMacro templates:");
            org.apache.turbine.util.Log.error(e);
        }
    }

    /**
     * This method must return a cloneable WebContext which can be
     * cloned for use in responding to individual requests.  Each
     * incoming request will receive a clone of the returned object as
     * its context.  The default implementation is to return a new
     * WebContext.
     *
     * @exception InitException.
     */
    private void initWebContext()
        throws InitException
    {
        wc = getContext();
    }
}
