package org.apache.turbine.tools;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes.
import java.io.*;

// Turbine stuff.
import org.apache.turbine.util.db.map.TableMap;
import org.apache.turbine.util.db.map.ColumnMap;
import org.apache.turbine.util.db.map.DatabaseMap;
import org.apache.turbine.util.db.map.MapBuilder;


// Ant Stuff
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

/**
 * Generate a SQL script for PostgreSQL from a DatabaseMap.
 *
 * <p>This class is a base class for generating a SQL script that
 * builds a database structure for PostgreSQL.  The source DatabaseMap
 * is given through the abstract getDatabaseMap() method, which
 * should be implemented in descendent.
 *
 * <p>String is mapped as text, Date as timestamp, Hashtable as oid
 * (BLOB) and Integer as integer.
 *
 * <p>This is a Ant build task that has one parameter: sqlfile.
 *
 * @author <a href="mailto:rkoenig@chez.com">Regis Koenig</a>
 * @version $Id: BaseGeneratePostgresSchemaTask.java,v 1.2 2000/08/31 20:09:36 gonzalo Exp $
 */
public abstract class BaseGeneratePostgresSchemaTask
{
    private String sqlfile;

    /** Current DatabaseMap. */
    private DatabaseMap dbmap;

    /** Current TableMap. */
    private TableMap tmap;

    /** Current columnMap. */
    private ColumnMap cmap;

    public static String INTEGER_MAP = "integer";
    public static String DATE_MAP = "timestamp";
    public static String HASHTABLE_MAP = "oid";
    public static String STRING_MAP = "text";
    public static String AUTOINCREMENT_MAP = "serial";

    /**
     * Get the source DatabaseMap.
     *
     * <p>This method must be implemented in descendents and should
     * return the source DabaseMap for SQL generation.
     *
     * @return A DatabaseMap.
     * @exception Exception, a generic exception.
     */
    public abstract DatabaseMap getDatabaseMap()
        throws Exception;

    /**
     * Ant execute method.
     *
     * @exception BuildException.
     */
    public void execute()
        throws BuildException
    {
        try
        {
            dbmap = getDatabaseMap();
            createSchema(dbmap);
        }
        catch (Exception e)
        {
            e.printStackTrace();
            throw new BuildException(e.toString());
        }
    }

    /**
     * Write the schema.
     *
     * @param dbmap A DatabaseMap.
     * @exception IOException.
     */
    public void createSchema(DatabaseMap dbmap)
        throws IOException
    {
        Writer wri = new FileWriter(getSqlfile());
        PrintWriter pw = new PrintWriter(wri);
        TableMap[] tMaps = dbmap.getTables();
        for (int i=0; i<tMaps.length; i++ )
        {
            tmap = tMaps[i];
            writeTable( tmap, pw );
        }
        wri.close();
    }

    /**
     * Write a per-table section.
     *
     * <p>Write <code>DROP TABLE</code> and <code>CREATE TABLE</code>
     * statements.
     *
     * @param tmap A TableMap.
     * @param out A PrintWriter.
     * @exception IOException.
     */
    private void writeTable(TableMap tmap,
                            PrintWriter out)
        throws IOException
    {
        out.println("\n\n-- table " + tmap.getName() );
        out.println("DROP TABLE "+tmap.getName()+";");

        ColumnMap[] cMaps = tmap.getColumns();
        if(tmap.getPrimaryKeyMethod().equals(TableMap.AUTOINCREMENT))
            for(int i=0; i<cMaps.length; i++)
                if( getType(cMaps[i]).equals(AUTOINCREMENT_MAP))
                    out.println("DROP SEQUENCE "+
                                tmap.getName() + "_" +
                                cMaps[i].getColumnName() + "_seq;");

        out.println("CREATE TABLE "+tmap.getName()+" (");

        for( int i=0; i< cMaps.length; i++)
        {
            cmap = cMaps[i];
            writeColumn(cmap, out, (i==cMaps.length-1) );
        }
        out.println(");");
    }

    /**
     * Write the column information (type, not null, ...).
     *
     * @param cmap A ColumnMap.
     * @param out A PrintWriter.
     * @param last True if this is the last column.
     * @exception IOException.
     */
    private void writeColumn(ColumnMap cmap,
                             PrintWriter out,
                             boolean last)
        throws IOException
    {
        out.print("      ");
        out.print(cmap.getColumnName());
        out.print(" "+getType(cmap));

        if( cmap.isForeignKey() )
        {
            out.print(" REFERENCES "+cmap.getRelatedTableName()+" ( "+
                      cmap.getRelatedColumnName() +
                      " ) DEFERRABLE INITIALLY DEFERRED");
        }
        else if( cmap.isPrimaryKey() )
            out.print(" PRIMARY KEY");
        else if( cmap.isNotNull() )
            out.print(" NOT NULL");

        if(!last)
            out.println(",");
        else
            out.println("");
    }

    /**
     * Get the SQL type for a given column.
     *
     * @param cmap A ColumnMap.
     * @return A String with the SQL column type.
     */
    private String getType(ColumnMap cmap)
    {
        Object obj = cmap.getType();
        if( obj instanceof Integer )
        {
            if( cmap.isPrimaryKey() && (!cmap.isForeignKey()) )
            {
                String pkm = tmap.getPrimaryKeyMethod();
                if(pkm.equals( TableMap.AUTOINCREMENT ))
                   return AUTOINCREMENT_MAP;
            }
            return INTEGER_MAP;
        }
        else if( obj instanceof java.util.Date )
            return DATE_MAP;
        else if( obj instanceof java.util.Hashtable )
            return HASHTABLE_MAP;
        else
            return STRING_MAP;
    }

    /**
     * Get the value of sqlfile.
     *
     * @return Value of sqlfile.
     */
    public String getSqlfile()
    {
        return sqlfile;
    }

    /**
     * Set the value of sqlfile.
     *
     * @param v Value to assign to sqlfile.
     */
    public void setSqlfile(String  v)
    {
        this.sqlfile = v;
    }
}
