package org.apache.turbine.tools;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes.
import java.sql.*;
import java.io.*;
import java.util.*;

// Turbine stuff.
import org.apache.turbine.util.db.map.*;

/**
 * A utility class that builds a DatabaseMap from metadata of a JDBC
 * database.  You provide the database driver, url, user and password
 * and the class provides an instance of DatabaseMap.
 *
 * <p> Please note that database metadata is an optional feature and
 * therefore some databases do not implement it.  To use this class
 * you will need a database and driver that support metadata.
 *
 * @author <a href="mailto:leon@opticode.co.za">Leon Messerschmidt</a>
 * @version $Id: JDBCSchemaToDatabaseMap.java,v 1.6 2000/08/31 20:09:37 gonzalo Exp $
 */
public class JDBCSchemaToDatabaseMap
{
    private String dbUrl = "";
    private String dbDriver = "";
    private String dbUser = "";
    private String dbPassword = "";

    /**
     * Get the database URL.
     *
     * @return A String.
     */
    public String getDbUrl ()
    {
        return dbUrl;
    }

    /**
     * Get the database driver.
     *
     * @return A String.
     */
    public String getDbDriver ()
    {
        return dbDriver;
    }

    /**
     * Get the username for the database.
     *
     * @return A String.
     */
    public String getDbUser ()
    {
        return dbUser;
    }

    /**
     * Get the password to log in.
     *
     * @return A String.
     */
    public String getDbPassword ()
    {
        return dbPassword;
    }

    /**
     * Set the database URL.
     *
     * @param v A String.
     */
    public void setDbUrl (String v)
    {
        dbUrl = v;
    }

    /**
     * Set the database driver.
     *
     * @param v A String.
     */
    public void setDbDriver (String v)
    {
        dbDriver = v;
    }

    /**
     * Set the username for the database.  By default the user is an
     * empty string.
     *
     * @param v A String.
     */
    public void setDbUser (String v)
    {
        dbUser = v;
    }

    /**
     * Set the password to log in.  By default the password is an
     * empty string.
     *
     * @param v A String.
     */
    public void setDbPassword (String v)
    {
        dbPassword = v;
    }

    /**
     * Generates a database map from a JDBC.
     *
     * @param mapName Name of the map.
     * @return A DatabaseMap.
     * @exception Exception, a generic exception.
     */
    public DatabaseMap generateMap(String mapName)
        throws Exception
    {
        // Load the Interbase Driver.
        Class.forName(getDbDriver());

        // Attempt to connect to a database.
        Connection con = DriverManager.getConnection(getDbUrl(),
                                                     getDbUser(),
                                                     getDbPassword());
        // Get the database Metadata.
        DatabaseMetaData dbMetaData = con.getMetaData();

        // The database map.
        DatabaseMap dbMap = new DatabaseMap(mapName);
        Vector tableList = getTableNames(dbMetaData);

        for (int i = 0;i < tableList.size(); i++)
        {
            // Add Table.
            String curTable = (String) tableList.elementAt(i);
            dbMap.addTable(curTable);

            // Add Columns.
            TableMap tblMap = dbMap.getTable(curTable);

            Vector columns = getColumns(dbMetaData,curTable);
            Vector primKeys = getPrimaryKeys(dbMetaData,curTable);
            Vector forgnKeys = getForeignKeys(dbMetaData,curTable);

            for (int j = 0;j < columns.size();j++)
            {
                Vector v = (Vector) columns.elementAt(j);
                String name = (String) v.elementAt(0);
                Integer type = (Integer) v.elementAt(1);

                if (type.intValue() == Types.BIGINT ||
                    type.intValue() == Types.INTEGER ||
                    type.intValue() == Types.SMALLINT ||
                    type.intValue() == Types.TINYINT)
                {
                    tblMap.addColumn(name, new Integer(0));
                }

                else if (type.intValue() == Types.CHAR ||
                         type.intValue() == Types.LONGVARCHAR ||
                         type.intValue() == Types.VARCHAR)
                {
                    tblMap.addColumn(name, "");
                }

                else if (type.intValue() == Types.DATE ||
                         type.intValue() == Types.TIME ||
                         type.intValue() == Types.TIMESTAMP)
                {
                    tblMap.addColumn(name, new java.util.Date());
                }
                else
                {
                    tblMap.addColumn(name, "");
                }

            }

            // Set the primary keys.
            for (int k = 0;k < primKeys.size();k++)
            {
                String curPrimaryKey = (String) primKeys.elementAt(k);
                ColumnMap clMap = tblMap.getColumn(curPrimaryKey);
                clMap.setPrimaryKey(true);
            }

            // Set the foreign keys.
            for (int l = 0;l < forgnKeys.size();l++)
            {
                Vector v = (Vector)forgnKeys.elementAt(l);
                ColumnMap clMap = tblMap.getColumn((String)v.elementAt(1));
                if (clMap != null)
                {
                    clMap.setForeignKey((String)v.elementAt(0),
                                        (String)v.elementAt(1));
                }
            }
        }
        return dbMap;
    }

    /**
     * Get all the table names in the current database that are not
     * system tables.
     *
     * @param dbMeta A DatabaseMetaData.
     * @return A Vector with table names.
     * @exception SQLException.
     */
    public Vector getTableNames(DatabaseMetaData dbMeta)
        throws SQLException
    {
        System.out.println ("Getting tables");

        String[] types = {"TABLE"};

        ResultSet tableNames = dbMeta.getTables("",null,"%",types);
        Vector tables = new Vector();
        while (tableNames.next())
        {
            String name = tableNames.getString(3);
            String type = tableNames.getString(4);
            tables.addElement(name);
        }
        return tables;
    }


    /**
     * Retrieves all the column names and types for a given table from
     * JDBC metadata.  It returns a Vector of Vectors.  Each element
     * of the returned vector is a vector with element 0 a String
     * object for the column name and element 1 an Integer object for
     * the column type.  You can use the Types class to decode the
     * column type.
     *
     * @param dbMeta A DatabaseMetaData.
     * @param tableName A String with the name of the table.
     * @return A Vector with Vectors with the column names and types.
     * @exception SQLException.
     */
    public Vector getColumns(DatabaseMetaData dbMeta,
                             String tableName)
        throws SQLException
    {
        ResultSet columnSet = dbMeta.getColumns("",null,tableName,null);
        Vector columns = new Vector();
        while (columnSet.next())
        {
            String name = columnSet.getString(4);
            Integer sqlType = new Integer(columnSet.getString(5));
            Vector v = new Vector();
            v.addElement (name);
            v.addElement (sqlType);
            columns.addElement (v);
        }
        return columns;
    }

    /**
     * Retrieves a list of primary key columns for a given table.
     *
     * @param dbMeta A DatabaseMetaData.
     * @param tableName A String with the name of the table.
     * @return A Vector with primary key columns.
     * @exception SQLException.
     */
    public Vector getPrimaryKeys(DatabaseMetaData dbMeta,
                                 String tableName)
        throws SQLException
    {
        ResultSet primaryKeys = dbMeta.getPrimaryKeys("",null,tableName);
        Vector keys = new Vector();
        while (primaryKeys.next())
        {
            keys.addElement(primaryKeys.getString(4));
        }
        return keys;
    }


    /**
     * Retrieves all the foreign key and table names for a given table
     * from JDBC metadata.  It returns a Vector of Vectors.  Each
     * element of the returned vector is a vector with element 0 a
     * String object for the table name and element 1 a Integer object
     * for the column name.
     *
     * @param dbMeta A DatabaseMetaData.
     * @param tableName A String with the name of the table.
     * @return A Vector with Vectors with the foreign keys.
     * @exception SQLException.
     */
    public Vector getForeignKeys(DatabaseMetaData dbMeta,
                                 String tableName)
        throws SQLException
    {
        ResultSet foreignKeys = dbMeta.getImportedKeys("",null,tableName);
        Vector keys = new Vector();

        if (foreignKeys != null)
        {
            while (foreignKeys.next())
            {
                String table = foreignKeys.getString(3);
                String column = foreignKeys.getString(4);
                Vector v = new Vector();
                v.addElement (table);
                v.addElement (column);
                keys.addElement (v);
            }
        }

        return keys;
    }

    /**
     * For testing purposes only.
     *
     * @param args a String[] with the command line arguments.
     * @exception Exception, a generic exception.
     */
    static public void main (String[] args)
        throws Exception
    {
        JDBCSchemaToDatabaseMap gen = new JDBCSchemaToDatabaseMap();
        gen.setDbUrl ("");
        gen.setDbDriver ("");
        gen.setDbPassword ("");
        gen.setDbUser ("");

        BufferedReader br = new BufferedReader (new InputStreamReader (System.in));

        DatabaseMap dbMap = gen.generateMap ("MapName");
        TableMap[] tables = dbMap.getTables();
        for (int i=0; i<tables.length; i++)
        {
            TableMap table = tables[i];
            ColumnMap[] columns = table.getColumns();
            for (int j=0; j<columns.length; j++)
            {
                ColumnMap c = columns[j];
                System.out.print ("   "+c.getColumnName()+"   "+c.getType().getClass()+"   ");
                if (c.isPrimaryKey()) System.out.print ("*");
                if (c.isForeignKey()) System.out.print ("^");
                System.out.println();
            }

            System.out.println (table.getName());
            br.readLine();
        }
    }
}
