package org.apache.turbine.tools.torque;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java stuff.
import java.io.*;
import java.util.*;

// Turbine stuff.
import org.apache.turbine.util.db.map.*;

/**
 * This class will read in a MySQL schema and attempt to create a
 * populated DatabaseMap from that schema.  The usefulness of this
 * class is then to take that DatabaseMap and output your MapBuilder
 * classes.  Right now, the only example of outputting the MapBuilder
 * classes is in the <a href="http://scarab.tigris.org"/>Scarab</a>
 * Project distribution.  You also need to write your MySQL schema in
 * a format that this parser can read.  For an example of this format,
 * please look at the MySQL schema for the Scarab Project.
 * Essentially, this format is a format that is produced by the
 * ERStudio commercial ER design software for an Oracle Schema and
 * then converted to be MySQL compliant with a few global
 * find/replaces in the file.  It should be pretty easy (ie: it should
 * work, but isn't tested) to also use this same class for an ERStudio
 * generated Oracle *.sql file as well because of the way that I coded
 * things.
 *
 * <p>The reason for this lack of abstraction is because it is
 * generally very project specific.  If someone would like to develop
 * something that is less project specific, that would be appreciated.
 * Hopefully though, with this class, you should be able to get an
 * understanding of what it takes to generate a parser to convert a
 * *.sql file into an in memory DatabaseMap representation.
 *
 * <p> Example usage is to simply pass in the name of the DatabaseMap
 * you want to create and the FileReader path to the file.  As a
 * result, you should get a fully populated DatabaseMap.
 *
 * @author <a href="mailto:jvanzyl@periapt.com">Jason van Zyl</a>
 * @author <a href="mailto:jon@latchkey.com">Jon S. Stevens </a>
 * @version $Id: SQLToDatabaseMap.java,v 1.3 2000/08/31 22:31:40 gonzalo Exp $
 */
public class SQLToDatabaseMap
{
    // Elements.
    private static final String COLUMN = "column";
    private static final String TABLE = "table";
    private static final String DATABASE = "database";

    // Atrributes global.
    private static final String NAME = "name";

    // Attributes database.
    private static final String CONNECTION_POOL = "connectionPool";

    // Attributes column.
    private static final String PRIMARY_KEY = "primaryKey";
    private static final String TYPE = "type";
    private static final String NOTNULL = "notNull";
    private static final String FOREIGN_TABLE_NAME = "foreignPKTable";
    private static final String FOREIGN_COLUMN_NAME = "foreignPKColumn";

    // Attributes column value.
    private static final String PK_AUTOINCREMENT = "autoIncrement";
    private static final String PK_NONE = "none";
    private static final String PK_SEQUENCE = "sequence";
    private static final String PK_IDBROKERTABLE = "idBroker";

    // Types value.
    private static final String TYPE_INTEGER = "integer";
    private static final String TYPE_DATE = "date";
    private static final String TYPE_HASHTABLE = "hashtable";
    private static final String TYPE_STRING = "string";

    /**
     * Pass in a FileReader which contains the path to the *.sql file.
     * This method will return the DatabaseMap fully populated.
     * Please see the class documentation for more details.  Also, as
     * always, read the source code.  It is pretty well documented.
     *
     * @param mapName A String with the name of the map.
     * @param fr A FileReader.
     * @return A DatabaseMap.
     * @exception Exception, a generic exception.
     */
    public static DatabaseMap generateMap(String mapName,
                                          FileReader fr)
        throws Exception
    {
        /** Name of the table. */
        String tableName = null;

        /** Our database map. */
        DatabaseMap dbMap = new DatabaseMap(mapName);

        /** A buffered reader to loop over things line by line. */
        BufferedReader br = new BufferedReader(fr);

        /** The current line. */
        String line = null;

        /** Are we in a create statement? */
        boolean inCreate = false;

        /** The data in the create statement. */
        Vector columndata = new Vector();

        /** Loop over every line in the file. */
        while ((line = br.readLine()) != null)
        {
            // Ignore crap we don't need.
            if (line == null ||
                line.length() == 0 ||
                line.startsWith("#") ||
                line.startsWith("//") ||
                line.startsWith(")") ||
                line.startsWith("--") )
                continue;

            // Found end of table creation; now parse data into a
            // TableMap/ColumnMap.
            if (line.indexOf(";") != -1 )
            {
                parseColumnData(dbMap, tableName, columndata);
                columndata = new Vector();
                inCreate = false;
                continue;
            }

            // Found beginning of CREATE TABLE.
            if (line.startsWith ("CREATE TABLE "))
            {
                inCreate = true;
                tableName = getTableName(line);
                continue;
            }

            // We are in CREATE TABLE.
            if (inCreate)
            {
                columndata.addElement ( line );
            }
        }
        return dbMap;
    }

    /**
     * Parses the vector of column data into the DatabaseMap.
     *
     * @param dbMap A DatabaseMap.
     * @param tableName A String with the name of the table.
     * @param columndata A Vector with the column data.
     */
    private static void parseColumnData(DatabaseMap dbMap,
                                        String tableName,
                                        Vector columndata)
    {
        dbMap.addTable(tableName);
        TableMap tmap = dbMap.getTable(tableName);

        // Go over each column line.
        String[] fkeyLines = null;

        // Loop over all the data in the create statement.
        for (Enumeration e = columndata.elements() ; e.hasMoreElements() ;)
        {
            // Grab a line.
            String line = (String)e.nextElement();
            if (line == null)
                continue;
            else
                // Make it uppercase.
                line = line.toUpperCase();

            // Make sure the line doesn't start with any of these (ie:
            // column creation). The order of the if tree is
            // important.
            if (line.indexOf("PRIMARY KEY") == -1 &&
                line.indexOf("FOREIGN KEY") == -1 &&
                line.indexOf("REFERENCES ") == -1)
            {
                StringTokenizer st = new StringTokenizer(line," ");
                String columnName = st.nextToken();
                String columnType = st.nextToken();

                // Strip comma off INTEGER,
                if (columnType.endsWith(","))
                    columnType = columnType.substring(0,
                                                      columnType.length()-1);
                // Strip (3) off INTEGER(3)
                if (columnType.endsWith(")"))
                    columnType = columnType.substring(0,
                                                      columnType.indexOf("("));

                columnMaker(tmap, columnName.trim(), columnType.trim(), "");
            }

            // The primary key lines.
            else if (line.indexOf("PRIMARY KEY") != -1)
            {
                // Grab the column names and get the columnmap and
                // mark it as primary.
                String[] keys = parsePrimaryKey(line);
                for (int i=0; i<keys.length;i++ )
                {
                    ColumnMap cmap = tmap.getColumn(keys[i]);
                    if (cmap == null )
                        continue;
                    cmap.setPrimaryKey(true);
                }
            }

            // The foreign key lines.
            else if (line.indexOf("FOREIGN KEY") != -1)
            {
                // Grab the column names into a String[] for later
                // processing in the References else if below.
                fkeyLines = parsePrimaryKey(line);
            }

            // The references lines.
            else if (line.indexOf("REFERENCES ") != -1 &&
                     fkeyLines != null &&
                     fkeyLines.length > 0)
            {
                // Go over the foreign keys and then make the
                // references associations.
                for (int i=0; i<fkeyLines.length;i++ )
                {
                    ColumnMap cmap = tmap.getColumn(fkeyLines[i]);
                    if (cmap == null )
                        continue;
                    String tblName = parseReferencesGetTableName(line);
                    cmap.setForeignKey(tblName,fkeyLines[i]);
                }
                fkeyLines = null;
            }
        }
    }

    /**
     * Parse a "primary key".
     *
     * <br>Input: PRIMARY KEY (FOO,BAR),
     * <br>Output: FOO    BAR
     *
     * @param line A String with the line.
     * @return A String[].
     */
    static final String[] parsePrimaryKey(String line)
    {
        // Get FOO,BAR
        String foo = getForeignKeyColumnName(line);
        StringTokenizer st = new StringTokenizer(foo,",");
        String[] tokens = new String[st.countTokens()];
        int counter = 0;

        // Get FOO and then BAR into a String[].
        while (st.hasMoreTokens())
        {
            tokens[counter] = st.nextToken();
            counter++;
        }
        return tokens;
    }

    /**
     * Parse a "references table".
     *
     * <br>Input: REFERENCES TABLE_NAME(COLUMN_NAME),
     * <br>Output: TABLE_NAME    COLUMN_NAME
     *
     * @param refline A String with the line.
     * @return A String.
     */
    static final String parseReferencesGetTableName(String refline)
    {
        // Skip over any pre-white space to the REFERENCES text, then
        // skip over that text...
        return refline.substring (refline.indexOf("REFERENCES") +
                                  "REFERENCES ".length(),
                                  refline.indexOf("("));
    }

    /**
     * Parse a "foreign key".
     *
     * <br>Input: FOREIGN KEY (FOO,BAR)
     * <br>Output: FOO,BAR
     *
     * @param fkeyline A String with the line.
     * @return A String.
     */
    static final String getForeignKeyColumnName(String fkeyline)
    {
        return fkeyline.substring (fkeyline.indexOf("(") +
                                   1,
                                   fkeyline.indexOf(")"));
    }

    /**
     * Parses out the table name from the CREATE TABLE statement.
     *
     * @param line A String with the line.
     * @return A String.
     */
    static final String getTableName(String line)
    {
        String tmp = null;
        StringTokenizer st = new StringTokenizer(line," ");
        while (st.hasMoreTokens())
        {
            String token = st.nextToken();
            if (! token.equalsIgnoreCase("TABLE"))
            {
                // Case for a "CREATE TABLE FOO(" and remove the '('.
                if (token.endsWith("("))
                {
                    tmp = token.substring(0,token.length()-1);
                }
                else
                {
                    tmp = token;
                }
            }
        }
        return tmp;
    }

    /**
     * Adds the appropriate column.
     *
     * @param tm A TableMap.
     * @param name A String.
     * @param type A String.
     * @param notNull A String.
     */
    static void columnMaker(TableMap tm,
                            String name,
                            String type,
                            String notNull)
    {
        if ( type.equalsIgnoreCase(TYPE_STRING) ||
             type.indexOf("CHAR") != -1 )
        {
            tm.addColumn(name, "");
        }
        else if ( type.equalsIgnoreCase(TYPE_HASHTABLE) )
        {
            tm.addColumn(name, new Hashtable(1));
        }
        else if ( type.equalsIgnoreCase(TYPE_INTEGER) ||
                  type.indexOf("INTEGER") != -1 ||
                  type.indexOf("NUMBER") != -1 )
        {
            tm.addColumn(name, new Integer(0));
        }
        else if ( type.equals(TYPE_DATE) ||
                  type.indexOf("TIMESTAMP") != -1 ||
                  type.indexOf ("DATETIME") != -1 )
        {
            tm.addColumn(name, new Date());
        }
        else
        {
            tm.addColumn(name, "");
        }
        if( notNull != null && notNull.equalsIgnoreCase("true") )
            tm.getColumn(name).setNotNull(true);
        else
            tm.getColumn(name).setNotNull(false);
    }
}
