package org.apache.turbine.tools.torque;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java stuff.
import java.io.*;
import java.util.*;
import java.text.*;

/**
 * Wrapper class around all the classes used to produce SQL, and
 * object model from an XML schema.
 *
 * @author <a href="mailto:jvanzyl@periapt.com">Jason van Zyl</a>
 * @author <a href="mailto:celkins@scardini.com">Christopher Elkins</a>
 * @version $Id: Torque.java,v 1.11 2000/09/05 14:07:32 gonzalo Exp $
 */
public class Torque
{
    /** Torque properties (torque.props). */
    Properties props;

    /** Torque macros (torque.macros). */
    Properties macros;

    /** Object that turns the XML database schema -> SQL. */
    XMLSchemaToSQL xmlSchemaToSQL;

    /** Object that turns the XML database schema -> OM. */
    XMLSchemaToObjectModel xmlSchemaToObjectModel;


    /**
     * Default constructor.
     */
    public Torque()
    {
        props = Utils.loadProps("config/torque.props");
        macros = Utils.loadProps("config/torque.macros");
        populateMacros();
    }

    /**
     * Constructor.
     *
     * @param propsFile The name of the Torque properties file.
     * @param macrosFile The name of the Torque macros file.
     */
    public Torque(String propsFile,
                  String macrosFile)
    {
        props = Utils.loadProps(propsFile);
        macros = Utils.loadProps(macrosFile);
        populateMacros();
    }

    /**
     * Constructor.
     *
     * @param propsFile The Torque properties file
     * @param macrosFile The Torque macros file
     */
    public Torque(File propsFile,
                  File macrosFile)
    {
        props = Utils.loadProps(propsFile);
        macros = Utils.loadProps(macrosFile);
        populateMacros();
    }

    /**
     * Turns the XML database schema into SQL for the target database,
     * and creates the corresponding object model.
     */
    public void execute()
    {
        File outputDir = new File(props.getProperty("outputDir"));

        // Make sure our output directory exists.
        if (! outputDir.exists())
        {
            outputDir.mkdir();
        }

        if (props.getProperty("mode").equalsIgnoreCase("sql"))
        {
            useSQLAsBasisForGeneration();
        }
        else if (props.getProperty("mode").equalsIgnoreCase("xml"))
        {
            useXMLAsBasisForGeneration();
        }
        else
        {
            System.out.println("You must select one of valid modes:");
            System.out.println("mode=xml or");
            System.out.println("mode=sql\n");
            System.out.println("Look at your torque.props file and set");
            System.out.println("this property correctly.");
        }
    }

    /**
     * Generate the stuff based on SQL.
     */
    public void useSQLAsBasisForGeneration()
    {
        if (props.getProperty("generateObjectModel").equalsIgnoreCase("true"))
        {
            xmlSchemaToObjectModel = new XMLSchemaToObjectModel();
            xmlSchemaToObjectModel.setProperties(props);
            xmlSchemaToObjectModel.setMacros(macros);
            xmlSchemaToObjectModel.executeSQL();
        }
    }

    /**
     * Generate the stuff based on XML.
     */
    public void useXMLAsBasisForGeneration()
    {
        if (props.getProperty("generateSQL").equalsIgnoreCase("true"))
        {
            xmlSchemaToSQL = new XMLSchemaToSQL();
            xmlSchemaToSQL.setProperties(props);
            xmlSchemaToSQL.setMacros(macros);
            xmlSchemaToSQL.execute();
        }

        if (props.getProperty("generateObjectModel").equalsIgnoreCase("true"))
        {
            xmlSchemaToObjectModel = new XMLSchemaToObjectModel();
            xmlSchemaToObjectModel.setProperties(props);
            xmlSchemaToObjectModel.setMacros(macros);
            xmlSchemaToObjectModel.execute();
        }
    }

    /**
     * Create some default macros for comment substitution.
     */
    private void populateMacros()
    {
        macros.put("db", props.getProperty("db"));
        macros.put("date",
                DateFormat.getDateTimeInstance(DateFormat.MEDIUM,
                DateFormat.MEDIUM, Locale.getDefault()).format(new Date()));
    }

    /**
     * Main function.
     *
     * @param args A String[] with the command line arguments.
     */
    public static void main(String[] args)
    {
        Torque torque = new Torque();
        torque.execute();
    }
}
