package org.apache.turbine.tools.torque;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java stuff.
import java.io.*;
import java.util.*;
import java.lang.reflect.*;

/**
 * This class provides some methods for dynamically invoking methods
 * in objects, and some string manipulation methods used by torque.
 * The string methods will soon be moved into the turbine string
 * utilities class.
 *
 *  @author <a href="mailto:jvanzyl@periapt.com">Jason van Zyl</a>
 *  @author <a href="mailto:celkins@scardini.com">Christopher Elkins</a>
 *  @version $Id: Utils.java,v 1.7 2000/08/31 22:31:40 gonzalo Exp $
 */
public class Utils
{
    /**
     * Dynamically invoke a method of an object.
     *
     * @param object Object which contains method to invoke.
     * @param method Name of method in object to invoke.
     */
    public static Object invoke(Object object,
                                String method)
    {
        try
        {
            // Figure the class we are (super even).
            Class c = Class.forName(object.getClass().getName());

            // See if there is a method named this command.  The
            // second parameter here are the parameter types: if you
            // need to ask for a specific method that has been
            // overloaded.  For our purposes here it will be null.
            Method m = c.getMethod(method, null);

            // Invoke the method.  The second parameter here is the
            // parameter list to the method.  We can probably get
            // around this nastiness by using private members in the
            // objects using this type of method invocation.
            Object returnObject = m.invoke(object, null);

            // Return true if we are done, false otherwise.
            return (returnObject);
        }
        catch (Throwable t)
        {
            System.err.println(t);
            return(null);
        }
    }

    /**
     * Load properties from the local filesystem.
     *
     * @param file Name of the properties file to load.
     * @return Properties, loaded properties file.
     */
    public static Properties loadProps(String file)
    {
        Properties props = new Properties();
        try
        {
            props.load(new FileInputStream(file));
        }
        catch (Exception e)
        {
            System.err.println(e);
        }

        return props;
    }

    /**
     * Load properties from the local filesystem.
     *
     * @param file The properties file to load.
     * @return Properties, loaded properties file.
     */
    public static Properties loadProps(File file)
    {
        Properties props = new Properties();
        try
        {
            props.load(new FileInputStream(file));
        }
        catch (Exception e)
        {
            System.err.println(e);
        }

        return props;
    }

    /**
     * Pull in the contents of a file and convert it to a single
     * String object.
     *
     * @param file Name of file to convert to a String.
     * @return String Resultant string created by reading file.
     */
    public static String fileContentsToString(String file)
    {
        StringBuffer sb = new StringBuffer();

        try
        {
            BufferedReader com = new BufferedReader(new FileReader(file));
            String line;

            while ((line = com.readLine()) != null)
            {
                sb.append(line + "\n");
            }
        }
        catch (IOException ioe)
        {
            ioe.printStackTrace();
        }

        return (sb.toString());
    }

    /**
     * For use with Torque. Take the contents of a file and format
     * them to a particular comment style.
     *
     * @param file Name of file to comment format.
     * @param commentTemplate Comment style template.
     * @param macros Macros to substitute into comments.
     * @return String Comment formatted, macros substituted string.
     */
    public static String fileContentsToCommentString(String file,
                                                     String commentTemplate,
                                                     Properties macros)
    {
        StringBuffer sb = new StringBuffer();

        try
        {
            BufferedReader com = new BufferedReader(new FileReader(file));
            String line;

            Hashtable comment = new Hashtable();
            StringBuffer commentLine;

            while ((line = com.readLine()) != null)
            {
                // Build our comment line, and substitute our
                // torque.macros values into the comment.
                comment.put("comment", line);
                commentLine = stringSubstitution(commentTemplate, comment);

                commentLine = stringSubstitution(commentLine.toString(),
                                                 macros);
                sb.append(commentLine.toString());
            }
            sb.append("\n");
        }
        catch (IOException ioe)
        {
            ioe.printStackTrace();
        }

        return (sb.toString());
    }

    /**
     * Takes a string and does macros substitution with values taken
     * from a Hashtable.
     *
     * The return valueshould be changed to a String.
     *
     * @param argStr String to perform macro substitution on.
     * @param vars Hashtable of macro->value pairs to substitute.
     * @return StringBuffer Resultant macro substituted string.
     */
    public static StringBuffer stringSubstitution(String argStr,
                                                  Hashtable vars)
    {
        StringBuffer argBuf = new StringBuffer();

        for (int cIdx = 0 ; cIdx < argStr.length();)
        {
            char ch = argStr.charAt(cIdx);

            switch (ch)
            {
            case '$':
                StringBuffer nameBuf = new StringBuffer();
                for (++cIdx ; cIdx < argStr.length(); ++cIdx)
                {
                    ch = argStr.charAt(cIdx);
                    if (ch == '_' || Character.isLetterOrDigit(ch))
                        nameBuf.append(ch);
                    else
                        break;
                }

                if (nameBuf.length() > 0)
                {
                    String value =
                        (String) vars.get(nameBuf.toString());

                    if (value != null)
                    {
                        argBuf.append(value);
                    }
                }
                break;

            default:
                argBuf.append(ch);
                ++cIdx;
                break;
            }
        }

        return argBuf;
    }

    /**
     * Remove/collapse multiple newline characters.
     *
     * @param argStr String to process.
     * @return String with processed answer.
     */
    public static String collapseNewlines(String argStr)
    {
        char last = argStr.charAt(0);
        StringBuffer argBuf = new StringBuffer();

        for (int cIdx = 0 ; cIdx < argStr.length(); cIdx++)
        {
            char ch = argStr.charAt(cIdx);
            if (ch != '\n' || last != '\n')
            {
                argBuf.append(ch);
                last = ch;
            }
        }

        return argBuf.toString();
    }

    /**
     * Remove/collapse multiple spaces.
     *
     * @param argStr String to process.
     * @return String with processed answer.
     */
    public static String collapseSpaces(String argStr)
    {
        char last = argStr.charAt(0);
        StringBuffer argBuf = new StringBuffer();

        for (int cIdx = 0 ; cIdx < argStr.length(); cIdx++)
        {
            char ch = argStr.charAt(cIdx);
            if (ch != ' ' || last != ' ')
            {
                argBuf.append(ch);
                last = ch;
            }
        }

        return argBuf.toString();
    }

    /**
     * Chop i characters off the end of a string.
     *
     * @param string String to chop.
     * @param i Number of characters to chop.
     * @return String with processed answer.
     */
    public static String chop(String string, int i)
    {
        return(string.substring(0, string.length() - i));
    }

    /**
     * Replaces all instances of oldString with newString in line.
     * Taken from the Jive forum package, it has an Apache license
     * too.
     *
     * @param line A String with the line to process.
     * @param oldString The string to look for.
     * @param newString The string to replace the matches with.
     * @return String with processed answer.
     */
    public static final String replace( String line,
                                        String oldString,
                                        String newString )
    {
        int i=0;
        if ( ( i=line.indexOf( oldString, i ) ) >= 0 ) {
            char [] line2 = line.toCharArray();
            char [] newString2 = newString.toCharArray();
            int oLength = oldString.length();
            StringBuffer buf = new StringBuffer(line2.length);
            buf.append(line2, 0, i).append(newString2);
            i += oLength;
            int j = i;
            while( ( i=line.indexOf( oldString, i ) ) > 0 ) {
                buf.append(line2, j, i-j).append(newString2);
                i += oLength;
                j = i;
            }
            buf.append(line2, j, line2.length - j);
            return buf.toString();
        }
        return line;
    }
}
