package org.apache.turbine.tools.torque;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java stuff.
import java.util.*;

// W3C stuff.
import org.w3c.dom.*;

// XML stuff.
import org.apache.xerces.parsers.*;

/**
 * This abstract class is a generic XML document parser.  Subclasses
 * of this class only have to define methods responsible for parsing
 * particular node types.
 *
 * For example, if you had an XML document that represented a database
 * schema such as the following:
 *
 * <database>
 *   <table name=table1">
 *     <column name="column1"/>
 *     <column name="column2"/>
 *     <column name="column3"/>
 *   </table>
 * </database>
 *
 * Then all you have to do to parse this document is to define the
 * methods:
 *
 * public boolean databaseParser();
 * public boolean tableParser();
 * public boolean columnParser()
 *
 * Within these methods the node currently be processed is available
 * as <b>currentNode</b>, and the attributes for this node are
 * available as <b>attributes</b>.
 *
 * @author <a href="mailto:jvanzyl@periapt.com">Jason van Zyl</a>
 * @version $Id: XMLParser.java,v 1.5 2000/08/31 22:31:40 gonzalo Exp $
 */
public abstract class XMLParser
{
    /** DOM document to be parsed. */
    protected Document document;

    /** Control canonical output. */
    protected boolean canonical;

    /** Current DOM Node being processed. */
    protected Node currentNode;

    /** Name of currentNodes parent. */
    protected String parentNodeName;

    /** Whether or not we are processing child Nodes. */
    protected boolean processingChildren;

    /** List of attributes for the currentNode. */
    protected Hashtable attributes;

    /** Name of the current node. */
    protected String name;

    /** Content of the current Node if it is a TEXT_NODE. */
    protected String content;

    // This is a hack, I'm not sure why the nodes are coming up as
    // ELEMENT_NODES, and TEXT_NODES?  So they are being parsed twice
    // which doesn't produce very pretty SQL.  This never used to
    // happen, many versions of xerces have passed since I've used
    // this code so I don't know what's going on.  I'll find it later.

    /** Whether or not we will process TEXT_NODEs. */
    protected boolean ignoreTextNodes = true;

    /** The type of the currentNode. */
    protected int nodeType;


    /**
     * Default constructor.
     */
    public XMLParser()
    {
        canonical = true;
        currentNode = null;
        parentNodeName = "";
        processingChildren = false;

        attributes = null;
        name = null;
        content = null;
    }

    /**
     * Set the URL of DOM document to parse.
     *
     * @param url URL of DOM document to parse.
     */
    public void setDocument(String url)
    {
        try
        {
            DOMParser ps = new DOMParser();
            ps.parse(url);
            document = ps.getDocument();
        }
        catch (Exception e)
        {
            System.err.println(e);
        }
    }

    /**
     * Parse our DOM document.
     */
    public void parse()
    {
        parse(document);
        finalCommand();
    }

    /**
     * Optional command to execute when parsing is complete.
     */
    public void finalCommand()
    {
    }

    /**
     * Recursively parse the document, passing off element nodes to
     * methods using reflection.
     *
     * @param node Node to recursively parse.
     */
    protected void parse(Node node)
    {
        String elementParser;
        NodeList children;

        // Is there anything to do?
        if (node == null)
        {
            return;
        }

        int type = node.getNodeType();
        nodeType = type;
        switch (type)
        {
            case Node.DOCUMENT_NODE:
                // This is the root node.
                parse(((Document) node).getDocumentElement());
                break;

            case Node.ELEMENT_NODE:
                // Process element node. I think this is what I'm
                // primarily interested in.

                // So if we have a menuitem element node, then we will
                // try to invoke a method named menuitemProcessor().
                // Hopefully this will make for easier parser
                // creation.
                name = node.getNodeName();
                elementParser = name + "Parser";

                // Maybe I should fire an event instead of invoking a
                // method by name.  What is easier when parsing a
                // relatively complex document.

                currentNode = node;
                attributes = getAttributes(node);

                if (Utils.invoke(this, elementParser) == null)
                {
                    defaultParser();
                }

                children = node.getChildNodes();
                if (children != null)
                {
                    processingChildren = true;
                    parentNodeName = node.getNodeName();

                    int len = children.getLength();
                    for (int i = 0; i < len; i++)
                    {
                        parse(children.item(i));
                    }

                    processingChildren = false;
                }
                break;

            case Node.ENTITY_REFERENCE_NODE:
                // Handle entity reference nodes.
                if (canonical)
                {
                    children = node.getChildNodes();
                    if (children != null)
                    {
                        int len = children.getLength();
                        for (int i = 0; i < len; i++)
                        {
                            parse(children.item(i));
                        }
                    }
                }
                else
                {
                    // out.print('&');
                    // out.print(node.getNodeName());
                    // out.print(';');
                }
                break;

            case Node.CDATA_SECTION_NODE:
                // Print cdata sections.

                /*
                if (canonical)
                {
                    out.print(normalize(node.getNodeValue()));
                }
                else
                {
                    out.print("<![CDATA[");
                    out.print(node.getNodeValue());
                    out.print("]]>");
                }
                */

                break;

            case Node.TEXT_NODE:
                // Print text.

                // In this case we want the value contained with an
                // element node.  This text node is probably the child
                // of an element node:
                //
                // <value>
                //     text
                // </value>

                if (ignoreTextNodes)
                    break;

                currentNode = node;
                elementParser = parentNodeName + "Parser";
                if (Utils.invoke(this, elementParser) == null)
                {
                    defaultParser();
                }

                break;
        }
    }

    /**
     * Default Node parser method if none can be found that matches
     * the name of the current Node.
     */
    public void defaultParser()
    {
        String nodeName = currentNode.getNodeName();
        System.out.println("No processor for the element node " + nodeName);
    }

    /**
     * Get the attributes of the currentNode and return them in the
     * form of a Hashtable.
     *
     * @param node Node from which to retrieve attributes.
     * @return Hashtable Attributes for the given node.
     */
    public static Hashtable getAttributes(Node node)
    {
        Hashtable attributes = new Hashtable();

        NamedNodeMap attrs = node.getAttributes();

        if (attrs != null)
        {
            int length = attrs.getLength();
            for (int i = 0; i < length; i++)
            {
                Attr attribute = (Attr) attrs.item(i);
                attributes.put(attribute.getName(), attribute.getValue());
            }
        }
        return(attributes);
    }
}
