package org.apache.turbine.util;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.Date;
import java.util.Vector;
import java.io.*;

// Turbine stuff.
import org.apache.turbine.services.resources.*;

/**
 * This is a utility class that is used for logging events.  Entries
 * are allowed in 3 levels - Notice, Warning, Error.  Methods are also
 * available to log exceptions, which are always logged at the Error
 * level.
 *
 * <p>The path to a writable file must be given in
 * TurbineResources.properties for the log to be written.  If the file
 * does not already exist, then the file will be created.  If the path
 * is invalid, then the file will not be created and nothing will
 * happen.
 *
 * <p>To use this class outside of Turbine, all you need to do is set
 * the path to the TurbineResources.properties file first. To do this,
 * you would write code like this:

 * <p>
 * TurbineResources.setPropertiesFileName ("/path/to/TurbineResources.properties");<br>
 * Log.note ("this is my notice!");
 *
 * @author <a href="mailto:jon@clearink.com">Jon S. Stevens</a>
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @author <a href="mailto:jthorhauer@phoenixcolor.com">John Thorhauer</a>
 * @version $Id: Log.java,v 1.3 2000/09/01 18:15:12 gonzalo Exp $
 */
public class Log
{
    private static String lf = System.getProperty("line.separator", "\n");

    // True if filename is valid and writeable.
    private static boolean isValid = true;

    // Filename for log file.
    private static FileWriter logfile = null;

    // Array to hold multiple FileWriter objects.
    private static Object logfiles[][];

    // Levels - Notice, Warning, Error.
    private static final String NOTICE = "NOTICE ";
    private static final String WARNING = "WARNING";
    private static final String ERROR = " ERROR ";

    /*
     * Static initialization.
     */
    static
    {
        init();
    }

    /**
     * Checks to see if log variables are set up properly and then
     * initializes the log(s).
     */
    private static void init()
    {
        try
        {
            String logFileString = TurbineResources.getString("logfile", null);
            Vector turbineLogs = TurbineResources.getVector("turbine.logs");

            if (logFileString == null && turbineLogs.isEmpty())
            {
                throw new Exception (
                "The logfile= property and the turbine.logs= property in the TurbineResources.properties file are null \n" +
                "or could not be found.  At least one of these properties must be defined in the TurbineResources.properties file. \n" +
                "Please make sure that you have properly defined the path to the TurbineResources.properties file\n" +
                "and you have called \"TurbineResourceService.setPropertiesFileName(\"/path/to/TurbineResources.properties\");\" in your code.\n" +
                "If you are using this class with the Turbine Servlet, this should have been called for you already.\n" +
                "If you are getting this error using the Turbine Servlet, the path to the properties file or\n" +
                "the path to the logfile or the logcount was not defined correctly. Please refer to the INSTALL \n" +
                "document for instructions on how to do this.\n");
            }

            if (logFileString != null)
            {
                logfile = new FileWriter( logFileString, true );
            }

            // If there are turbine.logs defined then set up an array
            // of FileWriters for use in the method
            // Log.note(logname,text) etc.
            if (!turbineLogs.isEmpty())
            {
                logfiles= new Object[turbineLogs.size()][2];
                for (int i=0;i<turbineLogs.size();i++)
                {
                    if (TurbineResources.getString("turbine.log." + (String)turbineLogs.elementAt(i),null) == null)
                    {
                        throw new Exception(
                        "There is an error in the TurbineResources.properties file. \n" +
                        "The turbine.log." + (String)turbineLogs.elementAt(i) + "= entry could not be found for the \n" +
                        "corresponding turbine.logs=" + (String)turbineLogs.elementAt(i) + " entry. \n" +
                        "Be sure that each turbine.logs= entry has a corresponding turbine.log.[logname]= \n" +
                        "in the TurbineResources.properties file. \n" +
                        "Please refer to the comments in the TurbineResources.properties file \n" +
                        "or the user documentation for more information on how to properly \n" +
                        "implement logging in a Turbine application. \n");
                    }
                    logfiles[i][0]= (String)turbineLogs.elementAt(i);
                    logfiles[i][1]= (FileWriter) new FileWriter(TurbineResources.getString("turbine.log." + (String)turbineLogs.elementAt(i),null),true);
                }
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
            isValid = false;
        }
        catch (Throwable t)
        {
            isValid = false;
        }
    }

    /*
     * Makes a log entry at the NOTICE level.
     *
     * @param description A String with the text for the entry.
     * @return True if entry was entered, false otherwise.
     */
    public static boolean note(String description)
    {
        return log(NOTICE, description);
    }

    /*
     * Specifies the log to print to and
     * makes a log entry at the NOTICE level
     *
     * @returns true if entry was entered, false otherwise
     */
    public static boolean note(String logname, String description)
    {
        return log(NOTICE, logname, description, null);
    }

    /*
     * Makes a log entry at the WARNING level.
     *
     * @param description A String with the text for the entry.
     * @return True if entry was entered, false otherwise.
     */
    public static boolean warn(String description)
    {
        return log(WARNING, description);
    }

    /*
     * Specifies the log to print to and
     * makes a log entry at the WARNING level
     *
     * @returns true if entry was entered, false otherwise
     */
    public static boolean warn(String logname, String description)
    {
        return log(WARNING, logname, description, null);
    }

    /*
     * Makes a log entry at the ERROR level.
     *
     * @param description A String with the text for the entry.
     * @return True if entry was entered, false otherwise.
     */
    public static boolean error(String description)
    {
   System.out.print("Error=" + description + "\n");
        return log(ERROR, description);

    }
    /*
     * Specifies the log to print to and
     * makes a log entry at the ERROR level
     *
     * @returns true if entry was entered, false otherwise
     */
    public static boolean error(String logname, String description)
    {
        return log(ERROR, logname, description, null);
    }
    /*
     * Makes a log entry at the ERROR level
     *
     * @returns true if entry was entered, false otherwise
     */
    public static boolean error(String description, Throwable t)
    {
        return log(ERROR, description, t);
    }

    /*
     * Specifies the log to print to and
     * makes a log entry at the ERROR level
     *
     * @returns true if entry was entered, false otherwise
     */
    public static boolean error(String logname, String description, Throwable t)
    {
        return log(ERROR, logname, description, t);
    }

    /*
     * Makes a log entry at the ERROR level.
     *
     * @param t A Throwable with the exception.
     * @return True if entry was entered, false otherwise.
     */
    public static boolean error(Throwable t)
    {
        return log(ERROR, "", t);
    }

    /*
     * Makes a log entry.
     *
     * @param level A String with the level.
     * @param description A String with the text for the entry.
     * @return True if entry was entered, false otherwise.
     */
    private static boolean log(String level,
                               String description)
    {
        return log(level, description, null);
    }

    /*
     * Makes a log entry.
     *
     * @param level A String with the level.
     * @param description A String with the text for the entry.
     * @param t A Throwable with the exception.
     * @return True if entry was entered, false otherwise.
     */
    private static boolean log(String level,
                               String description,
                               Throwable t)
    {
        if ( isValid )
        {
            return log(level, description, t, logfile);
        }
        return false;
    }

    /*
     * Makes a log entry in a specified log.
     *
     * @param level A String with the level.
     * @param logname A String with the log name.
     * @param description A String with the text for the entry.
     * @param t A Throwable with the exception.
     * @return True if entry was entered, false otherwise.
     */
    private static boolean log(String level,
                               String logname,
                               String description,
                               Throwable t)
    {
        boolean foundLog = false;
        int i = 0;
        int activeLogNumber = 0;
        for (i=0;i<logfiles.length;i++)
        {
            if (logfiles[i][0].equals(logname))
            {
                activeLogNumber = i;
                foundLog=true;
                break;
            }
        }
        if (foundLog)
        {
            return log(level, description, t,
                       (FileWriter)logfiles[activeLogNumber][1]);
        }
        return false;
    }

    /*
     * Makes a log entry in a given log file.
     *
     * @param level A String with the level.
     * @param description A String with the text for the entry.
     * @param t A Throwable with the exception.
     * @param fw A FileWriter.
     * @return True if entry was entered, false otherwise.
     */
    private static boolean log(String level,
                               String description,
                               Throwable t,
                               FileWriter fw)
    {
        Date date = new Date();
        StringBuffer logEntry = new StringBuffer();
        logEntry.append ( "[" );
        logEntry.append ( date.toString() );
        logEntry.append ( "] -- " );
        logEntry.append ( level );
        logEntry.append ( " -- " );
        logEntry.append ( description );
        logEntry.append ( lf );
        if (t != null)
        {
            ByteArrayOutputStream ostr = new ByteArrayOutputStream();
            PrintWriter out = new PrintWriter(ostr,true);
            out.write(logEntry.toString());
            out.write("\tException:  ");
            out.write(t.toString());
            out.write(lf+"\tStack Trace follows:"+lf+"\t");
            t.printStackTrace(out);
            logEntry = new StringBuffer( ostr.toString() );
        }
        synchronized( fw )
        {
            try
            {
                fw.write( logEntry.toString(), 0, logEntry.length());
                fw.flush();
            }
            catch(IOException ioe)
            {
                // Ignore, skip log, should not occur since we have
                // checked file but catching the exception allows Log
                // to be used elsewhere in catch blocks.
                return false;
            }
        }
        return true;
    }

    /**
     * Attempt to close the log files when the class is GC'd.
     */
    public void destroy()
    {
        try
        {
            if (logfile != null)
            {
                logfile.close();
            }
            if (logfiles.length>0)
            {
                for (int i=0;i<logfiles.length;i++)
                {
                    if(logfiles[i][1] != null)
                    {
                        ((FileWriter)logfiles[i][1]).close();
                    }
                }
            }
        }
        catch (Exception e)
        {
        }
    }
}
