package org.apache.turbine.util;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.io.*;
import java.util.*;
import java.beans.*;
import java.lang.reflect.*;
import java.text.*;

// Java Servlet Classes
import javax.servlet.*;
import javax.servlet.http.*;

// JServ Utilities
import org.apache.jserv.*;

// Turbine stuff
import org.apache.turbine.services.resources.*;
import org.apache.turbine.services.upload.*;
import org.apache.turbine.util.upload.*;

/**
 *
 * ParameterParser is a utility object to handle parsing and
 * retrieving the data passed via the GET/POST/PATH_INFO arguments.
 *
 * <p>NOTE: The name= portion of a name=value pair will be converted
 * to lowercase when the object is initialized and when new data is
 * added.  This may overwrite existing name=value pairs:
 *
 * <pre>
 * ParameterParser pp = data.getParameters();
 * pp.add("ERROR",1);
 * pp.add("eRrOr",2);
 * int result = pp.getInt("ERROR");
 * </pre>
 *
 * In the above example, result is 2.
 *
 * @author <a href="mailto:jon@clearink.com">Jon S. Stevens</a>
 * @version $Id: ParameterParser.java,v 1.6 2000/09/01 21:09:59 gonzalo Exp $
 */
public class ParameterParser
    extends Hashtable
{
    private HttpServletRequest req = null;

    /**
     * Create a new instance of ParameterParser.  This requires a
     * valid HttpServletRequest object.  It will attempt to parse out
     * the GET/POST/PATH_INFO data and place the data into itself
     * which is an extension of Hashtable.  There are convenience
     * methods for retrieving the data as a number of different
     * datatypes.  The PATH_INFO data must be a URLEncoded() string.
     *
     * <p>To add things into this object, use the add() method and not
     * the Hashtable.put() method.
     *
     * @param req An HttpServletRequest.
     */
    public ParameterParser(HttpServletRequest req)
    {
        this.req = req;

        // String object re-use at its best.
        String tmp = null;

        tmp = req.getHeader("Content-type");
        if (tmp != null && tmp.startsWith("multipart/form-data") &&
            TurbineUpload.getAutomatic())
        {
            try
            {
                TurbineUpload.parseRequest(req, this);
            }
            catch(IOException e)
            {
                Log.error("File upload failed");
                Log.error(e);
            }
        }

        Enumeration names = req.getParameterNames();
        if ( names != null )
        {
            while(names.hasMoreElements())
            {
                tmp = (String) names.nextElement();
                this.put( convert(tmp), (Object) req.getParameterValues(tmp) );
            }
        }

        // Also cache any pathinfo variables that are passed around as
        // if they are query string data.
        try
        {
            StringTokenizer st = new StringTokenizer(req.getPathInfo(), "/");
            boolean name = true;
            String tmp2 = null;
            while(st.hasMoreTokens())
            {
                if ( name == true )
                {
                    tmp = JServUtils.URLDecode(st.nextToken());
                    name = false;
                }
                else
                {
                    tmp2 = JServUtils.URLDecode(st.nextToken());
                    if ( tmp.length() != 0 )
                    {
                        add (convert(tmp), tmp2);
                    }
                    name = true;
                }
            }
        }
        catch ( Exception e )
        {
            // If anything goes wrong above, don't worry about it.
            // Chances are that the path info was wrong anyways and
            // things that depend on it being right will fail later
            // and should be caught later.
        }
    }

    /**
     * Add a name/value pair into this object.  The reason for using
     * an add() method instead of Hashtable.put is because things must
     * be in a String[].
     *
     * @param name A String with the name.
     * @param value A double with the value.
     */
    public void add ( String name,
                      double value )
    {
        add ( name, Double.toString(value));
    }

    /**
     * Add a name/value pair into this object.  The reason for using
     * an add() method instead of Hashtable.put is because things must
     * be in a String[].
     *
     * @param name A String with the name.
     * @param value An int with the value.
     */
    public void add ( String name,
                      int value )
    {
        add ( name, Integer.toString(value));
    }

    /**
     * Add a name/value pair into this object.  The reason for using
     * an add() method instead of Hashtable.put is because things must
     * be in a String[].
     *
     * @param name A String with the name.
     * @param value A long with the value.
     */
    public void add ( String name,
                      long value )
    {
        add ( name, Long.toString(value));
    }

    /**
     * Add a name/value pair into this object.  The reason for using
     * an add() method instead of Hashtable.put is because things must
     * be in a String[].
     *
     * @param name A String with the name.
     * @param value A String with the value.
     */
    public void add ( String name,
                      String value )
    {
        String[] strtmp = new String[1];
        strtmp[0] = value;
        this.put ( convert(name), (String[]) strtmp);
    }

    /**
     * Add a String parameter.  If there are any Strings already
     * associated with the name, append to the array.  This is used
     * for handling parameters from mulitipart POST requests.
     *
     * @param name A String with the name.
     * @param value A String with the value.
     */
    public void append( String name,
                        String value )
    {
        String[] items = this.getStrings(name);
        if(items == null)
        {
            items = new String[1];
            items[0] = value;
            this.put( convert(name), items );
        }
        else
        {
            String[] newItems = new String[items.length+1];
            System.arraycopy(items, 0, newItems, 0, items.length);
            newItems[items.length] = value;
            this.put( convert(name), newItems );
        }
    }

    /**
     * Add a FileItem object as a parameter.  If there are any
     * FileItems already associated with the name, append to the
     * array.  The reason for this is that RFC 1867 allows multiple
     * files to be associated with single HTML input element.
     *
     * @param name A String with the name.
     * @param value A FileItem with the value.
     */
    public void append( String name,
                        FileItem value )
    {
        FileItem[] items = this.getFileItems(name);
        if(items == null)
        {
            items = new FileItem[1];
            items[0] = value;
            this.put( convert(name), items );
        }
        else
        {
            FileItem[] newItems = new FileItem[items.length+1];
            System.arraycopy(items, 0, newItems, 0, items.length);
            newItems[items.length] = value;
            this.put( convert(name), newItems );
        }
    }

    /**
     * Trims the string data and converts it to lowercase.  It returns
     * a new string so that it does not destroy the value data.
     *
     * @param value A String to be processed.
     * @return A new String converted to lowercase and trimmed.
     */
    public static String convert ( String value )
    {
        return (value.trim().toLowerCase());
    }

    /**
     * Determine whether a given key has been inserted.  All keys are
     * stored in lowercase strings, so override method to account for
     * this.
     *
     * @param key An Object with the key to search for.
     * @return True if the object is found.
     */
    public boolean containsKey( Object key )
    {
        return super.containsKey(convert((String)key));
    }

    /**
     * Check for existence of key_day, key_month and key_year
     * parameters (as returned by DateSelector generated HTML).
     *
     * @param key A String with the selector name.
     * @return True if keys are found.
     */
    public boolean containsDateSelectorKeys(String key)
    {
        return (containsKey(key + DateSelector.DAY_SUFFIX) &&
                containsKey(key + DateSelector.MONTH_SUFFIX) &&
                containsKey(key + DateSelector.YEAR_SUFFIX));
    }

    /**
     * Return a boolean for the given name.  If the name does not
     * exist, return defaultValue.
     *
     * @param name A String with the name.
     * @param defaultValue The default value.
     * @return A boolean.
     */
    public boolean getBoolean(String name,
                              boolean defaultValue)
    {
        boolean value = defaultValue;
        Object object = this.get(convert(name));
        if (object != null)
        {
            String tmp = getString(name);
            if ( tmp.equalsIgnoreCase ("1") ||
                 tmp.equalsIgnoreCase ("true") )
            {
                value = true;
            }
            if ( tmp.equalsIgnoreCase ("0") ||
                 tmp.equalsIgnoreCase ("false") )
            {
                value = false;
            }
        }
        return value;
    }

    /**
     * Return a boolean for the given name.  If the name does not
     * exist, return false.
     *
     * @param name A String with the name.
     * @return A boolean.
     */
    public boolean getBoolean(String name)
    {
        return getBoolean(name, false);
    }

    /**
     * Return a Boolean for the given name.  If the name does not
     * exist, return defaultValue.
     *
     * @param name A String with the name.
     * @param defaultValue The default value.
     * @return A Boolean.
     */
    public Boolean getBool(String name,
                           boolean defaultValue)
    {
        return new Boolean(getBoolean(name, defaultValue));
    }

    /**
     * Return a Boolean for the given name.  If the name does not
     * exist, return false.
     *
     * @param name A String with the name.
     * @return A Boolean.
     */
    public Boolean getBool(String name)
    {
        return new Boolean(getBoolean(name, false));
    }

    /**
     * Return a double for the given name.  If the name does not
     * exist, return defaultValue.
     *
     * @param name A String with the name.
     * @param defaultValue The default value.
     * @return A double.
     */
    public double getDouble(String name,
                            double defaultValue)
    {
        double value = defaultValue;
        try
        {
            Object object = this.get(convert(name));
            if (object != null)
                value = Double.valueOf(((String[])object)[0]).doubleValue();
        }
        catch (NumberFormatException exception)
        {
        }
        return value;
    }

    /**
     * Return a double for the given name.  If the name does not
     * exist, return 0.0.
     *
     * @param name A String with the name.
     * @return A double.
     */
    public double getDouble(String name)
    {
        return getDouble(name, 0.0);
    }

    /**
     * Return an int for the given name.  If the name does not exist,
     * return defaultValue.
     *
     * @param name A String with the name.
     * @param defaultValue The default value.
     * @return An int.
     */
    public int getInt(String name,
                      int defaultValue )
    {
        int value = defaultValue;
        try
        {
            Object object = this.get(convert(name));
            if (object != null)
                value = Integer.valueOf(((String[])object)[0]).intValue();
        }
        catch (NumberFormatException exception)
        {
        }
        return value;
    }

    /**
     * Return an int for the given name.  If the name does not exist,
     * return 0.
     *
     * @param name A String with the name.
     * @return An int.
     */
    public int getInt(String name)
    {
        return getInt(name, 0);
    }

    /**
     * Return an Integer for the given name.  If the name does not
     * exist, return defaultValue.
     *
     * @param name A String with the name.
     * @param defaultValue The default value.
     * @return An Integer.
     */
    public Integer getInteger(String name,
                              int defaultValue)
    {
        return new Integer(getInt(name, defaultValue));
    }

    /**
     * Return an Integer for the given name.  If the name does not
     * exist, return defaultValue.  You cannot pass in a null here for
     * the default value.
     *
     * @param name A String with the name.
     * @param defaultValue The default value.
     * @return An Integer.
     */
    public Integer getInteger(String name,
                              Integer def)
    {
        return new Integer(getInt(name, def.intValue()));
    }

    /**
     * Return an Integer for the given name.  If the name does not
     * exist, return 0.
     *
     * @param name A String with the name.
     * @return An Integer.
     */
    public Integer getInteger(String name)
    {
        return new Integer(getInt(name, 0));
    }

    /**
     * Return an array of ints for the given name.  If the name does
     * not exist, return null.
     *
     * @param name A String with the name.
     * @return An int[].
     */
    public int[] getInts(String name)
    {
        int[] value = null;
        Object object = getStrings(convert(name));
        if (object != null)
        {
            String[] temp = (String[])object;
            value = new int[temp.length];
            for (int i=0; i<temp.length; i++)
                value[i] = Integer.parseInt( temp[i] );
        }
        return value;
    }

    /**
     * Return an array of Integers for the given name.  If the name
     * does not exist, return null.
     *
     * @param name A String with the name.
     * @return An Integer[].
     */
    public Integer[] getIntegers(String name)
    {
        Integer[] value = null;
        Object object = getStrings(convert(name));
        if (object != null)
        {
            String[] temp = (String[])object;
            value = new Integer[temp.length];
            for (int i=0; i<temp.length; i++)
                value[i] = Integer.valueOf( temp[i] );
        }
        return value;
    }

    /**
     * Return a long for the given name.  If the name does not exist,
     * return defaultValue.
     *
     * @param name A String with the name.
     * @param defaultValue The default value.
     * @return A long.
     */
    public long getLong(String name,
                        long defaultValue )
    {
        long value = defaultValue;
        try
        {
            Object object = this.get(convert(name));
            if (object != null)
                value = Long.valueOf(((String[])object)[0]).longValue();
        }
        catch (NumberFormatException exception)
        {
        }
        return value;
    }

    /**
     * Return a long for the given name.  If the name does not exist,
     * return 0.
     *
     * @param name A String with the name.
     * @return A long.
     */
    public long getLong(String name)
    {
        return getLong(name, 0);
    }

    /**
     * Return an array of longs for the given name.  If the name does
     * not exist, return null.
     *
     * @param name A String with the name.
     * @return A long[].
     */
    public long[] getLongs(String name)
    {
        long[] value = null;
        Object object = getStrings(convert(name));
        if (object != null)
        {
            String[] temp = (String[])object;
            value = new long[temp.length];
            for (int i=0; i<temp.length; i++)
                value[i] = Long.parseLong( temp[i] );
        }
        return value;
    }

    /**
     * Return an array of Longs for the given name.  If the name does
     * not exist, return null.
     *
     * @param name A String with the name.
     * @return A Long[].
     */
    public Long[] getLongObjects(String name)
    {
        Long[] value = null;
        Object object = getStrings(convert(name));
        if (object != null)
        {
            String[] temp = (String[])object;
            value = new Long[temp.length];
            for (int i=0; i<temp.length; i++)
                value[i] = Long.valueOf( temp[i] );
        }
        return value;
    }

    /**
     * Return a byte for the given name.  If the name does not exist,
     * return defaultValue.
     *
     * @param name A String with the name.
     * @param defaultValue The default value.
     * @return A byte.
     */
    public byte getByte(String name,
                        byte defaultValue )
    {
        byte value = defaultValue;
        try
        {
            Object object = this.get(convert(name));
            if (object != null)
                value = Byte.valueOf(((String[])object)[0]).byteValue();
        }
        catch (NumberFormatException exception)
        {
        }
        return value;
    }

    /**
     * Return a byte for the given name.  If the name does not exist,
     * return 0.
     *
     * @param name A String with the name.
     * @return A byte.
     */
    public byte getByte(String name)
    {
        return getByte(name, (byte) 0);
    }

    /**
     * Return an array of bytes for the given name.  If the name does
     * not exist, return null. The array is returned according to the
     * HttpRequest's character encoding.
     *
     * @param name A String with the name.
     * @return A byte[].
     * @exception UnsupportedEncodingException.
     */
    public byte[] getBytes(String name)
        throws UnsupportedEncodingException
    {
        String tempStr = getString(name);
        if ( tempStr != null )
            return tempStr.getBytes(req.getCharacterEncoding());
        return null;
    }

    /**
     * Return a String for the given name.  If the name does not
     * exist, return null.
     *
     * @param name A String with the name.
     * @return A String.
     */
    public String getString(String name)
    {
        try
        {
            String value = null;
            Object object = this.get(convert(name));
            if (object != null)
                value = ((String[])object)[0];
            return value;
        }
        catch ( ClassCastException e )
        {
            return null;
        }
    }

    /**
     * Return a String for the given name.  If the name does not
     * exist, return the defaultValue.
     *
     * @param name A String with the name.
     * @param defaultValue The default value.
     * @return A String.
     */
    public String getString(String name,
                            String defaultValue)
    {
        String value = getString(name);
        if (value == null ||
            value.length() == 0 ||
            value.equals("null"))
            return defaultValue;
        else
            return value;
    }

    /**
     * Return an array of Strings for the given name.  If the name
     * does not exist, return null.
     *
     * @param name A String with the name.
     * @return A String[].
     */
    public String[] getStrings(String name)
    {
        String[] value = null;
        Object object = this.get(convert(name));
        if (object != null)
            value = ((String[])object);
        return value;
    }

    /**
     * Return an array of Strings for the given name.  If the name
     * does not exist, return the defaultValue.
     *
     * @param name A String with the name.
     * @param defaultValue The default value.
     * @return A String[].
     */
    public String[] getStrings(String name,
                               String[] defaultValue)
    {
        String[] value = getStrings(name);
        if (value == null ||
            value.length == 0)
            return defaultValue;
        else
            return value;
    }

    /**
     * Return an Object for the given name.  If the name does not
     * exist, return null.
     *
     * @param name A String with the name.
     * @return An Object.
     */
    public Object getObject(String name)
    {
        try
        {
            Object value = null;
            Object object = this.get(convert(name));
            if (object != null)
                value = ((Object[])object)[0];
            return value;
        }
        catch ( ClassCastException e )
        {
            return null;
        }
    }

    /**
     * Return an array of Objects for the given name.  If the name
     * does not exist, return null.
     *
     * @param name A String with the name.
     * @return An Object[].
     */
    public Object[] getObjects(String name)
    {
        try
        {
            return (Object[])this.get(convert(name));
        }
        catch ( ClassCastException e )
        {
            return null;
        }
    }

    /**
     * Returns a java.util.Date object.  String is parsed by supplied
     * DateFormat.  If the name does not exist, return the
     * defaultValue.
     *
     * @param name A String with the name.
     * @param df A DateFormat.
     * @param defaultValue The default value.
     * @return A Date.
     */
    public Date getDate(String name,
                        DateFormat df,
                        Date defaultValue)
    {
        Date date = null;

        if (containsKey(name))
        {
            try
            {
                // Reject invalid dates.
                df.setLenient(false);
                date = df.parse(getString(name));
            }
            catch (ParseException e)
            {
                // Thrown if couldn't parse date.
                date = defaultValue;
            }
        }
        else
            date = defaultValue;

        return date;
    }

    /**
     * Returns a java.util.Date object.  If there are DateSelector
     * style parameters then these are used.  If not and there is a
     * parameter 'name' then this is parsed by DateFormat.  If the
     * name does not exist, return null.
     *
     * @param name A String with the name.
     * @return A Date.
     */
    public Date getDate(String name)
    {
        Date date = null;

        if (containsDateSelectorKeys(name))
        {
            try
            {
                Calendar cal =  new GregorianCalendar(
                        getInt(name + DateSelector.YEAR_SUFFIX),
                        getInt(name + DateSelector.MONTH_SUFFIX),
                        getInt(name + DateSelector.DAY_SUFFIX));

                // Reject invalid dates.
                cal.setLenient(false);
                date = cal.getTime();
            }
            catch (IllegalArgumentException e)
            {
                // Thrown if an invalid date.
            }
        }
        else
        {
            DateFormat df = DateFormat.getDateInstance();
            date = getDate(name, df, null);
        }

        return date;
    }

    /**
     * Returns a java.util.Date object.  String is parsed by supplied
     * DateFormat.  If the name does not exist, return null.
     *
     * @param name A String with the name.
     * @param df A DateFormat.
     * @return A Date.
     */
    public Date getDate(String name,
                        DateFormat df)
    {
        return getDate(name, df, null);
    }

    /**
     * Return a FileItem object for the given name.  If the name does
     * not exist or the object stored is not a FileItem, return null.
     *
     * @param name A String with the name.
     * @return A FileItem.
     */
    public FileItem getFileItem(String name)
    {
        try
        {
            FileItem value = null;
            Object object = this.get(convert(name));
            if (object != null)
                value = ((FileItem[])object)[0];
            return value;
        }
        catch ( ClassCastException e )
        {
            return null;
        }
    }

    /**
     * Return an array of FileItem objects for the given name.  If the
     * name does not exist or the object stored is not a FileItem
     * array, return null.
     *
     * @param name A String with the name.
     * @return A FileItem[].
     */
    public FileItem[] getFileItems(String name)
    {
        try
        {
            return (FileItem[])this.get(convert(name));
        }
        catch ( ClassCastException e )
        {
            return null;
        }
    }

    /**
     * Uses bean introspection to set writable properties of bean from
     * the parameters, where a (case-insensitive) name match between
     * the bean property and the parameter is looked for.
     *
     * @param bean An Object.
     * @exception Exception, a generic exception.
     */
    public void setProperties(Object bean)
        throws Exception
    {
        Class beanClass = bean.getClass();
        PropertyDescriptor[] props
            = Introspector.getBeanInfo(beanClass).getPropertyDescriptors();

        for (int i = 0; i < props.length; i++)
        {
            String propname = props[i].getName();
            Method setter = props[i].getWriteMethod();
            if (setter != null &&
                (containsKey(propname) ||
                 containsDateSelectorKeys(propname)))
            {
                setProperty(bean, props[i]);
            }
        }
    }

    /**
     * Set the property 'prop' in the bean to the value of the
     * corresponding parameter.  Supports all types supported by
     * getXXX methods plus a few more that come for free because
     * primitives have to be wrapped before being passed to invoke
     * anyway.
     *
     * @param bean An Object.
     * @param prop A PropertyDescriptor.
     * @exception Exception, a generic exception.
     */
    private void setProperty(Object bean,
                             PropertyDescriptor prop)
        throws Exception
    {
        if (prop instanceof IndexedPropertyDescriptor)
        {
            throw new Exception(prop.getName() +
                                " is an indexed property (not supported)");
        }

        Method setter = prop.getWriteMethod();
        if (setter == null)
        {
            throw new Exception(prop.getName() +
                                " is a read only property");
        }

        Class propclass = prop.getPropertyType();
        Object[] args = { null };

        if (propclass == Boolean.class || propclass == Boolean.TYPE)
            args[0] = getBool(prop.getName());
        else if (propclass == Double.class  || propclass == Double.TYPE)
            args[0] = new Double(getDouble(prop.getName()));
        else if (propclass == Integer.class || propclass == Integer.TYPE)
            args[0] = getInteger(prop.getName());
        else if (propclass == Long.class    || propclass == Long.TYPE)
            args[0] = new Long(getLong(prop.getName()));
        else if (propclass == Object.class)
            args[0] = getObject(prop.getName());
        else if (propclass == String.class)
            args[0] = getString(prop.getName());
        else if (propclass == int[].class)
            args[0] = getInts(prop.getName());
        else if (propclass == Integer[].class)
            args[0] = getIntegers(prop.getName());
        else if (propclass == String[].class)
            args[0] = getStrings(prop.getName());
        else if (propclass == Date.class)
            args[0] = getDate(prop.getName());
        else
        {
            throw new Exception("property "
                                + prop.getName()
                                + " is of unsupported type "
                                + propclass.toString());
        }

        setter.invoke(bean, args);
    }

    /**
     * Simple method that attempts to get a toString() representation
     * of this object.  It doesn't do well with String[]'s though.
     *
     * @return A String.
     */
    public String toString()
    {
        StringBuffer sb = new StringBuffer();
        for (Enumeration e = this.keys() ; e.hasMoreElements() ;)
        {
            String name = (String) e.nextElement();
            try
            {
                sb.append ("{");
                sb.append(name);
                sb.append("=");
                sb.append(this.getString(name));
                sb.append ("}\n");
            }
            catch ( Exception ee)
            {
                try
                {
                    sb.append ("{");
                    sb.append(name);
                    sb.append("=");
                    sb.append ("ERROR?");
                    sb.append ("}\n");
                }
                catch ( Exception eee )
                {
                }
            }
        }
        return sb.toString();
    }
}
