package org.apache.turbine.util;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.sql.*;
import java.util.*;
import java.io.IOException;

// Java Servlet Classes
import javax.servlet.*;
import javax.servlet.http.*;

// Turbine Utility Classes
import org.apache.turbine.util.db.*;
import org.apache.turbine.om.security.*;
import org.apache.turbine.om.user.User;
import org.apache.turbine.services.resources.TurbineResources;

// ECS Classes
import org.apache.ecs.*;
import org.apache.ecs.html.*;

// Support for ServletResponse.getWriter
import java.io.PrintWriter;

/**
 * RunData is an object that is passed within Turbine.  This provides
 * the threading mechanism for the entire system because multiple
 * requests can potentially come in at the same time.  Thus, there is
 * only one RunData object for each request that is being serviced.
 *
 * <p>This object caches a lot of frequently used data, such as a
 * database connection, the queryTable and other important
 * information.
 *
 * @author <a href="mailto:jon@clearink.com">Jon S. Stevens</a>
 * @author <a href="mailto:bhoeneis@ee.ethz.ch">Bernie Hoeneisen</a>
 * @author <a href="mailto:dlr@finemaltcoding.com">Daniel Rall</a>
 * @version $Id: RunData.java,v 1.8 2000/09/06 18:15:59 gonzalo Exp $
 */
public class RunData
{
    /** Servlet request interfaces. */
    public HttpServletRequest req = null;

    /** Servlet response interfaces. */
    public HttpServletResponse res = null;

    /** Servlet Session information. */
    public HttpSession session = null;

    /**
     * Servlet context information.  Note that this is from the
     * "Turbine" Servlet context.
     */
    public ServletContext servletContext = null;

    /** The servlet configuration. */
    private ServletConfig config = null;

    /** JNDI Contexts. */
    public Hashtable jndiContexts = null;

    /** The HTTP content type to return. */
    public String contentType = "text/html";

    /** The HTTP charset. */
    public String charSet = null;

    /** The HTTP status code to return. */
    public int statusCode = HttpServletResponse.SC_OK;

    /** If this is set, also set the status code to 302. */
    public String redirectURI = null;

    /** This contains a reference to the GET/POST data. */
    public ParameterParser parameters = null;

    /** Cached screen name to execute for this request. */
    private String screen = null;

    /** Cached action name to execute for this request. */
    private String action = null;

    /** The User object. */
    public User user = null;

    /** The AccessControl object. */
    public AccessControlList acl = null;

    /** This creates an ECS Document. */
    private Document page = null;

    /** Determines if there is information in the document or not. */
    private boolean pageSet = false;

    /**
     * Cache the output stream because it can be used in many
     * different places.
     */
    private PrintWriter out = null;

    /**
     * Determines if there is information in the outputstream or
     * not.
     */
    private boolean outSet = false;

    /**
     * This is what will build the <title></title> of the document.
     */
    private String title = null;

    /**
     * This is the layout that the page will use to render the screen.
     */
    private String layout = null;

    /** This is where output messages from actions should go. */
    private StringElement message = null;

    /**
     * This is a dedicated message class where output messages from
     * actions should go.
     */
    private FormMessages messages = null;

    /** This is a vector to hold critical system errors. */
    private Vector errors = new Vector();

    public String stackTrace = null;
    public Throwable stackTraceException = null;

    /**
     * Holds ServerData (basic properties) about this RunData object.
     */
    private ServerData serverData = null;

    /**
     * Information used by a Template system (such as
     * Velocity/Freemarker).
     */
    private TemplateInfo templateInfo = null;


    /**
     * Put things here and they will be shown on the default Error
     * screen.  This is great for debugging variable values when an
     * exception is thrown.
     */
    public Hashtable varDebug = new Hashtable();

    /**
     * @see #getRemoteAddr()
     */
    private String remoteAddr = null;

    /**
     * @see #getRemoteHost()
     */
    private String remoteHost = null;

    /**
     * @see #getUserAgent()
     */
    private String userAgent = null;


    /**
     * Get the AccessControl object.
     *
     * @return An AccessControlList.
     */
    public AccessControlList getACL()
    {
        return acl;
    }

    /**
     * Set the AccessControl object.
     *
     * @param acl An AccessControlList.
     */
    public void setACL(AccessControlList acl)
    {
        this.acl = acl;
    }

    /**
     * Gets the page.
     *
     * @return A Document.
     */
    public Document getPage()
    {
        pageSet = true;
        if ( this.page == null )
            this.page = new Document();
        return this.page;
    }

    /**
     * Gets the parameters.
     *
     * @return A ParameterParser.
     */
    public ParameterParser getParameters()
    {
        return parameters;
    }

    /**
     * Gets the PrintWriter.  First time calling this will set the
     * PrintWriter via res.getWriter().
     *
     * @return A PrintWriter.
     * @exception IOException.
     */
    public PrintWriter getOut()
        throws IOException
    {
        // Check to see if null first.
        if ( this.out == null )
            setOut(res.getWriter());
        pageSet = false;
        outSet = true;
        return this.out;
    }

    /**
     * Sets the PrintWriter.
     *
     * @param out A PrintWriter.
     */
    public void setOut(PrintWriter out)
    {
        this.out = out;
    }

    /**
     * Sets the parameters.
     *
     * @param params A ParameterParser.
     */
    public void setParameters(ParameterParser params)
    {
        this.parameters = params;
    }

    /**
     * Check to see if page is set.
     *
     * @return True if page is set.
     */
    public boolean isPageSet()
    {
        return pageSet;
    }

    /**
     * Check to see if out is set.
     *
     * @return True if out is set.
     */
    public boolean isOutSet()
    {
        return outSet;
    }

    /**
     * Adds the string to message.  If message has prior messages from
     * other Actions or Screens, this method can be used to chain
     * them.
     *
     * @param msg A String.
     */
    public void addMessage(String msg)
    {
        addMessage(new StringElement(msg));
    }

    /**
     * Adds the ECS Element to message.  If message has prior messages
     * from other Actions or Screens, this method can be used to chain
     * them.
     *
     * @param msg An Element.
     */
    public void addMessage(Element msg)
    {
        if (msg != null)
        {
            if ( message != null )
            {
                message.addElement(msg);
            }
            else
            {
                message = new StringElement(msg);
            }
        }
    }

    /**
     * Get the ACTION= value.  It returns an empty string if null so
     * that it is easy to do conditionals on it based on the
     * equalsIgnoreCase() method.
     *
     * @return A String, "" if null.
     */
    public String getAction()
    {
        return (hasAction() ? this.action : "");
    }

    /**
     * If the Layout has not been defined by the Screen then set the
     * layout to be "DefaultLayout".  The Screen object can also
     * override this method to provide intelligent determination of
     * the Layout to execute.  You can also define that logic here as
     * well if you want it to apply on a global scale.  For example,
     * if you wanted to allow someone to define Layout "preferences"
     * where they could dynamicially change the Layout for the entire
     * site.
     *
     * @return A String.
     */
    public String getLayout()
    {
        if ( this.layout == null )
            return TurbineResources.getString("layout.default",
                                              "DefaultLayout");
        return this.layout;
    }

    /**
     * The results of an Action or other message to be displayed
     * should be placed here.
     *
     * @return A String.
     */
    public String getMessage()
    {
        return (this.message == null ? null : this.message.toString());
    }

    /**
     * Returns a FormMessages object where all the messages to the
     * user should be stored.
     *
     * @return A FormMessages.
     */
    public FormMessages getMessages()
    {
        return this.messages;
    }

    /**
     * Returns an array of system errors.
     *
     * @return A SystemError[].
     */
    public SystemError[] getSystemErrors()
    {
        // SystemError[] result = new SystemError[errors.size()];
        // errors.copyInto(result);
        return toSystemErrorArray(errors);
    }

    /**
     * Utility to copy Vector of known InternetAddress objects into an
     * array.  This method is here to simulate a method available in
     * jdk1.2 Vector.
     *
     * @param v A Vector.
     * @return A SystemError[].
     */
    private SystemError[] toSystemErrorArray(Vector v)
    {
        int size = v.size();
        SystemError[] se = new SystemError[size];
        for (int i=0; i<size; i++)
        {
            se[i] = (SystemError)v.elementAt(i);
        }
        return se;
    }

    /**
     * The results of an Action or other message to be displayed
     * should be placed here.
     *
     * @return A StringElement.
     */
    public StringElement getMessageAsHTML()
    {
        return this.message;
    }

    /**
     * The screen to execute.
     *
     * @return A String.
     */
    public String getScreen()
    {
        return (hasScreen() ? this.screen : "");
    }

    /**
     * Gets JNDI Contexts.
     *
     * @return A Hashtable.
     */
    public Hashtable getJNDIContexts()
    {
        if ( jndiContexts == null )
            jndiContexts = new Hashtable();
        return jndiContexts;
    }

    /**
     * Sets JNDI Contexts.
     *
     * @param contexts A Hashtable.
     */
    public void setJNDIContexts(Hashtable contexts)
    {
        this.jndiContexts = contexts;
    }

    /**
     * Gets the cached script name that is stored in the ServerData
     * object.
     *
     * @return A String.
     */
    public String getScriptName()
    {
        return getServerData().getScriptName();
    }

    /**
     * Sets the cached script name that is stored in the ServerData
     * object.
     *
     * @param sn A String.
     */
    public void setScriptName(String sn)
    {
        getServerData().setScriptName(sn);
    }

    /**
     * Gets the cached serverName that is stored in the ServerData
     * object.
     *
     * @return A String.
     */
    public String getServerName()
    {
        return getServerData().getServerName();
    }

    /**
     * Sets the cached serverName that is stored in the ServerData
     * object.
     *
     * @param sn A String.
     */
    public void setServerName(String sn)
    {
        getServerData().setServerName(sn);
    }

    /**
     * Gets the cached serverPort that is stored in the ServerData
     * object.
     *
     * @return An int.
     */
    public int getServerPort()
    {
        return getServerData().getServerPort();
    }

    /**
     * Sets the cached serverPort that is stored in the ServerData
     * object.
     *
     * @param port An int.
     */
    public void setServerPort(int port)
    {
        getServerData().setServerPort(port);
    }

    /**
     * Gets the cached serverScheme that is stored in the ServerData
     * object.
     *
     * @return A String.
     */
    public String getServerScheme()
    {
        return getServerData().getServerScheme();
    }

    /**
     * Sets the cached serverScheme that is stored in the ServerData
     * object.
     *
     * @param ss A String.
     */
    public void setServerScheme(String ss)
    {
        getServerData().setServerScheme(ss);
    }

    /**
     * Gets the Servlet Session information.
     *
     * @return An HttpSession.
     */
    public HttpSession getSession()
    {
        return session;
    }

    /**
     * Sets the Servlet Session information.
     *
     * @param sess An HttpSession.
     */
    public void setSession(HttpSession sess)
    {
        this.session = sess;
    }

    /**
     * Get the charset.  If it has not already been defined with
     * setCharSet(), then attempt to get it from the
     * TurbineResources.properties file under the property name of
     * "locale.default.charset".
     *
     * @return A String.
     */
    public String getCharSet()
    {
        if (this.charSet == null)
            return TurbineResources.getString("locale.default.charset", null);
        return this.charSet;
    }

    /**
     * Set the charset.
     *
     * @param lcharset A String.
     */
    public void setCharSet(String lcharset)
    {
        this.charSet = lcharset;
    }

    /**
     * Gets the HTTP content type to return.
     *
     * @return A String.
     */
    public String getContentType()
    {
        if ( this.contentType == null )
            return "";
        String tmpCharset = getCharSet();
        if (tmpCharset == null)
        {
            return contentType;
        }
        else
        {
            return contentType + "; charset=" + tmpCharset;
        }
    }

    /**
     * Sets the HTTP content type to return.
     *
     * @param ct A String.
     */
    public void setContentType(String ct)
    {
        this.contentType = ct;
    }

    /**
     * Gets the HTTP status code to return.
     *
     * @return An int.
     */
    public int getStatusCode()
    {
        return statusCode;
    }

    /**
     * Sets the HTTP status code to return.
     *
     * @param sc An int.
     */
    public void setStatusCode(int sc)
    {
        this.statusCode = sc;
    }

    /**
     * Get the redirect URI.  If this is set, also make sure to set
     * the status code to 302.
     *
     * @return A String, "" if null.
     */
    public String getRedirectURI()
    {
        return (this.redirectURI == null ? "" : redirectURI);
    }

    /**
     * Sets the redirect uri.  If this is set, also make sure to set
     * the status code to 302.
     *
     * @param ruri A String.
     */
    public void setRedirectURI(String ruri)
    {
        this.redirectURI = ruri;
    }

    /**
     * Gets the title of the page.
     *
     * @return A String.
     */
    public String getTitle()
    {
        return (this.title == null ? "" : this.title);
    }

    /**
     * Return the user object.
     *
     * @return A User.
     */
    public User getUser()
    {
        return this.user;
    }

    /**
     * Set the user object.
     *
     * @param user A User.
     */
    public void setUser(User user)
    {
        this.user = user;
    }

    /**
     * Whether or not an action has been defined.
     *
     * @return True if an action has been defined.
     */
    public boolean hasAction()
    {
        return ( this.action != null &&
                 this.action.length() > 0 &&
                 !this.action.equalsIgnoreCase("null") );
    }

    /**
     * Whether or not a message has been defined.
     *
     * @return True if a message has been defined.
     */
    public boolean hasMessage()
    {
        return ( this.message != null &&
                 this.message.toString().length() > 0 );
    }

    /**
     * Whether or not a screen has been defined.
     *
     * @return True if a screen has been defined.
     */
    public boolean hasScreen()
    {
        return ( this.screen != null &&
                 this.screen.length() > 0 );
    }

    /**
     * Set the action for the request.
     *
     * @param action A String.
     */
    public void setAction (String action )
    {
        this.action = action;
    }

    /**
     * Set the layout for the request.
     *
     * @param layout A String.
     */
    public void setLayout ( String layout )
    {
        this.layout = layout;
    }

    /**
     * Set the message for the request.
     *
     * @param msg A String.
     */
    public void setMessage (String msg )
    {
        this.message = new StringElement(msg);
    }

    /**
     * Set the FormMessages object for the request.
     *
     * @param msgs A FormMessages.
     */
    public void setMessages (FormMessages msgs)
    {
        this.messages = msgs;
    }

    /**
     * Add critical system error.
     *
     * @param err A SystemError.
     */
    public void setSystemError (SystemError err)
    {
        this.errors.addElement( err );
    }

    /**
     * Set the message for the request.
     *
     * @param msg An Element.
     */
    public void setMessage (Element msg )
    {
        this.message = new StringElement(msg);
    }

    /**
     * Unset the message for the request.
     */
    public void unsetMessage ( )
    {
        this.message = null;
    }

    /**
     * Set the screen for the request.
     *
     * @param screen A String.
     */
    public void setScreen (String screen )
    {
        this.screen = screen;
    }

    /**
     * Set the title for the request.
     *
     * @param title A String.
     */
    public void setTitle (String title )
    {
        this.title = title;
    }

    /**
     * See if a User object exists in this session.
     *
     * @return True if a User object exists in this session.
     */
    public boolean userExists()
    {
        user = getUserFromSession();
        return ( user != null );
    }

    /**
     * Attempts to get the User object from the session.  If it does
     * not exist, it returns null.
     *
     * @return A User.
     */
    public User getUserFromSession()
    {
        return getUserFromSession(session);
    }

    /**
     * Attempts to get the User object from the session.  If it does
     * not exist, it returns null.
     *
     * @param session An HttpSession.
     * @return A User.
     */
    public static User getUserFromSession(HttpSession session)
    {
        try
        {
            return (User) session.getValue(User.SESSION_KEY);
        }
        catch ( ClassCastException e )
        {
            return null;
        }
    }

    /**
     * Allows one to invalidate the user in the default session.
     *
     * @return True if user was invalidated.
     */
    public boolean removeUserFromSession()
    {
        return removeUserFromSession(session);
    }

    /**
     * Allows one to invalidate the user in a session.
     *
     * @param session An HttpSession.
     * @return True if user was invalidated.
     */
    public static boolean removeUserFromSession(HttpSession session)
    {
        try
        {
            session.removeValue(User.SESSION_KEY);
        }
        catch ( Exception e )
        {
            return false;
        }
        return true;
    }

    /**
     * Pulls a User object from the session and increments the access
     * counter and sets the last access date for the object.
     */
    public void populate()
    {
        user = getUserFromSession();

        if ( user != null )
        {
            user.setLastAccessDate();
            user.incrementAccessCounter();
            user.incrementAccessCounterForSession();
        }
    }

    /**
     * Saves a User object into the session.
     */
    public void save()
    {
        session.putValue(User.SESSION_KEY, (Object) user );
    }

    /**
     * Gets the HttpServletRequest object.
     *
     * @return An HttpServletRequest.
     */
    public HttpServletRequest getRequest()
    {
        return this.req;
    }

    /**
     * Gets the HttpServletResponse object.
     *
     * @return An HttpServletResponse.
     */
    public HttpServletResponse getResponse()
    {
        return this.res;
    }

    /**
     * Sets the HttpServletRequest object.
     *
     * @param req An HttpServletRequest.
     */
    public void setRequest(HttpServletRequest req)
    {
        this.req = req;
    }

    /**
     * Sets the HttpServletResponse object.
     *
     * @param res An HttpServletResponse.
     */
    public void setResponse(HttpServletResponse res)
    {
        this.res = res;
    }

    /**
     * Set the Servlet Configuration used during servlet init.
     *
     * @param config A ServletConfig.
     */
    public void setServletConfig (ServletConfig config)
    {
        this.config = config;
        if (config == null)
        {
            this.servletContext = null;
        }
        else
        {
            this.servletContext = config.getServletContext();
        }
    }

    /**
     * Get the Servlet Configuration used during servlet init.
     *
     * @return A ServletConfig.
     */
    public ServletConfig getServletConfig()
    {
        return this.config;
    }

    /**
     * Get the Servlet Context used during servlet init.
     *
     * @return A ServletContext.
     */
    public ServletContext getServletContext()
    {
        return this.servletContext;
    }

    /**
     * Get the ServerData used by this RunData.
     *
     * @return A ServerData.
     */
    public ServerData getServerData()
    {
        return this.serverData;
    }

    /**
     * Set the ServerData used by this RunData.
     *
     * @param serverData A ServerData.
     */
    public void setServerData( ServerData serverData )
    {
        this.serverData = serverData;
    }

    /**
     * Return the TemplateInfo object.  Create a new one if needed.
     *
     * @return A TemplateInfo.
     */
    public TemplateInfo getTemplateInfo()
    {
        if ( templateInfo == null )
        {
            templateInfo = new TemplateInfo( this );
        }
        return templateInfo;
    }

    /**
     * Returns the Internet Protocol (IP) address of the client that
     * sent the request.
     *
     * @return A String.
     */
    public String getRemoteAddr()
    {
        if ( this.remoteAddr == null )
        {
            this.remoteAddr = this.getRequest().getRemoteAddr();
        }

        return this.remoteAddr;
    }

    /**
     * Returns the fully qualified name of the client that sent the
     * request.
     *
     * @return A String.
     */
    public String getRemoteHost()
    {
        if ( this.remoteHost == null )
        {
            this.remoteHost = this.getRequest().getRemoteHost();
        }

        return this.remoteHost;
    }

    /**
     * Get the user agent for this RunData.
     *
     * @return A String.
     */
    public String getUserAgent()
    {
        if ( this.userAgent == null )
        {
            this.userAgent = this.getRequest().getHeader( "User-agent" );
        }

        return this.userAgent;
    }
}
