package org.apache.turbine.util.db;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.util.*;
import java.io.*;

// Turbine Classes
import org.apache.turbine.util.*;
import org.apache.turbine.util.db.pool.DBBroker;
import org.apache.turbine.util.db.map.*;
import java.math.BigDecimal;

/**
 * This is a utility class that is used for retrieving different types
 * of values from a hashtable based on a simple name string.  This
 * class is meant to minimize the amount of casting that needs to be
 * done when working with Hashtables.
 *
 * NOTE: other methods will be added as needed and as time permits.
 *
 * @author <a href="mailto:frank.kim@clearink.com">Frank Y. Kim</a>
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @author <a href="mailto:bmclaugh@algx.net">Brett McLaughlin</a>
 * @version $Id: Criteria.java,v 1.3 2000/09/05 18:50:37 gonzalo Exp $
 */
public class Criteria extends Hashtable // extends Vector
{
    public static final String EQUAL = "=";
    public static final String NOT_EQUAL = "<>";
    public static final String ALT_NOT_EQUAL = "!=";
    public static final String GREATER_THAN = ">";
    public static final String LESS_THAN = "<";
    public static final String GREATER_EQUAL = ">=";
    public static final String LESS_EQUAL = "<=";
    public static final String LIKE = " LIKE ";
    // public static final String STRING_EQUAL = "='";
    public static final String CUSTOM = "CUSTOM";
    // public static final String FOREIGN_KEY = "fk";
    public static final String DISTINCT = "DISTINCT ";
    public static final String IN = " IN ";
    public static final String NOT_IN = " NOT IN ";
    public static final String ALL = "ALL ";

    private boolean ignoreCase = false;
    private boolean singleRecord = false;
    private boolean cascade = false;
    private StringStackBuffer selectModifiers = new StringStackBuffer();
    private StringStackBuffer selectColumns = new StringStackBuffer();
    private StringStackBuffer orderByColumns = new StringStackBuffer();

    /** The name of the database. */
    private String dbName = DBBroker.DEFAULT;

    /** The name of the MapBuilder. */
    private String dbMap = DBBroker.DEFAULT;

    /**
     * To limit the number of rows to return.  -1 means return all
     * rows.
     */
    private int limit = -1;

    /** To start the results at a row other than the first one. */
    private int offset = 0;

    // public static final String CUSTOM = "true";


    /**
     * Constructor.
     */
    public Criteria()
    {
        super(10);
    }

    /**
     * Constructor.
     *
     * @param initialCapacity An int.
     */
    public Criteria(int initialCapacity)
    {
        super(initialCapacity);
    }

    /**
     * Constructor.
     *
     * @param dbName A String.
     */
    public Criteria(String dbName)
    {
        super(10);
        this.dbName = dbName;
    }

    /**
     * Constructor.
     *
     * @param dbName A String.
     * @param initialCapacity An int.
     */
    public Criteria(String dbName,
                    int initialCapacity)
    {
        super(initialCapacity);
        this.dbName = dbName;
    }

    /**
     * Does this Criteria Object contain the specified key?
     *
     * @param table The name of the table.
     * @param column The name of the column.
     * @return True if this Criteria Object contain the specified key.
     */
    public boolean containsKey( String table,
                                String column )
    {
        return containsKey(table + "." + column);
    }

    /**
     * Convenience method to return value as a boolean.
     *
     * @param key String name of column.
     * @return A boolean.
     */
    public boolean getBoolean(String name)
    {
        return ((Boolean)getCObject(name).getValue()).booleanValue();
    }

    /**
     * Convenience method to return value as a boolean.
     *
     * @param table String name of table.
     * @param column String name of column.
     * @return A boolean.
     */
    public boolean getBoolean(String table,
                              String column)
    {
        return getBoolean( new StringBuffer(table).append('.').append(column)
            .toString() );
    }

    /**
     * Returns true if any of the tables in the criteria contain an
     * Object column.
     *
     * @return A boolean.
     * @exception Exception, a generic exception.
     */
    public boolean containsObjectColumn() throws Exception
    {
        return containsObjectColumn(dbMap);
    }

    /**
     * Returns true if any of the tables in the criteria contain an
     * Object column.
     *
     * @param databaseMapName A String.
     * @return A boolean.
     * @exception Exception, a generic exception.
     */
    public boolean containsObjectColumn(String databaseMapName)
        throws Exception
    {
        DatabaseMap map =
            DBBroker.getInstance().getDatabaseMap(databaseMapName);
        StringStackBuffer tables = new StringStackBuffer();
        for (Enumeration e = super.elements(); e.hasMoreElements(); )
        {
            CriteriaObject co = (CriteriaObject)e.nextElement();
            String tableName = co.getTable();
            if (!tables.contains(tableName))
            {
                if (map.getTable(tableName).containsObjectColumn())
                  return true;
                tables.add(tableName);
            }
        }
        return false;
    }

    /**
     * Method to return criteria related to columns in a table.
     *
     * @param name String name of column.
     * @return A CriteriaObject.
     */
    private CriteriaObject getCObject(String name)
    {
        return (CriteriaObject)super.get(name);
    }

    /**
     * Method to return criteria related to a column in a table.
     *
     * @param table String name of table.
     * @param column String name of column.
     * @return A CriteriaObject.
     */
    private CriteriaObject getCObject(String table,
                                      String column)
    {
        return getCObject( new StringBuffer(table).append('.').append(column)
            .toString() );
    }

    /**
     * Method to return a String table name.
     *
     * @param name A String with the name of the key.
     * @return A String with the value of the object at key.
     */
     public String getColumnName(String name)
    {
        return getCObject(name).column;
    }

    /**
     * Method to return a comparison String.
     *
     * @param key String name of key.
     * @return A String with the value of the object at key.
     */
    public String getComparison(String key)
    {
        return getCObject(key).comparison;
    }

    /**
     * Method to return a comparison String.
     *
     * @param table String name of table.
     * @param column String name of column.
     * @return A String with the value of the object at key.
     */
    public String getComparison(String table,
                                String column)
    {
        return getComparison( new StringBuffer(table)
            .append('.').append(column)
            .toString() );
    }

    /**
     * Convenience method to return a Date.
     *
     * @param table String name.
     * @return A java.util.Date with the value of object at key.
     */
    public java.util.Date getDate(String name)
    {
        return (java.util.Date)getCObject(name).getValue();
    }

    /**
     * Convenience method to return a Date.
     *
     * @param table String name of table.
     * @param column String name of column.
     * @return A java.util.Date with the value of object at key.
     */
    public java.util.Date getDate(String table,
                                  String column)
    {
        return getDate( new StringBuffer(table).append('.').append(column)
            .toString() );
    }

    /**
     * Get the database name for this criteria object.
     *
     * @return A String with the database name.  By default, this is
     * DBBroker.DEFAULT.
     */
    public String getDbName()
    {
        if ( this.dbName == null )
            return DBBroker.DEFAULT;
        return dbName;
    }

    /**
     * Set the database name for this criteria object.
     *
     * @param name A String with the database name.
     */
    public void setDbName(String name)
    {
        this.dbName = name;
    }

    /**
     * Get the DatabaseMap name.
     *
     * @return A String with the DatabaseMap name.  By default, this
     * is DBBroker.DEFAULT.
     */
    public String getMapName()
    {
        if ( this.dbMap == null )
            return DBBroker.DEFAULT;
        return dbMap;
    }

    /**
     * Set the DatabaseMap name.
     *
     * @param map A String with the DatabaseMap name.
     */
    public void setMapName(String map)
    {
        this.dbMap = map;
    }

    /**
     * Convenience method to return a double.
     *
     * @param name A String with the name of the key.
     * @return A double with the value of object at key.
     */
    public double getDouble(String name)
    {
        Object obj = getCObject(name).getValue();
        if (obj instanceof String)
            return new Double((String)obj).doubleValue();
        return ((Double) obj).doubleValue();
    }

    /**
     * Convenience method to return a double.
     *
     * @param table String name of table.
     * @param column String name of column.
     * @return A double with the value of object at key.
     */
    public double getDouble(String table,
                            String column)
    {
        return getDouble( new StringBuffer(table).append('.').append(column)
            .toString() );
    }

    /**
     * Convenience method to return a float.
     *
     * @param name A String with the name of the key.
     * @return A float with the value of object at key.
     */
    public float getFloat(String name)
    {
        Object obj = getCObject(name).getValue();
        if (obj instanceof String)
            return new Float((String)obj).floatValue();
        return ((Float) obj).floatValue();
    }

    /**
     * Convenience method to return a float.
     *
     * @param table String name of table.
     * @param column String name of column.
     * @return A float with the value of object at key.
     */
    public float getFloat(String table,
                          String column)
    {
        return getFloat( new StringBuffer(table).append('.').append(column)
            .toString() );
    }

    /**
     * Convenience method to return an Integer.
     *
     * @param name A String with the name of the key.
     * @return An Integer with the value of object at key.
     */
    public Integer getInteger(String name)
    {
        Object obj = getCObject(name).getValue();
        if (obj instanceof String)
            return new Integer((String)obj);
        return ((Integer)obj);
    }

    /**
     * Convenience method to return an Integer.
     *
     * @param table String name of table.
     * @param column String name of column.
     * @return An Integer with the value of object at key.
     */
    public Integer getInteger(String table,
                              String column)
    {
        return getInteger( new StringBuffer(table).append('.').append(column)
            .toString() );
    }

    /**
     * Convenience method to return an int.
     *
     * @param name A String with the name of the key.
     * @return An int with the value of object at key.
     */
    public int getInt(String name)
    {
        Object obj = getCObject(name).getValue();
        if (obj instanceof String)
            return new Integer((String)obj).intValue();
        return ((Integer)obj).intValue();
    }

    /**
     * Convenience method to return an int.
     *
     * @param table String name of table.
     * @param column String name of column.
     * @return An int with the value of object at key.
     */
    public int getInt(String table,
                      String column)
    {
        return getInt( new StringBuffer(table).append('.').append(column)
            .toString() );
    }

    /**
     * Convenience method to return a BigDecimal.
     *
     * @param name A String with the name of the key.
     * @return A BigDecimal with the value of object at key.
     */
    public BigDecimal getBigDecimal(String name)
    {
        Object obj = getCObject(name).getValue();
        if (obj instanceof String)
            return new BigDecimal((String)obj);
        return (BigDecimal)obj;
    }

    /**
     * Convenience method to return a BigDecimal.
     *
     * @param table String name of table.
     * @param column String name of column.
     * @return A BigDecimal with the value of object at key.
     */
    public BigDecimal getBigDecimal(String table,
                                    String column)
    {
        return getBigDecimal( new StringBuffer(table)
            .append('.').append(column)
            .toString() );
    }

    /**
     * Convenience method to return a long.
     *
     * @param name A String with the name of the key.
     * @return A long with the value of object at key.
     */
    public long getLong(String name)
    {
        Object obj = getCObject(name).getValue();
        if (obj instanceof String)
            return new Long((String)obj).longValue();
        return ((Long) obj).longValue();
    }

    /**
     * Convenience method to return a long.
     *
     * @param table String name of table.
     * @param column String name of column.
     * @return A long with the value of object at key.
     */
    public long getLong(String table,
                        String column)
    {
        return getLong( new StringBuffer(table).append('.').append(column)
            .toString() );
    }

    /**
     * Convenience method to return a String.
     *
     * @param name A String with the name of the key.
     * @return A String with the value of object at key.
     */
    public String getString(String name)
    {
        return (String) getCObject(name).getValue();
    }

    /**
     * Convenience method to return a String.
     *
     * @param table String name of table.
     * @param column String name of column.
     * @return A String with the value of object at key.
     */
    public String getString(String table,
                            String column)
    {
        return getString( new StringBuffer(table).append('.').append(column)
            .toString() );
    }

    /**
     * Method to return a String table name.
     *
     * @param name A String with the name of the key.
     * @return A String with the value of object at key.
     */
     public String getTableName(String name)
    {
        return getCObject(name).getTable();
    }

    /**
     * Convenience method to return a Vector.
     *
     * @param name A String with the name of the key.
     * @return A Vector with the value of object at key.
     */
    public Vector getVector(String name)
    {
        return (Vector) getCObject(name).getValue();
    }

    /**
     * Convenience method to return a String.
     *
     * @param table String name of table.
     * @param column String name of column.
     * @return A String with the value of object at key.
     */
    public Vector getVector(String table,
                            String column)
    {
        return getVector( new StringBuffer(table).append('.').append(column)
            .toString() );
    }

    /**
     * Method to return the value that was added to Criteria.
     *
     * @param name A String with the name of the key.
     * @return An Object with the value of object at key.
     */
    public Object getValue( String name )
    {
        return getCObject(name).getValue();
    }

    /**
     * Method to return the value that was added to Criteria.
     *
     * @param table String name of table.
     * @param column String name of column.
     * @return An Object with the value of object at key.
     */
    public Object getValue( String table,
                            String column )
    {
        return getValue( new StringBuffer(table).append('.').append(column)
            .toString() );
    }

    /**
     * Overrides Hashtable get, so that the value placed in the
     * CriteriaObject is returned instead of the CriteriaObject.
     *
     * @param key An Object.
     * @return An Object.
     */
    public Object get( Object key )
    {
        return getValue( (String)key );
    }

    /**
     * Overrides Hashtable put, so that the value placed in the
     * CriteriaObject is returned instead of the CriteriaObject.
     *
     * @param key An Object.
     * @param value An Object.
     * @return An Object.
     */
    public Object put( Object key, Object value )
    {
        if ( value instanceof CriteriaObject )
        {
            return super.put(key, value);
        }
        return super.put( key,
                          new CriteriaObject((String)key, value, EQUAL) );
    }

    /**
     * This method is a slightly faster alternative to using the
     * method put().  It also allows code that looks like this:
     *
     * <p>
     * <code>
     * Criteria crit = new Criteria().add ( &quot;name&quot;,
     *                                      &quot;value&quot; );
     * </code>
     *
     * @param key A String.
     * @param value An Object.
     * @return A Criteria.
     */
    public Criteria add ( String key,
                          Object value )
    {
        add(key, value, EQUAL);
        return this;
    }

    /**
     * This method is a slightly faster alternative to using the
     * method put().  It also allows code that looks like this:
     *
     * <p>
     * <code>
     * Criteria crit = new Criteria().add ( &quot;name&quot;,
     *                                      &quot;value&quot; );
     * </code>
     *
     * @param key A String.
     * @param value An Object.
     * @param comparison A String.
     * @return A Criteria.
     */
    public Criteria add ( String key,
                          Object value,
                          String comparison )
    {
        super.put( key, new CriteriaObject(key, value, comparison) );
        return this;
    }

    /**
     * This method is a slightly faster alternative to using the
     * method put().  It also allows code that looks like this:
     *
     * <p>
     * <code>
     * Criteria crit = new Criteria().add ( &quot;name&quot;,
     *                                      &quot;value&quot; );
     * </code>
     *
     * @param table String name of table.
     * @param column String name of column.
     * @param value An Object.
     * @return A Criteria.
     */
    public Criteria add ( String table,
                          String column,
                          Object value )
    {
        add(table, column, value, EQUAL);
        return this;
    }

    /**
     * This method is a slightly faster alternative to using the
     * method put().  It also allows code that looks like this:
     *
     * <p>
     * <code>
     * Criteria crit = new Criteria().add ( &quot;name&quot;,
     *                                      &quot;value&quot; );
     * </code>
     *
     * @param table String name of table.
     * @param column String name of column.
     * @param value An Object.
     * @param comparison A String.
     * @return A Criteria.
     */
    public Criteria add ( String table,
                          String column,
                          Object value,
                          String comparison )
    {
        super.put( new StringBuffer(table).append('.')
            .append(column).toString(),
                   new CriteriaObject(table, column, value, comparison) );
        return this;
    }

    /**
     * Convenience method to add a boolean to Criteria.
     *
     * @param key A String value to use as key.
     * @param value A Boolean.
     * @return A modified Criteria object.
     */
    public Criteria add ( String key,
                          boolean value )
    {
        add( key, new Boolean(value) );
        return this;
    }
    /**
     * Convenience method to add a boolean to Criteria.
     *
     * @param key A String value to use as key.
     * @param value A Boolean.
     * @param comparison A String.
     * @return A modified Criteria object.
     */
    public Criteria add ( String key,
                          boolean value,
                          String comparison )
    {
        add( key, new Boolean(value), comparison );
        return this;
    }

    /**
     * Convenience method to add an int to Criteria.
     *
     * @param key A String value to use as key.
     * @param value An int.
     * @return A modified Criteria object.
     */
    public Criteria add ( String key,
                          int value )
    {
        add( key, new Integer(value) );
        return this;
    }

    /**
     * Convenience method to add an int to Criteria.
     *
     * @param key A String value to use as key.
     * @param value An int.
     * @param comparison A String.
     * @return A modified Criteria object.
     */
    public Criteria add ( String key,
                          int value,
                          String comparison )
    {
        add( key, new Integer(value), comparison );
        return this;
    }

    /*
     * Convenience method to add a long to Criteria.
     *
     * @param key A String value to use as key.
     * @param value A long.
     * @return A modified Criteria object.
     */
    public Criteria add(String key,
                        long value)
    {
        add(key, new Long(value));
        return this;
    }

    /*
     * Convenience method to add a long to Criteria
     *
     * @param key A String value to use as key.
     * @param value A long.
     * @param comparison A String.
     * @return A modified Criteria object.
     */
    public Criteria add(String key,
                        long value,
                        String comparison)
    {
        add(key, new Long(value), comparison);
        return this;
    }

    /*
     * Convenience method to add a float to Criteria.
     *
     * @param key A String value to use as key.
     * @param value A float.
     * @return A modified Criteria object.
     */
    public Criteria add(String key,
                        float value)
    {
        add(key, new Float(value));
        return this;
    }

    /*
     * Convenience method to add a float to Criteria
     *
     * @param key A String value to use as key.
     * @param value A float.
     * @param comparison A String.
     * @return A modified Criteria object.
     */
    public Criteria add(String key,
                        float value,
                        String comparison)
    {
        add(key, new Float(value), comparison);
        return this;
    }

    /*
     * Convenience method to add a double to Criteria.
     *
     * @param key A String value to use as key.
     * @param value A double.
     * @return A modified Criteria object.
     */
    public Criteria add(String key,
                        double value)
    {
        add(key, new Double(value));
        return this;
    }

    /*
     * Convenience method to add a double to Criteria
     *
     * @param key A String value to use as key.
     * @param value A double.
     * @param comparison A String.
     * @return A modified Criteria object.
     */
    public Criteria add(String key,
                        double value,
                        String comparison)
    {
        add(key, new Double(value), comparison);
        return this;
    }

    /**
     * This method lets you put a Date object into the criteria object
     * by specifying the year, month, and date.
     *
     * @param key A String value to use as key.
     * @param year An int with the year.
     * @param month An int with the month.
     * @param date An int with the date.
     * @return A modified Criteria object.
     */
    public Criteria addTime( String key,
                             int year,
                             int month,
                             int date )
    {
        add( key, new GregorianCalendar(year, month, date) );
        return this;
    }

    /**
     * This method lets you put a Date object into the criteria object
     * by specifying the year, month, and date.
     *
     * @param key A String value to use as key.
     * @param year An int with the year.
     * @param month An int with the month.
     * @param date An int with the date.
     * @param comparison A String.
     * @return A modified Criteria object.
     */
    public Criteria addTime( String key,
                             int year,
                             int month,
                             int date,
                             String comparison)
    {
        add( key, new GregorianCalendar(year, month, date), comparison );
        return this;
    }

    /**
     * This is the way that you should add a join of two tables.  For
     * example:
     *
     * <p>
     * AND PROJECT.PROJECT_ID=FOO.PROJECT_ID
     * <p>
     *
     * left = PROJECT.PROJECT_ID
     * right = FOO.PROJECT_ID
     *
     * @param left A String with the left side of the join.
     * @param right A String with the right side of the join.
     * @return A modified Criteria object.
     */
    public Criteria addJoin( String left,
                             String right)
    {
        add(left, (Object)(left + "=" + right), Criteria.CUSTOM);
        return this;
    }

    /**
     * Adds an 'IN' clause with the criteria supplied as an Object
     * array.  For example:
     *
     * <p>
     * FOO.NAME IN ('FOO', 'BAR', 'ZOW')
     * <p>
     *
     * where 'values' contains three objects that evaluate to the
     * respective strings above when .toString() is called.
     *
     * @param key A String value to use as key.
     * @param values An Object[] with the allowed values.
     * @return A modified Criteria object.
     */
    public Criteria addIn(String key,
                          Object[] values)
    {
        add(key, (Object)values, Criteria.IN);
        return this;
    }

    /**
     * Adds an 'IN' clause with the criteria supplied as an int array.
     * For example:
     *
     * <p>
     * FOO.ID IN ('2', '3', '7')
     * <p>
     *
     * where 'values' contains those three integers.
     *
     * @param key A String value to use as key.
     * @param values An int[] with the allowed values.
     * @return A modified Criteria object.
     */
    public Criteria addIn(String key,
                          int[] values)
    {
        add(key, (Object)values, Criteria.IN);
        return this;
    }

    /**
     * Adds an 'IN' clause with the criteria supplied as a Vector.
     * For example:
     *
     * <p>
     * FOO.NAME IN ('FOO', 'BAR', 'ZOW')
     * <p>
     *
     * where 'values' contains three objects that evaluate to the
     * respective strings above when .toString() is called.
     *
     * @param key A String value to use as key.
     * @param values A Vector with the allowed values.
     * @return A modified Criteria object.
     */
    public Criteria addIn(String key,
                          Vector values)
    {
        add(key, (Object)values, Criteria.IN);
        return this;
    }

    /**
     * Adds a 'NOT IN' clause with the criteria supplied as an Object
     * array.  For example:
     *
     * <p>
     * FOO.NAME NOT IN ('FOO', 'BAR', 'ZOW')
     * <p>
     *
     * where 'values' contains three objects that evaluate to the
     * respective strings above when .toString() is called.
     *
     * @param key A String value to use as key.
     * @param values An Object[] with the disallowed values.
     * @return A modified Criteria object.
     */
    public Criteria addNotIn(String key,
                             Object[] values)
    {
        add(key, (Object)values, Criteria.NOT_IN);
        return this;
    }

    /**
     * Adds a 'NOT IN' clause with the criteria supplied as an int
     * array.  For example:
     *
     * <p>
     * FOO.ID NOT IN ('2', '3', '7')
     * <p>
     *
     * where 'values' contains those three integers.
     *
     * @param key A String value to use as key.
     * @param values An int[] with the disallowed values.
     * @return A modified Criteria object.
     */
    public Criteria addNotIn(String key,
                             int[] values)
    {
        add(key, (Object)values, Criteria.NOT_IN);
        return this;
    }

    /**
     * Adds a 'NOT IN' clause with the criteria supplied as a Vector.
     * For example:
     *
     * <p>
     * FOO.NAME NOT IN ('FOO', 'BAR', 'ZOW')
     * <p>
     *
     * where 'values' contains three objects that evaluate to the
     * respective strings above when .toString() is called.
     *
     * @param key A String value to use as key.
     * @param values A Vector with the disallowed values.
     * @return A modified Criteria object.
     */
    public Criteria addNotIn(String key,
                             Vector values)
    {
        add(key, (Object)values, Criteria.NOT_IN);
        return this;
    }

    /**
     * Adds "ALL " to the SQL statement.
     */
    public void setAll()
    {
        selectModifiers.add(ALL);
    }

    /**
     * Adds "DISTINCT " to the SQL statement.
     */
    public void setDistinct()
    {
        selectModifiers.add(DISTINCT);
    }

    /**
     * Sets ignore case.
     *
     * @param b True if case should be ignored.
     * @return A modified Criteria object.
     */
    public Criteria setIgnoreCase(boolean b)
    {
        ignoreCase = b;
        return this;
    }

    /**
     * Is ignore case on or off?
     *
     * @return True if case is ignored.
     */
    public boolean isIgnoreCase()
    {
        return ignoreCase;
    }

    /**
     * Set single record?
     *
     * @param b True if a single record should be returned.
     * @return A modified Criteria object.
     */
    public Criteria setSingleRecord(boolean b)
    {
        singleRecord = b;
        return this;
    }

    /**
     * Is single record?
     *
     * @return True if a single record is being returned.
     */
    public boolean isSingleRecord()
    {
        return singleRecord;
    }

    /**
     * Set cascade.
     *
     * @param b True if cascade is set.
     * @return A modified Criteria object.
     */
    public Criteria setCascade(boolean b)
    {
        cascade = b;
        return this;
    }

    /**
     * Is cascade set?
     *
     * @return True if cascade is set.
     */
    public boolean isCascade()
    {
        return cascade;
    }

    /**
     * Set limit.
     *
     * @param limit An int with the value for limit.
     * @return A modified Criteria object.
     */
    public Criteria setLimit(int limit)
    {
        this.limit = limit;
        return this;
    }

    /**
     * Get limit.
     *
     * @return An int with the value for limit.
     */
    public int getLimit()
    {
        return limit;
    }

    /**
     * Set offset.
     *
     * @param offset An int with the value for offset.
     * @return A modified Criteria object.
     */
    public Criteria setOffset(int offset)
    {
        this.offset = offset;
        return this;
    }

    /**
     * Get offset.
     *
     * @return An int with the value for offset.
     */
    public int getOffset()
    {
        return offset;
    }

    /**
     * Add select column.
     *
     * @param name A String with the name of the select column.
     * @return A modified Criteria object.
     */
    public Criteria addSelectColumn( String name )
    {
        selectColumns.add( name );
        return this;
    }

    /**
     * Get select columns.
     *
     * @return A StringStackBuffer with the name of the select
     * columns.
     */
    public StringStackBuffer getSelectColumns()
    {
        return selectColumns;
    }

    /**
     * Get select modifiers.
     *
     * @return A StringStackBuffer with the select modifiers.
     */
    public StringStackBuffer getSelectModifiers()
    {
        return selectModifiers;
    }

    /**
     * Add order by column name.
     *
     * @param name A String with the name of the order column.
     * @return A modified Criteria object.
     */
    public Criteria addOrderByColumn( String name )
    {
        orderByColumns.add( name );
        return this;
    }

    /**
     * Get order by columns.
     *
     * @return A StringStackBuffer with the name of the order columns.
     */
    public StringStackBuffer getOrderByColumns()
    {
        return orderByColumns;
    }

    /**
     * Remove an object from the criteria.
     *
     * @param key A String with the key to be removed.
     * @return The removed object.
     */
    public Object remove(String key)
    {
        Object foo = super.remove(key);
        if (foo instanceof CriteriaObject)
            return ((CriteriaObject)foo).getValue();
        return foo;
    }

    /**
     * Build a string representation of the Criteria.
     *
     * @return A String with the representation of the Criteria.
     */
    public String toString()
    {
        StringBuffer sb = new StringBuffer("Criteria:: ");
        Enumeration e = keys();
        while (e.hasMoreElements() )
        {
            String key = (String)e.nextElement();
            sb.append(key).append("<=>")
                .append(getValue(key).toString()).append(":  ");
        }
        return sb.toString();
    }

    /**
     * This is a private inner class that describes an object in the
     * criteria.
     */
    private class CriteriaObject
    {
        /** Value of the CO. */
        private Object value = null;

        /** Comparison value. */
        private String comparison = "";

        /** Table name. */
        private String table = "";

        /** Column name. */
        private String column = "";

        /**
         * Constructor.
         *
         * @param table A String with the name of the table.
         * @param column A String with the name of the column.
         * @param val An Object with the value for the Criteria.
         * @param comp A String with the comparison value.
         */
        CriteriaObject( String table,
                        String column,
                        Object val,
                        String comp )
        {
            this.table = table;
            this.column = column;
            value = val;
            comparison = comp;
        }

        /**
         * Constructor.
         *
         * @param tableColumn A String with the full name of the
         * column.
         * @param val An Object with the value for the Criteria.
         * @param comp A String with the comparison value.
         */
        CriteriaObject( String tableColumn,
                        Object val,
                        String comp )
        {
            int dot = tableColumn.indexOf('.');
            if ( dot == -1 )
            {
                column = tableColumn;
            }
            else
            {
                table = tableColumn.substring(0, dot);
                column = tableColumn.substring(dot+1);
            }
            value = val;
            comparison = comp;
        }

        /**
         * Constructor.
         *
         * @param table A String with the name of the table.
         * @param column A String with the name of the column.
         * @param val An Object with the value for the Criteria.
         */
        CriteriaObject( String table,
                        String column,
                        Object val )
        {
            this.table = table;
            this.column = column;
            value = val;
            comparison = EQUAL;
        }

        /**
         * Constructor.
         *
         * @param tableColumn A String with the full name of the
         * column.
         * @param val An Object with the value for the Criteria.
         */
        CriteriaObject( String tableColumn,
                        Object val )
        {
            int dot = tableColumn.indexOf('.');
            if ( dot == -1 )
            {
                column = tableColumn;
            }
            else
            {
                table = tableColumn.substring(0, dot);
                column = tableColumn.substring(dot+1);
            }
            value = val;
            comparison = EQUAL;
        }

        /**
         * Get the column name.
         *
         * @return A String with the column name.
         */
        public String getColumn()
        {
            return this.column;
        }

        /**
         * Get the table name.
         *
         * @return A String with the table name.
         */
        public String getTable()
        {
            return this.table;
        }

        /**
         * Get the comparison.
         *
         * @return A String with the comparison.
         */
        public String getComparison()
        {
            return this.comparison;
        }

        /**
         * Get the value.
         *
         * @return An Object with the value.
         */
        public Object getValue()
        {
            return this.value;
        }

    }
}
