package org.apache.turbine.util.db;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.sql.*;
import java.util.*;
import java.text.SimpleDateFormat;
import java.math.BigDecimal;

// Turbine Utility Classes
import org.apache.turbine.om.peer.BasePeer;
import org.apache.turbine.util.*;
import org.apache.turbine.services.resources.*;
import org.apache.turbine.util.db.*;
import org.apache.turbine.util.db.pool.*;
import org.apache.turbine.util.db.map.*;

// Village Database Support Classes
import com.workingdogs.village.*;

/**
 * This singleton mangages and allocates simple int IDs on a per table
 * basis.  This method of ID generation is used to ensure that code is
 * more database independent.  For example, mySQL has an
 * auto-increment feature while Oracle uses sequences.
 *
 * This class uses the table ID_TABLE defined in docs/*_id_table.sql.
 * The columns in ID_TABLE are used as follows:
 *
 * ID_TABLE_ID - The PK for this row (any unique int).
 * TABLE_NAME - The name of the table you want ids for.
 * NEXT_ID - The next id returned by IDBroker when it queries the
 *           database (not when it returns an id from memory).
 * QUANTITY - The number of ids that IDBroker will cache in memory.
 *
 * Use this class like this:
 *
 * int id = IDBroker.getInstance().getNextId("TABLE_NAME");
 * int[] ids = IDBroker.getInstance().getNextIds("TABLE_NAME",
 *                                               numOfIdsToReturn);
 *
 * NOTE: When the ID_TABLE must be updated we must ensure that
 * IDBroker objects running in different JVMs do not overwrite each
 * other.  Originally I was thinking of locking tables on
 * select/update but have some other ideas now.
 *
 * @author <a href="mailto:frank.kim@clearink.com">Frank Y. Kim</a>
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @version $Id: IDBroker.java,v 1.5 2000/09/05 18:50:37 gonzalo Exp $
 */
public class IDBroker
    implements Runnable
{
    /** The TableMap referencing the ID_TABLE for this IDBroker. */
    private TableMap tableMap;

    /**
     * The cached IDs for each table.
     *
     * Key: String table name.
     * Value: Vector of Integer IDs.
     */
    private Hashtable ids = new Hashtable(40);

    /**
     * The quantity of ids to grab for each table.
     *
     * Key: String table name.
     * Value: Integer quantity.
     */
    private Hashtable quantityStore = new Hashtable(40);

    /**
     * The last time this IDBroker queried the database for ids.
     *
     * Key: String table name.
     * Value: Date of last id request.
     */
    private Hashtable lastQueryTime = new Hashtable(40);

    private static final int sleepPeriod = 1 * 60000;
    private static final float safetyMargin = 1.2f;

    private Thread thread;
    private boolean transactionsSupported = false;

    /**
     * Creates an IDBroker for the ID table.
     *
     * @param tMap A TableMap.
     */
    public IDBroker(TableMap tMap)
    {
        this.tableMap = tMap;
        thread = new Thread(this);
        thread.start();

        // Check for Transaction support.  Give warning message if
        // IDBroker is being used with a database that does not
        // support transactions.
        String dbName = tMap.getDatabaseMap().getName();
        DBConnection dbCon = null;
        try
        {
            dbCon = DBBroker.getInstance().getConnection(dbName);
            transactionsSupported = dbCon.getConnection()
                .getMetaData().supportsTransactions();
        }
        catch (Exception e)
        {
            transactionsSupported = false;
        }
        finally
        {
            try
            {
                // Return the connection to the pool.
                DBBroker.getInstance().releaseConnection(dbCon);
            }
            catch (Exception e)
            {
            }
        }
        if (!transactionsSupported)
        {
            Log.warn("IDBroker is being used with db: " + dbName +
                "\n which does not support transactions.  It is possible to " +
                "\n generate duplicate keys, if multiple JVM's are used or other " +
                "\n means are used to write to the database.");
        }
    }

    /**
     * This method returns the next available id for the given table.
     *
     * @param tableName The name of the table for which we want an id.
     * @return A BigDecimal.
     * @exception Exception, a generic exception.
     */
    public BigDecimal getNextId(String tableName)
        throws Exception
    {
        BigDecimal[] id = getNextIds(tableName, 1);
        return id[0];
    }


    /**
     * This method returns x number of ids for the given table.
     *
     * @param tableName The name of the table for which we want an id.
     * @param numOfIdsToReturn The desired number of ids.
     * @return A BigDecimal.
     * @exception Exception, a generic exception.
     */
    public BigDecimal[] getNextIds(String tableName,
                                   int numOfIdsToReturn)
        throws Exception
    {
        Vector availableIds = (Vector)ids.get(tableName);

        if ( availableIds == null ||
             availableIds.size() < numOfIdsToReturn )
        {
            if (availableIds == null)
                Log.note ("Forced id retrieval - no available vector");
            else
                Log.note ("Forced id retrieval - " + availableIds.size());

            storeIDs(tableName, true);
            availableIds = (Vector)ids.get(tableName);
        }
        return getIds(availableIds, numOfIdsToReturn);
    }

    /**
     * A background thread that tries to ensure that when someone asks
     * for ids, that there are already some loaded and that the
     * database is not accessed.
     */
    public void run()
    {
        Log.note("IDBroker thread was started.");
        while (true)
        {
            // Log.note("IDBroker thread checking for more keys.");
            Enumeration e = ids.keys();
            while (e.hasMoreElements())
            {
                String tableName = (String)e.nextElement();
                Log.note("IDBroker thread checking for more keys on table: " +
                         tableName);
                Vector availableIds = (Vector)ids.get(tableName);
                int quantity = getQuantity(tableName);
                if ( quantity > availableIds.size() )
                {
                    try
                    {
                        // Second parameter is false because we don't
                        // want the quantity to be adjusted for thread
                        // calls.
                        storeIDs(tableName, false);
                        Log.note("Retrieved more ids for table: " +
                                 tableName);
                    }
                    catch (Exception exc)
                    {
                        Log.error("There was a problem getting new IDs for table: " +
                                  tableName, exc);
                    }
                }
            }
            try
            {
                thread.sleep(sleepPeriod);
            }
            catch (Exception exc)
            {
                Log.error(exc);
            }
        }
    }

    /**
     * Check the frequency of retrieving new ids from the database.
     * If the frequency is high then we increase the amount (i.e.
     * quantity column) of ids retrieved on each access.  Tries to
     * alter number of keys grabbed so that IDBroker retrieves a new
     * set of ID's prior to their being needed.
     *
     * @param tableName The name of the table for which we want an id.
     */
    private void checkTiming(String tableName)
    {
        // Check if quantity changing is switched on.
        if (!TurbineResources.getBoolean(TurbineResources
                                         .DB_IDBROKER_CLEVERQUANTITY))
            return;

        // Get the last id request for this table.
        java.util.Date lastTime =
            (java.util.Date)lastQueryTime.get (tableName);
        java.util.Date now = new java.util.Date();

        if ( lastTime != null)
        {
            long thenLong = lastTime.getTime();
            long nowLong = now.getTime();
            int timeLapse = (int)(nowLong-thenLong);
            if ( timeLapse < sleepPeriod )
            {
                Log.note("Unscheduled retrieval of more ids for table: " +
                         tableName);
                // Increase quantity, so that hopefully this does not
                // happen again.
                float rate = (float)getQuantity(tableName)/(float)timeLapse;
                quantityStore.put(tableName,
                    new Integer((int)(sleepPeriod*rate*safetyMargin)));
            }
        }
        lastQueryTime.put (tableName,now);
    }

    /**
     * Grabs more ids from the id_table and stores it in the ids
     * Hashtable.  If adjustQuantity is set to true the amount of id's
     * retrieved for each call to storeIDs will be adjusted.
     *
     * @param tableName The name of the table for which we want an id.
     * @param adjustQuantity True if amount should be adjusted.
     * @exception Exception, a generic exception.
     */
    private void storeIDs(String tableName,
                          boolean adjustQuantity)
        throws Exception
    {
        int nextId = 0;
        int quantity = 0;
        DatabaseMap dbMap = tableMap.getDatabaseMap();
        TableMap tMap = dbMap.getTable(tableName);

        // Block on the table.  Multiple tables are allowed to ask for
        // ids simultaneously.
        synchronized(tMap)
        {
            if (adjustQuantity)
                checkTiming(tableName);

            DBConnection dbCon = null;
            try
            {
                String databaseName = dbMap.getName();

                // Get a connection to the db by starting a
                // transaction.
                if (transactionsSupported)
                {
                    dbCon = BasePeer.beginTransaction(databaseName);
                }
                else
                {
                    dbCon = DBBroker.getInstance().getConnection(databaseName);
                }
                Connection connection = dbCon.getConnection();

                // Write the current value of quantity of keys to grab
                // to the database, primarily to obtain a write lock
                // on the table/row, but this value will also be used
                // as the starting value when an IDBroker is
                // instantiated.
                quantity = getQuantity(tableName);
                Criteria criteria = new Criteria(2)
                    .add( "ID_TABLE.QUANTITY", quantity );
                Criteria selectCriteria = new Criteria(2)
                    .add( "ID_TABLE.TABLE_NAME", tableName );
                criteria.setMapName(dbMap.getName());
                selectCriteria.setMapName(dbMap.getName());
                BasePeer.doUpdate( selectCriteria, criteria, dbCon );

                // Read the next starting ID from the ID_TABLE.
                int[] results = selectRow(connection, tableName);
                nextId = results[0]; // NEXT_ID column

                // Update the row based on the quantity in the
                // ID_TABLE.
                int newNextId = nextId + quantity;
                updateRow(connection, tableName, newNextId);

                if (transactionsSupported)
                {
                    BasePeer.commitTransaction(dbCon);
                }
            }
            catch (Exception e)
            {
                if (transactionsSupported)
                {
                    BasePeer.rollBackTransaction(dbCon);
                }
                throw e;
            }
            finally
            {
                if (!transactionsSupported)
                {
                    // Return the connection to the pool.
                    DBBroker.getInstance().releaseConnection(dbCon);
                }
            }


            Vector availableIds = (Vector)ids.get(tableName);
            if ( availableIds == null )
            {
                availableIds = new Vector();
                ids.put( tableName, availableIds );
            }

            // Create the ids and store them in the Vector.
            for (int i=0; i<quantity; i++)
            {
                availableIds.addElement( new BigDecimal((double)nextId + i) );
            }
        }
    }


    /**
     * This method allows you to change the number of ids that are
     * cached in memory (ie the value in ID_TABLE.QUANTITY).
     *
     * @param tableName The name of the table we want to update.
     * @param quantity The default number of ids that are retrieved.
     * @exception Exception, a generic exception.
     */
    public void setQuantity(String tableName,
                            int quantity)
        throws Exception
    {
        Criteria criteria = new Criteria(1);
        Criteria selectCriteria = new Criteria(1);

        criteria.add( "ID_TABLE.QUANTITY", new Integer(quantity) );
        selectCriteria.add( "ID_TABLE.TABLE_NAME", tableName );

        BasePeer.doUpdate( selectCriteria, criteria );
    }

    /**
     * This method allows you to get the number of ids that are to be
     * cached in memory.  This is either stored in quantityStore or
     * read from the db. (ie the value in ID_TABLE.QUANTITY).
     *
     * @param tableName The name of the table we want to query.
     * @return An int with the number of ids cached in memory.
     */
    private int getQuantity(String tableName)
    {
        int quantity = -1;

        // Initialize quantity, if necessary.
        if (quantityStore.containsKey(tableName))
        {
            quantity = ((Integer)quantityStore.get(tableName)).intValue();
        }
        else
        {
            DBConnection dbCon = null;
            try
            {
                String databaseName = tableMap.getDatabaseMap().getName();

                // Get a connection to the db by starting a
                // transaction.
                dbCon = DBBroker.getInstance().getConnection(databaseName);
                Connection connection = dbCon.getConnection();

                // Read the row from the ID_TABLE.
                int[] results = selectRow(connection, tableName);

                // QUANTITY column.
                quantity = results[1];
                quantityStore.put(tableName, new Integer(quantity));
            }
            catch (Exception e)
            {
                quantity = 10;
            }
            finally
            {
                // Return the connection to the pool.
                try
                {
                    DBBroker.getInstance().releaseConnection(dbCon);
                }
                catch (Exception e) {}
            }
        }
        return quantity;
    }

    /**
     * Helper method which takes a Vector of Integer ids and returns
     * numOfIdsToReturn ids in an array.
     *
     * @param availableIds A Vector.
     * @param numOfIdsToReturn The desired number of ids.
     * @return A BigDecimal[].
     */
    private BigDecimal[] getIds(Vector availableIds,
                                int numOfIdsToReturn)
    {
        BigDecimal[] results = new BigDecimal[numOfIdsToReturn];

        // We assume that availableIds will always come from the ids
        // Hashtable and would therefore always be the same object for
        // a specific table.
        synchronized (availableIds)
        {
            for (int i=0; i<numOfIdsToReturn; i++)
            {
                results[i] = (BigDecimal)availableIds.elementAt(i);
                availableIds.removeElementAt(i);
            }
        }
        return results;
    }

    /**
     * Helper method to select a row in the ID_TABLE and return a
     * int[].
     *
     * @param con A Connection.
     * @param tableName The name of the table to identify the row.
     * @return An int[].
     * @exception Exception, a generic exception.
     */
    private int[] selectRow(Connection con,
                            String tableName)
        throws Exception
    {
        StringBuffer stmt = new StringBuffer();
        stmt.append( "SELECT NEXT_ID, QUANTITY" )
            .append( " FROM ID_TABLE" )
            .append( " WHERE TABLE_NAME = '" )
            .append( tableName )
            .append( "'" );

        Statement statement = null;

        try
        {
            statement = con.createStatement();
            ResultSet rs = statement.executeQuery( stmt.toString() );

            int[] results = new int[2];
            rs.next();
            results[0] = rs.getInt(1); // next_id
            results[1] = rs.getInt(2); // quantity
            return results;
        }
        finally
        {
            if (statement != null) statement.close();
        }
    }

    /**
     * Helper method to update a row in the ID_TABLE.
     *
     * @param con A Connection.
     * @param tableName The name of the table to identify the row.
     * @param id An int with the value to set for the id.
     * @exception Exception, a generic exception.
     */
    private void updateRow(Connection con,
                           String tableName,
                           int id)
        throws Exception
    {
        StringBuffer stmt = new StringBuffer();
        stmt.append( "UPDATE ID_TABLE" )
            .append( " SET next_id = " )
            .append( id )
            .append( " WHERE TABLE_NAME = '" )
            .append( tableName )
            .append( "'" );

        Statement statement = null;

        try
        {
            statement = con.createStatement();
            statement.executeUpdate( stmt.toString() );
        }
        finally
        {
            if (statement != null) statement.close();
        }
    }

    /**
     * This is a helper method that generates unique ids from a
     * sequence.
     *
     * @param sequenceInfo An object containing info the DB needs to
     * generate the id.
     * @return An int with the value for the id.
     * @exception Exception, a generic exception.
     */
    public static int getNextSequenceIdAsInt(Object sequenceInfo)
        throws Exception
    {
        return getNextSequenceId(sequenceInfo, "default").intValue();
    }

    /**
     * This is a helper method that generates unique ids from a
     * sequence.
     *
     * @param sequenceInfo An object containing info the DB needs to
     * generate the id.
     * @return A long with the value for the id.
     * @exception Exception, a generic exception.
     */
    public static long getNextSequenceIdAsLong(Object sequenceInfo)
        throws Exception
    {
        return getNextSequenceId(sequenceInfo, "default").longValue();
    }

    /**
     * This is a helper method that generates unique ids from a
     * sequence.
     *
     * @param sequenceInfo An object containing info the DB needs to
     * generate the id.
     * @return A BigDecimal with the value for the id.
     * @exception Exception, a generic exception.
     */
    public static BigDecimal getNextSequenceId(Object sequenceInfo)
        throws Exception
    {
        return getNextSequenceId(sequenceInfo, "default");
    }

    /**
     * This is a helper method that generates unique ids from a
     * sequence.
     *
     * @param sequenceInfo An object containing info the DB needs to
     * generate the id.
     * @param dbName A String with the name of a database,
     * @return An int with the value for the id.
     * @exception Exception, a generic exception.
     */
    public static int getNextSequenceIdAsInt(Object sequenceInfo,
                                             String dbName)
        throws Exception
    {
        return getNextSequenceId(sequenceInfo, dbName).intValue();
    }

    /**
     * This is a helper method that generates unique ids from a
     * sequence.
     *
     * @param sequenceInfo An object containing info the DB needs to
     * generate the id.
     * @param dbName A String with the name of a database,
     * @return A long with the value for the id.
     * @exception Exception, a generic exception.
     */
    public static long getNextSequenceIdAsLong(Object sequenceInfo,
                                               String dbName)
        throws Exception
    {
        return getNextSequenceId(sequenceInfo, dbName).longValue();
    }

    /**
     * This is a helper method that generates unique ids from a
     * sequence.
     *
     * @param sequenceInfo An object containing info the DB needs to
     * generate the id.
     * @param dbName A String with the name of a database,
     * @return A BigDecimal with the value for the id.
     * @exception Exception, a generic exception.
     */
    public static BigDecimal getNextSequenceId(Object sequenceInfo,
                                               String dbName)
        throws Exception
    {
        // Get a connection to the db.
        DBConnection db = DBBroker.getInstance().getConnection(dbName);
        Connection connection = db.getConnection();

        // Create the statement.
        String stmt = DBBroker.getInstance().getDB(dbName)
            .getSequenceSql(sequenceInfo);

        // Execute the query.
        QueryDataSet qds = new QueryDataSet( connection, stmt );
        Record rec;
        try
        {
            qds.fetchRecords(1);
            rec = qds.getRecord(0);  // Records are 0 based.
        }
        finally
        {
            if (qds != null) qds.close();
            DBBroker.getInstance().releaseConnection(db);
        }
        return rec.getValue(1).asBigDecimal(); // Values are 1 based.
    }

    /**
     * Convenience method.
     *
     * @param db A DB.
     * @param dbCon A DBConnection.
     * @return An int with the value for the id.
     * @exception Exception, a generic exception.
     */
    public static int getAutoIncrementIdAsInt(DB db,
                                              DBConnection dbCon)
        throws Exception
    {
        return getAutoIncrementId(db, dbCon).intValue();
    }

    /**
     * Convenience method.
     *
     * @param db A DB.
     * @param dbCon A DBConnection.
     * @return A long with the value for the id.
     * @exception Exception, a generic exception.
     */
    public static long getAutoIncrementIdAsLong(DB db,
                                                DBConnection dbCon)
        throws Exception
    {
        return getAutoIncrementId(db, dbCon).longValue();
    }

    /**
     * TODO: Document
     *
     * @param db A DB.
     * @param dbCon A DBConnection.
     * @return A BigDecimal with the last value auto-incremented as a
     * result of an insert.
     * @exception Exception, a generic exception.
     */
    public static BigDecimal getAutoIncrementId(DB db,
                                                DBConnection dbCon)
        throws Exception
    {
        String IDSql = db.getIdSqlForAutoIncrement();
        BigDecimal id = null;
        QueryDataSet qds = null;
        try
        {
            qds = new QueryDataSet(dbCon.getConnection(), IDSql);
            qds.fetchRecords(1);
            Record rec = qds.getRecord(0);
            id = rec.getValue(1).asBigDecimal();
        }
        finally
        {
            if (qds != null) qds.close();
        }
        return id;
    }
}
