package org.apache.turbine.util.db;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java core classes
import java.util.*;
import java.lang.reflect.Array;

// Turbine classes
import org.apache.turbine.util.*;
import org.apache.turbine.util.db.pool.DB;

/**
 * This class represents a part of an SQL query found in the WHERE
 * section.  For example:
 *
 * table_a.column_a = table_b.column_a
 * column LIKE 'F%'
 * table.column < 3
 *
 * This class is used primarily by BasePeer.
 *
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @version $Id: SqlExpression.java,v 1.3 2000/09/05 18:50:37 gonzalo Exp $
 */
public class SqlExpression
{
    private static final String LIKE = " LIKE ";

    /**
     * Used to specify a join on two columns.
     *
     * @param column A column in one of the tables to be joined.
     * @param relatedColumn The column in the other table to be
     * joined.
     * @return A join expression, e.g. UPPER(table_a.column_a) =
     * UPPER(table_b.column_b).
     */
    public static String buildInnerJoin( String column,
                                         String relatedColumn )
    {
        // 'db' can be null because 'ignoreCase' is null.
        return buildInnerJoin( column, relatedColumn, false, null );
    }

    /**
     * Used to specify a join on two columns.
     *
     * @param column A column in one of the tables to be joined.
     * @param relatedColumn The column in the other table to be
     * joined.
     * @param ignoreCase If true and columns represent Strings, the
     * appropriate function defined for the database will be used to
     * ignore differences in case.
     * @param db Represents the database in use for vendor-specific
     * functions.
     * @return A join expression, e.g. UPPER(table_a.column_a) =
     * UPPER(table_b.column_b).
     */
    public static String buildInnerJoin( String column,
                                         String relatedColumn,
                                         boolean ignoreCase,
                                         DB db )
    {
            if (ignoreCase)
            {
                return new StringBuffer()
                   .append(db.ignoreCase(column))
                   .append("=")
                   .append(db.ignoreCase(relatedColumn))
                   .toString();
            }
            else
            {
                return new StringBuffer()
                    .append(column)
                    .append("=")
                    .append( relatedColumn )
                    .toString();
            }
    }

    /**
     * Builds a simple SQL expression.
     *
     * @param columnName A column.
     * @param criteria The value to compare the column against.
     * @param comparison One of =, &lt;, &gt;, ^lt;=, &gt;=, &lt;&gt;,
     * !=, LIKE, etc.
     * @return A simple SQL expression, e.g. UPPER(table_a.column_a)
     * LIKE UPPER('ab%c').
     * @exception Exception, a generic exception.
     */
    public static String build( String columnName,
                                Object criteria,
                                String comparison )
        throws Exception
    {
        // 'db' can be null because 'ignoreCase' is null
        return build( columnName, criteria, comparison, false, null );
    }

    /**
     * Builds a simple sql expression.
     *
     * @param columnName A column.
     * @param criteria The value to compare the column against.
     * @param comparison One of =, &lt;, &gt;, ^lt;=, &gt;=, &lt;&gt;,
     * !=, LIKE, etc.
     * @param ignoreCase If true and columns represent Strings, the
     * appropriate function defined for the database will be used to
     * ignore differences in case.
     * @param db Represents the database in use, for vendor specific
     * functions.
     * @return A simple sql expression, e.g. UPPER(table_a.column_a)
     * LIKE UPPER('ab%c').
     * @exception Exception, a generic exception.
     */
    public static String build( String columnName,
                                Object criteria,
                                String comparison,
                                boolean ignoreCase,
                                DB db )
        throws Exception
    {
        if (criteria == null)
            throw new Exception ("Criteria: null, Column Name: " +
                                 columnName +
                                 ", Comparison: " +
                                 comparison );

        if ( (criteria instanceof String) ||
             (criteria instanceof java.util.Date ))
        {
            criteria = quoteString(criteria.toString());
        }
        if ( comparison.equals(Criteria.LIKE) )
        {
            return buildLike( columnName, (String)criteria, ignoreCase, db );
        }
        else if ( comparison.equals(Criteria.IN) ||
                  comparison.equals(Criteria.NOT_IN) )
        {
            return buildIn( columnName, criteria, comparison, ignoreCase, db );
        }
        else
        {
            if (ignoreCase && db != null)
            {
                return new StringBuffer()
                   .append(db.ignoreCase(columnName))
                   .append(comparison)
                   .append(db.ignoreCase(criteria.toString()))
                   .toString();
            }
            else
            {
                return new StringBuffer()
                    .append(columnName)
                    .append(comparison)
                    .append(criteria.toString())
                    .toString();
            }
        }
    }

    /**
     * Takes a columnName and criteria and builds an SQL phrase based
     * on whether wildcards are present and the state of the
     * ignoreCase flag.  Multicharacter wildcards % and * may be used
     * as well as single character wildcards, _ and ?.  These
     * characters can be escaped with \.
     *
     * e.g. criteria = "fre%" -> columnName LIKE 'fre%'
     *                        -> UPPER(columnName) LIKE UPPER('fre%')
     *      criteria = "50\%" -> columnName = '50%'
     *
     * @param columnName A column.
     * @param criteria The value to compare the column against.
     * @param ignoreCase If true and columns represent Strings, the
     * appropriate function defined for the database will be used to
     * ignore differences in case.
     * @param db Represents the database in use, for vendor specific
     * functions.
     * @return An SQL expression.
     */
    static String buildLike( String columnName,
                             String criteria,
                             boolean ignoreCase,
                             DB db )
    {
        StringBuffer whereClause = new StringBuffer();
        buildLike( columnName, criteria, ignoreCase, db, whereClause );
        return whereClause.toString();
    }

    /**
     * Takes a columnName and criteria and builds an SQL phrase based
     * on whether wildcards are present and the state of the
     * ignoreCase flag.  Multicharacter wildcards % and * may be used
     * as well as single character wildcards, _ and ?.  These
     * characters can be escaped with \.
     *
     * e.g. criteria = "fre%" -> columnName LIKE 'fre%'
     *                        -> UPPER(columnName) LIKE UPPER('fre%')
     *      criteria = "50\%" -> columnName = '50%'
     *
     * @param columnName A column.
     * @param criteria The value to compare the column against.
     * @param ignoreCase If true and columns represent Strings, the
     * appropriate function defined for the database will be used to
     * ignore differences in case.
     * @param db Represents the database in use, for vendor specific
     * functions.
     * @param whereClause A StringBuffer to which the sql expression
     * will be appended.
     */
    static void buildLike( String columnName,
                           String criteria,
                           boolean ignoreCase,
                           DB db,
                           StringBuffer whereClause )
    {
        // If selection is case insensitive use SQL UPPER() function
        // on column name.
        if (ignoreCase)
        {
            whereClause.append(db.ignoreCase(columnName));
        }
        else
        {
            whereClause.append(columnName);
        }

        // If selection criteria contains wildcards use LIKE otherwise
        // use = (equals).  Wildcards can be escaped by prepending
        // them with \ (backslash).
        String equalsOrLike = " = ";
        String parsedCriteria = null;
        int position = 0;
        StringBuffer sb = new StringBuffer();
        while( position<criteria.length() )
        {
            char checkWildcard = criteria.charAt(position);
            if ( checkWildcard == '\\' &&
                 (criteria.charAt(position+1) == '%' ||
                  criteria.charAt(position+1) == '_' ||
                  criteria.charAt(position+1) == '*' ||
                  criteria.charAt(position+1) == '?' ||
                  criteria.charAt(position+1) == '\\' ))
            {
                position++;
            }
            else if ( checkWildcard == '%'  ||
                      checkWildcard == '_' )
            {
                equalsOrLike = LIKE;
            }
            else if ( criteria.charAt(position) == '*' )
            {
                equalsOrLike = LIKE;
                checkWildcard = '%';
            }
            else if ( criteria.charAt(position) == '?' )
            {
                equalsOrLike = LIKE;
                checkWildcard = '_';
            }
            sb.append( checkWildcard );
            position++;
        }
        whereClause.append(equalsOrLike);

        // If selection is case insensitive use SQL UPPER() function
        // on criteria.
        if (ignoreCase)
        {
            whereClause.append(db.ignoreCase(sb.toString()));
        }
        else
        {
            whereClause.append(sb.toString());
        }
        return;
    }

    /**
     * Takes a columnName and criteria (which must be an array) and
     * builds a SQL 'IN' expression taking into account the ignoreCase
     * flag.
     *
     * @param columnName A column.
     * @param criteria The value to compare the column against.
     * @param comparison Either " IN " or " NOT IN ".
     * @param ignoreCase If true and columns represent Strings, the
     * appropriate function defined for the database will be used to
     * ignore differences in case.
     * @param db Represents the database in use, for vendor specific
     * functions.
     * @return An SQL expression.
     */
    static String buildIn(String columnName,
                          Object criteria,
                          String comparison,
                          boolean ignoreCase,
                          DB db)
    {
        StringBuffer whereClause = new StringBuffer();
        buildIn(columnName, criteria, comparison,
                ignoreCase, db, whereClause);
        return whereClause.toString();
    }

    /**
     * Takes a columnName and criteria (which must be an array) and
     * builds a SQL 'IN' expression taking into account the ignoreCase
     * flag.
     *
     * @param columnName A column.
     * @param criteria The value to compare the column against.
     * @param comparison Either " IN " or " NOT IN ".
     * @param ignoreCase If true and columns represent Strings, the
     * appropriate function defined for the database will be used to
     * ignore differences in case.
     * @param db Represents the database in use, for vendor specific
     * functions.
     * @param whereClause A StringBuffer to which the sql expression
     * will be appended.
     */
    static void buildIn(String columnName,
                        Object criteria,
                        String comparison,
                        boolean ignoreCase,
                        DB db,
                        StringBuffer whereClause)
    {
        if (ignoreCase)
        {
            whereClause.append(db.ignoreCase(columnName));
        }
        else
        {
            whereClause.append(columnName);
        }

        whereClause.append(comparison);
        StringStackBuffer inClause = new StringStackBuffer();
        if (criteria instanceof Vector)
        {
            Vector v = (Vector)criteria;
            Enumeration e = v.elements();
            while (e.hasMoreElements())
            {
                Object value = e.nextElement();

                // The method processInValue() quotes the string
                // and/or wraps it in UPPER().
                inClause.add(processInValue(value, ignoreCase, db));
            }
        }
        else
        {
            // Assume array.
            for (int i = 0; i < Array.getLength(criteria); i++)
            {
                Object value = Array.get(criteria, i);

                // The method processInValue() quotes the string
                // and/or wraps it in UPPER().
                inClause.add(processInValue(value, ignoreCase, db));
            }
        }
        whereClause.append('(')
                   .append(inClause.toString(","))
                   .append(')');
    }

    /**
     * Creates an appropriate string for an 'IN' clause from an
     * object.  Adds quoting and/or UPPER() as appropriate.  This is
     * broken out into a seperate method as it is used in two places
     * in buildIn, depending on whether an array or Vector is being
     * looped over.
     *
     * @param value The value to process.
     * @param ignoreCase Coerce the value suitably for ignoring case.
     * @param db Represents the database in use for vendor specific
     * functions.
     * @return Processed value as String.
     */
    static String processInValue(Object value,
                                 boolean ignoreCase,
                                 DB db)
    {
        String ret = null;
        if (value instanceof String)
        {
            ret = quoteString((String)value);
        }
        else
        {
            ret = value.toString();
        }
        if (ignoreCase)
        {
            ret = db.ignoreCase(ret);
        }
        return ret;
    }

    /**
     * Quotes and escapes a String ready for placement in a SQL
     * expression.
     *
     * @param value The String to quote.
     * @return Quoted value as String.
     */
    static String quoteString(String value)
    {
        StringTokenizer st = new StringTokenizer(value, "'");
        boolean firstToken = true;
        while(st.hasMoreTokens())
        {
            if (firstToken)
            {
                value = st.nextToken();
                firstToken = false;
            }
            else
            {
                value = value + "''" + st.nextToken();
            }
        }
        return new StringBuffer().append("'")
                                 .append(value)
                                 .append("'")
                                 .toString();
    }
}
