package org.apache.turbine.util.db.map;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

/**
 * ColumnMap is used to model a column of a table in a database.
 *
 * @author <a href="mailto:john.mcnally@clearink.com">John D. McNally</a>
 * @version $Id: ColumnMap.java,v 1.2 2000/09/05 19:36:57 gonzalo Exp $
 */
public class ColumnMap
{
    /** Type of the column. */
    private Object type = null;

    /** Size of the column. */
    private int size = 0;

    /** Is it a primary key? */
    private boolean pk = false;

    /** Is null value allowed ?*/
    private boolean notNull = false;

    /** Name of the table that this column is related to. */
    private String relatedTableName = "";

    /** Name of the column that this column is related to. */
    private String relatedColumnName = "";

    /** The TableMap for this column. */
    private TableMap table;

    /** The name of the column. */
    private String columnName;


    /**
     * Constructor.
     *
     * @param name The name of the column.
     * @param containingTable TableMap of the table this column is in.
     */
    public ColumnMap( String name,
                      TableMap containingTable )
    {
        this.columnName = name;
        table = containingTable;
    }

    /**
     * Get the name of a column.
     *
     * @return A String with the column name.
     */
    public String getColumnName()
    {
        return columnName;
    }

    /**
     * Get the table name + column name.
     *
     * @return A String with the full column name.
     */
    public String getFullyQualifiedName()
    {
        return table.getName() + "." + columnName;
    }

    /**
     * Get the name of the table this column is in.
     *
     * @return A String with the table name.
     */
    public String getTableName()
    {
        return table.getName();
    }

    /**
     * Set the type of this column.
     *
     * @param type An Object specifying the type.
     */
    public void setType ( Object type )
    {
        this.type = type;
    }

    /**
     * Set the size of this column.
     *
     * @param size An int specifying the size.
     */
    public void setSize( int size )
    {
        this.size = size;
    }

    /**
     * Set if this column is a primary key or not.
     *
     * @param pk True if column is a primary key.
     */
    public void setPrimaryKey( boolean pk )
    {
        this.pk = pk;
    }

    /**
     * Set if this column may be null.
     *
     * @param nn True if column may be null.
     */
    public void setNotNull( boolean nn )
    {
        this.notNull = nn;
    }

    /**
     * Set the foreign key for this column.
     *
     * @param fullyQualifiedName The name of the table.column that is
     * foreign.
     */
    public void setForeignKey(String fullyQualifiedName )
    {
        if ( fullyQualifiedName != null &&
             fullyQualifiedName.length() > 0)
        {
            relatedTableName = fullyQualifiedName.substring( 0, fullyQualifiedName.indexOf('.') );
            relatedColumnName = fullyQualifiedName.substring( fullyQualifiedName.indexOf('.')+1 );
        }
        else
        {
            relatedTableName = "";
            relatedColumnName = "";
        }
    }

    /**
     * Set the foreign key for this column.
     *
     * @param tableName The name of the table that is foreign.
     * @param columnName The name of the column that is foreign.
     */
    public void setForeignKey(String tableName,
                              String columnName)
    {
        if ( tableName != null &&
             tableName.length() > 0 &&
             columnName != null &&
             columnName.length() > 0 )
        {
            relatedTableName = tableName;
            relatedColumnName = columnName;
        }
        else
        {
            relatedTableName = "";
            relatedColumnName = "";
        }
    }

    /**
     * Get the type of this column.
     *
     * @return An Object specifying the type.
     */
    public Object getType()
    {
        return type;
    }

    /**
     * Get the size of this column.
     *
     * @return An int specifying the size.
     */
    public int getSize()
    {
        return size;
    }

    /**
     * Is this column a primary key?
     *
     * @return True if column is a primary key.
     */
    public boolean isPrimaryKey()
    {
        return pk;
    }

    /**
     * Is null value allowed ?
     *
     * @return True if column may be null.
     */
    public boolean isNotNull()
    {
        return (notNull || isPrimaryKey());
    }

    /**
     * Is this column a foreign key?
     *
     * @return True if column is a foreign key.
     */
    public boolean isForeignKey()
    {
        if ( relatedTableName != null && relatedTableName.length() > 0)
            return true;
        else
            return false;
    }

    /**
     * Get the table.column that this column is related to.
     *
     * @return A String with the full name for the related column.
     */
    public String getRelatedName()
    {
        return relatedTableName+"."+relatedColumnName;
    }

    /**
     * Get the table name that this column is related to.
     *
     * @return A String with the name for the related table.
     */
    public String getRelatedTableName()
    {
        return relatedTableName;
    }

    /**
     * Get the column name that this column is related to.
     *
     * @return A String with the name for the related column.
     */
    public String getRelatedColumnName()
    {
        return relatedColumnName;
    }
}
