package org.apache.turbine.util.db.pool;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Java Core Classes
import java.sql.*;
import java.util.*;

// Turbine Utility Classes
import org.apache.turbine.util.*;
import org.apache.turbine.services.resources.TurbineResources;

/**
 * This class implements a simple connection pooling scheme.  Multiple
 * pools are available through use of the <code>DBBroker</code>.
 *
 * @author <a href="mailto:frank.kim@clearink.com">Frank Y. Kim</a>
 * @author <a href="mailto:bmclaugh@algx.net">Brett McLaughlin</a>
 * @author <a href="mailto:greg@shwoop.com">Greg Ritter</a>
 * @version $Id: ConnectionPool.java,v 1.2 2000/09/05 21:09:34 gonzalo Exp $
 */
public class ConnectionPool // implements Runnable
{
    /** Pool containing database connections. */
    private Stack pool = null;

    /**
     * The current number of database connections that have been
     * created.
     */
    private int totalConnections = 0;

    /**
     * The maximum number of database connections that can be
     * created.
     */
    private int maxConnections = 10;

    /**
     * The amount of time in milliseconds that a connection will be
     * pooled.
     */
    private long expiryTime = 3600000;   // 1 hour

    /**
     * The number of times to attempt to obtain a pooled connection
     * before giving up.
     */
    private long maxConnectionAttempts = 50;

    /**
     * The number of times that an attempt to obtain a pooled
     * connection has been made.
     */
    private long connectionAttemptsCounter = 0;

    /**
     * Thread sleep time between checks for database connectivity
     * problems.
     */
    private long dbCheckFrequency = 5000;

    /**
     * The class containing the database specific info for connections
     * in this pool.
     */
    private DB db = null;


    /**
     * Creates a <code>ConnectionPool</code> with the default
     * attributes.
     */
    protected ConnectionPool()
    {
        pool = new Stack();

        maxConnections = TurbineResources.getInt("database.maxConnections",
                                                 10);
        expiryTime = TurbineResources.getLong("database.expiryTime", 3600000);
        maxConnectionAttempts =
            TurbineResources.getLong("database.maxConnectionAttempts", 50);
    }

    /**
     * Creates a <code>ConnectionPool</code> with the specified
     * attributes.
     *
     * @param maxCons The maximum number of connectinos for this pool.
     * @param expiryTime The expiration time in milliseconds.
     */
    protected ConnectionPool(int maxCons, long expiryTime)
    {
        pool = new Stack();

        this.maxConnections = maxCons;
        this.expiryTime = expiryTime;
    }

    /**
     * Close any open connections when this object is garbage
     * collected.
     *
     * @exception Throwable Anything might happen...
     */
    protected void finalize()
        throws Throwable
    {
        while (!pool.empty())
        {
            DBConnection dbcon = (DBConnection)pool.pop();
            closeConnection(dbcon);
        }
    }

    /**
     * Returns a connection.
     *
     * @param driver The fully-qualified name of the JDBC driver to use.
     * @param url The URL of the database from which the connection is
     * desired.
     * @param username The name of the database user.
     * @param password The password of the database user.
     * @return A database connection.
     * @exception Exception.
     */
    protected synchronized DBConnection getConnection(String driver,
                                                      String url,
                                                      String username,
                                                      String password)
        throws Exception
    {
        DBConnection connection = null;

        if ( pool.empty() && totalConnections<maxConnections )
        {
            connection = getNewConnection(driver, url, username, password);
        }
        else
        {
            connection = getPooledConnection(driver, url, username, password);
        }
        return connection;
    }

    /**
     * Returns an instance of the database adapter associated with
     * this pool.
     *
     * @return The <code>DB</code> associated with this pool.
     */
    public DB getDB()
    {
        return db;
    }

    /**
     * Returns a fresh connection to the database.  The database type
     * is specified by <code>driver</code>, and its connection
     * information by <code>url</code>, <code>username</code>, and
     * <code>password</code>.
     *
     * @param driver The fully-qualified name of the JDBC driver to use.
     * @param url The URL of the database from which the connection is
     * desired.
     * @param username The name of the database user.
     * @param password The password of the database user.
     * @return A database connection.
     * @exception Exception
     */
    private DBConnection getNewConnection(String driver,
                                          String url,
                                          String username,
                                          String password)
        throws Exception
    {
        // DBBroker keeps a collection of ConnectionPools, each one of
        // which contains connections to a single database.  The
        // initialization of a pool should only occur once.
        if (db == null)
        {
            db = DBFactory.create( driver );
            db.init( url, username, password );
        }
        totalConnections++;
        return new DBConnection( db.getConnection(), url );
    }

    /**
     * Gets a pooled database connection.
     *
     * @param driver The fully-qualified name of the JDBC driver to use.
     * @param url The URL of the database from which the connection is
     * desired.
     * @param username The name of the database user.
     * @param password The password of the database user.
     * @return A database connection.
     * @exception Exception
     */
    private synchronized DBConnection getPooledConnection(String driver,
                                                          String url,
                                                          String username,
                                                          String password)
        throws Exception
    {
        while (true)
        {
            if (!pool.empty())
            {
                DBConnection con = (DBConnection) pool.pop();

                // It's really not safe to assume this connection is
                // valid even though it's checked before being pooled.
                if (isValid(con))
                {
                    connectionAttemptsCounter = 0;
                    return con;
                }
                else
                {
                    closeConnection(con);
                    totalConnections--;
                    connectionAttemptsCounter = 0;

                    // Need to create a new connection here, if the
                    // pool is empty.  Otherwise we can get stuck
                    // waiting for a connection to be returned and no
                    // one has any checked out.
                    if (pool.empty())
                    {
                        return getNewConnection(driver, url, username, password);
                    }

                    // Don't return to the wait state here - just go
                    // to the next iteration and try again.
                    continue;
                }
            }
            else
            {
                connectionAttemptsCounter++;
            }
            // wait(1);
            wait();
        }
    }

    /**
     * Helper method which determines if a connection has expired.
     *
     * @param connection The connection to test.
     * @return True if the connection is expired, false otherwise.
     * @exception Exception
     */
    private boolean isExpired( DBConnection connection )
        throws Exception
    {
        // Test the age of the connection (defined as current time
        // minus connection birthday) against the connection pool
        // expiration time.
        return ((System.currentTimeMillis() -
                 connection.getTimestamp()) > expiryTime);
    }

    /**
     * Determines if a connection is still valid.
     *
     * @param connection The connection to test.
     * @return True if the connection is valid, false otherwise.
     * @exception Exception
     */
    private boolean isValid( DBConnection connection )
        throws Exception
    {
        // Test whether the connection is valid and return the result.
        return ( connection.getConnection() != null &&
                 !connection.getConnection().isClosed() &&
                 !isExpired(connection) );
    }

    /**
     * Force the close of a database connection.
     *
     * @param dbcon Connection to close.
     */
    private void closeConnection ( DBConnection dbcon )
    {
        try
        {
            Connection conn = dbcon.getConnection();
            if ( conn != null && ! conn.isClosed() )
                conn.close();
        }
        catch (Exception e)
        {
            // Currently ignoring a connection close error.
        }
    }

    /**
     * This method returns a connection to the pool, and <b>must</b>
     * be called by the requestor when finished with the connection.
     *
     * @param connection The database connection to release.
     * @exception Exception Trouble releasing the connection.
     */
    protected synchronized void releaseConnection(DBConnection connection)
        throws Exception
    {
        if ( isValid(connection) )
        {
            pool.push(connection);
            notify();
        }
        else
        {
            closeConnection(connection);
            totalConnections--;
        }
    }

    /*

      This is being left commented for now because it requires more
      thought that I'm not ready to give to it yet. -JSS

    public void run()
    {
        while(true)
        {
            // Wait for a bit.
            try
            {
                Thread.sleep(dbCheckFrequency);
            }
            catch(InterruptedException exc)
            {
            }

            // Check for database connectivity problems.
            if ( connectionAttemptsCounter >= maxConnectionAttempts )
            {
                // Dump all connections.
                try
                {
                    finalize();
                }
                catch ( Throwable e )
                {
                }

                connectionAttemptsCounter = 0;
                notify();
            }
        }
    }

    */
}
