package org.apache.turbine.util.webmacro;

/*
 * Copyright (c) 1997-2000 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * 4. The names "Apache JServ", "Apache JServ Servlet Engine", "Turbine",
 *    "Apache Turbine", "Turbine Project", "Apache Turbine Project" and
 *    "Java Apache Project" must not be used to endorse or promote products
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JServ"
 *    nor may "Apache" nor "Apache JServ" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project for use in the Apache JServ servlet engine project
 *    <http://java.apache.org/>."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Group. For more information
 * on the Java Apache Project and the Apache JServ Servlet Engine project,
 * please see <http://java.apache.org/>.
 *
 */

// Turbine Classes
import org.apache.turbine.modules.*;
import org.apache.turbine.services.resources.TurbineResources;
import org.apache.turbine.util.*;

/**
 * Utility class to help check for proper authorization when using
 * WebMacroSiteScreens.  Sample usages:
 *
 * <p><code>
 * WebMacroSecurityCheck secCheck = new WebMacroSecurityCheck( data );
 * secCheck.setMessage( "Sorry, you do not have permission to " +
 *                      "access this area." );
 * secCheck.setFailTemplate("login.wm");
 * if ( !secCheck.hasRole("ADMIN") )
 *     return;
 * </code>
 *
 * @author <a href="mbryson@mont.mindspring.com">Dave Bryson</a>
 * @version $Id: WebMacroSecurityCheck.java,v 1.2 2000/09/06 15:46:14 gonzalo Exp $
 */
public class WebMacroSecurityCheck
{
    private String message =
        "Sorry, you do not have permission to access this area.";
    private String failScreen = "WebMacroSiteScreen";
    private String failTemplate;
    private RunData data = null;

    /**
     * Constructor.
     *
     * @param data A Turbine RunData object.
     * @param message A String with the message to display upon
     * failure.
     */
    public WebMacroSecurityCheck ( RunData data,
                                   String message )
    {
        this.data=data;
        this.message=message;
    }

    /**
     * Generic Constructor.
     *
     * @param data A Turbine RunData object.
     */
    public WebMacroSecurityCheck( RunData data )
    {
        this.data=data;
    }

    /**
     * Does the User have this role?
     *
     * @param rolename A String with the role to be checked.
     * @return True if user has the role.
     * @exception Exception, a generic exception.
     */
    public boolean hasRole(String rolename)
        throws Exception
    {
        if ( !checkLogin() )
        {
            return false;
        }

        if ( data.getACL() == null ||
             !data.getACL().hasRole(rolename) )
        {
            data.setScreen( getFailScreen() );
            data.getTemplateInfo().setScreenTemplate( getFailTemplate() );
            data.setMessage(getMessage());
            return false;
        }
        else
        {
            return true;
        }
    }

    /**
     * Does the User have this permission?
     *
     * @param permission A String with the permission to be checked.
     * @return True if user has the permission.
     * @exception Exception, a generic exception.
     */
    public boolean hasPermission(String permission)
        throws Exception
    {
        if ( data.getACL() == null ||
             !data.getACL().hasPermission(permission) )
        {
            data.setScreen( getFailScreen() );
            data.getTemplateInfo().setScreenTemplate( getFailTemplate() );
            data.setMessage(getMessage());
            return false;
        }
        else
        {
            return true;
        }
    }

    /**
     * Check that the user has logged in.
     *
     * @return True if user has logged in.
     * @exception Exception, a generic exception.
     */
    public boolean checkLogin()
        throws Exception
    {
        boolean value = false;

        if ( data.getUser() != null &&
             !data.getUser().hasLoggedIn() )
        {
            data.setMessage( TurbineResources.getString("login.message") );
            data.getTemplateInfo().setScreenTemplate( getFailTemplate() );
            value = false;
        }
        else
        {
            value = true;
        }
        return value;
    }

    /**
     * Set the message that should be displayed.  This is initialized
     * in the constructor.
     *
     * @param v A String with the message that should be displayed.
     */
    public void setMessage( String v )
    {
        this.message=v;
    }

    /**
     * Get the message that should be displayed.  This is initialized
     * in the constructor.
     *
     * @return A String with the message that should be displayed.
     */
    public String getMessage()
    {
        return message;
    }

    /**
     * Get the value of failScreen.
     *
     * @return A String with the value of failScreen.
     */
    public String getFailScreen()
    {
        return failScreen;
    }

    /**
     * Set the value of failScreen.
     *
     * @param v A String with the value of failScreen.
     */
    public void setFailScreen(String v)
    {
        this.failScreen = v;
    }

    /**
     * Get the value of failTemplate.
     *
     * @return A String with the value of failTemplate.
     */
    public String getFailTemplate()
    {
        return failTemplate;
    }

    /**
     * Set the value of failTemplate.
     *
     * @param v A String with the value of failTemplate.
     */
    public void setFailTemplate(String v)
    {
        this.failTemplate = v;
    }
}
